!-------- Specification of current simulation --------

#define RUNNAME 'emtp2sge_expA'
!                      Name of simulation

#define SICO_VERSION '2.9'
!                      Version number of SICOPOLIS

#define INPATH '/uchi/greve/sico_exchange/sicopolis/sico_in'
!                      Directory for SICOPOLIS input
!                      (no '/' at the end!)

#define OUTPATH '/uchi/greve/sico_exchange/sicopolis/sico_out'
!                      Directory for SICOPOLIS output
!                      (no '/' at the end!)

!  ------------------

#define EMTP2SGE
!                      Simulated domain:
!                        ANT      - Antarctica
!                        GRL      - Greenland
!                        SCAND    - Scandinavia
!                        NHEM     - Northern hemisphere
!                        TIBET    - Tibet
!                        EMTP2SGE - EISMINT Phase 2 Simplified Geometry Experiment
!                        HEINO    - ISMIP HEINO
!                        NMARS    - North polar cap of Mars
!                        SMARS    - South polar cap of Mars

!  ------------------

#define GRID 0
!                         0 : Cartesian coordinates in the
!                             stereographic plane
!                         1 : Stereographically expanded Cartesian
!                             coordinates on the surface of the earth
!                         2 : Latitude/longitude grid on the surface
!                             of the earth (not allowed for this application)

#define X0 0.0d0
!                       x coordinate (in km) of the origin point (i,j) = (0,0),
!                       for GRID==0 or GRID==1

#define Y0 0.0d0
!                       y coordinate (in km) of the origin point (i,j) = (0,0),
!                       for GRID==0 or GRID==1

#define DX 25.0d0
!                       Horizontal grid spacing in km, for GRID==0
!                       or GRID==1
!                       (25 km requires IMAX=60 and JMAX=60,
!                        75 km requires IMAX=20 and JMAX=20)

#define IMAX 60
!                       IMAX+1: number of grid points in x-direction
!                               (i=0...IMAX)

#define JMAX 60
!                       JMAX+1: number of grid points in y-direction
!                               (j=0...JMAX)

#define KCMAX 80
!                       KCMAX+1: number of grid points in z-direction
!                                in cold ice (kc=0...KCMAX)

#define KTMAX 10
!                       KTMAX+1: number of grid points in z-direction
!                                in temperate ice (kt=0...KTMAX)

#define KRMAX 10
!                       KRMAX+1: number of grid points in z-direction
!                                in the bedrock (kr=0...KRMAX)

#define DEFORM 2.0d0
!                       Exponential stretch parameter of the non-
!                       equidistant grid in z-direction in cold ice

!  ------------------

#define TIME_INIT0 0.0d0
!                             Initial time of simulation (in a)

#define TIME_END0 200000.0d0
!                             Final time of simulation (in a)

#define DTIME0 200.0d0
!                       Time step (in a) for computation of velocity
!                       and topography

#define DTIME_TEMP0 200.0d0
!                       Time step (in a) for computation of
!                       temperature, water content and age of the ice

#define DTIME_SER0 100.0d0
!                             Time step (in a) for writing of data to
!                             the time-series files '.ser' and '.thk'

#define YEAR_SEC 31556926.0d0
!                       Conversion from years to seconds

!  ------------------

#define CALCMOD 1
!                         0 : Simulation in cold-ice mode with
!                             resetting of temperatures above melting
!                        *1*: Simulation in polythermal mode

!  ------------------

#define MARGIN 1
!                         1 : Ice extent restricted to land area
!                         2 : Ice can spread into the sea, constant value
!                             for the minimum bedrock elevation
!                         3 : Ice can spread into the sea, minimum bedrock
!                             elevation proportional to sea-level stand

#define Z_MAR 0.0d0
!                             Minimum bedrock elevation (in m) allowed
!                             to glaciate (only for MARGIN==2)

#define FACT_Z_MAR 0.0d0
!                             Minimum bedrock elevation allowed to glaciate:
!                             proportionality factor to sea-level stand
!                             (only for MARGIN==3)

!  ------------------

#define FLOW_LAW 1
!                         1 : Glen's flow law with stress exponent n=3
!                         2 : Goldsby-Kohlstedt flow law with stress exponent
!                             n=1.8 and grain-size exponent p=1.4
!                         3 : Durham's flow law with stress exponent n=4

#define FIN_VISC 1
!                         1 : Unmodified flow law with infinite viscosity
!                             for low strain rates
!                         2 : Modified flow law with additional
!                             finite-viscosity parameter SIGMA_RES
!                             (defined in physical-parameter file)

#define GR_SIZE 1.0d-03
!                         Average grain size (in m; only for FLOW_LAW==2)

!  ------------------

#define ENHMOD 1
!                         1 : Flow enhancement factor enh=ENH
!                             everywhere
!                         2 : enh=1 for ice younger than 11 ka
!                             (Holocene ice), enh=ENH for ice older
!                             than 11 ka (Pleistocene ice); for present
!                             steady-state simulations
!                         3 : enh=1 for Holocene and Eemian ice,
!                             enh=ENH for Weichselian and pre-Eemian
!                             ice; for palaeoclimatic and
!                             greenhouse-warming scenarios

#define ENH 1.0d0
!                         Flow enhancement factor for glacial (for instance,
!                         Weichselian) ice

!  ------------------

#define DNS 1
!                         1 : Computation of vx and vy in the
!                             shallow-ice approximation
!                             (no deviatoric normal stresses)
!                         2 : Algorithm of Heinz Blatter for the
!                             computation of vx und vy
!                             (with deviatoric normal stresses);
!                             requires CALCMOD=0 und CALCZS=0
!                             (***under construction***)

!  ------------------

#define ANF_DAT 2
!                         1 : Present initial topography
!                         2 : Ice-free initial topography with
!                             relaxed lithosphere
!                         3 : Initial values from previous
!                             simulation

#define ANFDATNAME 'none'
!                             Initial-value file (only for ANF_DAT==3)

#define ANFDATPATH 'none'
!                             Directory for initial-value file
!                             (no '/' at the end!)

!  ------------------

#define REBOUND 0
!                         0 : No bedrock adjustment
!                        *1*: Isostatic bedrock adjustment with
!                             time lag (LLRA model)

#define FRAC_LLRA 1.0d0
!                             Fraction of isostatic compensation in the LLRA
!                             model (REBOUND==1). Range 0 <= FRAC_LLRA <= 1;
!                             0: no bedrock adjustment, 1: full adjustment.

#define Q_LITHO 0
!                         0 : No coupled heat-conducting bedrock
!                        *1*: Coupled heat-conducting bedrock

!  ------------------

#define ZS_EVOL 1
!                         0 : No evolution of the free surface, kept fixed on
!                             the initial topography
!                        *1*: Evolution of the free surface

#define CALCZS 3
!                             Solution of the ice-thickness equation:
!                         0 : Explicit scheme
!                         1 : ADI scheme
!                         2 : Over-implicit ADI scheme
!                         3 : Over-implicit scheme with iterative SOR solution
!                             of the resulting systems of linear equations 

#define OVI_WEIGHT 1.5d0
!                       Weighing parameter for the over-implicit
!                       schemes (only for CALCZS==2, 3)

#define OMEGA_SOR 1.0d0
!                       Relaxation parameter for the iterative SOR solver
!                       for systems of linear equations
!                       (0 < OMEGA_SOR < 2, only for CALCZS==3)

!  ------------------

#define ADV_HOR 2
!                         Discretization of horizontal advection terms in the
!                         3-d temperature and age equations:
!                         1 : Not defined
!                             (central differences would be unstable!)
!                         2 : Non-conservative first-order upstream
!                         3 : Non-conservative first-order upstream for
!                             temperature equation, conservative first-order
!                             upstream for age equation (by B. Muegge;
!                             requires ADV_VERT==3)
!                         4 : Conservative first-order upstream
!                             (requires also a conservative scheme for
!                              vertical advection which has not yet been
!                              implemented -> ***under construction***)

#define ADV_VERT 2
!                         Discretization of vertical advection terms in the
!                         3-d temperature and age equations:
!                         1 : Central differences (plus artificial diffusion
!                             for the age equation)
!                         2 : Non-conservative first-order upstream
!                         3 : Non-conservative first-order upstream for
!                             temperature equation, conservative first-order
!                             upstream for age equation (by B. Muegge;
!                             requires ADV_HOR==3)

#define AGEDIFF 5.0d-08
!                       Numerical age diffusivity
!                       (in m2/s, only for ADV_VERT==1)

#define M_AGE 200.0d0
!                       Parameter for the prescribed numerical age
!                       gradient at the ice base (only for ADV_VERT==1)

!  ------------------

#define TOPOGRAD 1
!                             Topography gradients at grid points
!                             with
!                         0 : second-order discretisation
!                        *1*: fourth-order discretisation

!  ------------------

#define TEMP_MIN -3.5d+01   /* in deg C    */
#define S_T       1.67d-02  /* in K/km     */
#define X_HAT     7.5d+02   /* in km       */
#define Y_HAT     7.5d+02   /* in km       */
#define B_MAX     5.0d-01   /* in m/a      */
#define S_B       1.0d-02   /* in m/(a*km) */
#define ELD       4.5d+02   /* in km       */
!       Parameters for the surface temperature and surface mass balance

!  ------------------

#define TSURFACE 1
!                         1 : delta_ts = DELTA_TS0, steady state
!                         3 : Sinusoidal air-temperature forcing
!                             between delta_ts = 0 C and delta_ts =
!                             -2*SINE_AMPLIT C with period
!                             SINE_PERIOD (in a)
!                         4 : delta_ts from ice-core data
!                             (e.g., GRIP, Vostok)
!                         5 : Arbitrary scenario to be added in
!                             subroutine 'boundary'

#define DELTA_TS0 0.0d0
!                       Constant air-temperature deviation for steady
!                       states (only for TSURFACE==1)

#define SINE_AMPLIT 10.0d0
!                       Amplitude (in C) for sinusoidal air-temperature
!                       forcing (only for TSURFACE==3)

#define SINE_PERIOD 100000.0d0
!                       Period (in a) for sinusoidal air-temperature
!                       forcing (only for TSURFACE==3)

#define GRIP_TEMP_FILE 'grip95_temp_nosmooth.dat'
!                       Name of the file containing the ice-core
!                       air-temperature forcing (only for TSURFACE==4)

#define GRIP_TEMP_FACT 1.0d0
!                       Modification factor for ice-core air-temperature
!                       forcing (only for TSURFACE==4)

!  ------------------

#define SEA_LEVEL 1
!                         1 : Constant sea level z_sl = Z_SL0
!                         2 : Saw-tooth-shaped sea-level forcing
!                             with glacial minimum z_sl = -130;
!                             for palaeoclimatic scenarios
!                         3 : Sea-level reconstruction from data
!                             (e.g., SPECMAP); for palaeoclimatic scenarios
!                         4 : Sea-level forcing by previously computed
!                             sea-level-equivalent scenario

#define Z_SL0 0.0d0
!                       Constant sea level
!                       (in m, only for SEA_LEVEL==1)

#define SEA_LEVEL_FILE 'specmap_zsl_782kyr.dat'
!                       Name of the file containing the sea-level
!                       forcing (only for SEA_LEVEL==3)

#define Z_SLE_FILE 'none'
!                       Name of the previously computed time-series
!                       file containing the sea-level-equivalent
!                       scenario (only for SEA_LEVEL==4)

#define Z_SLE_PATH 'none'
!                       Directory for file Z_SLE_FILE
!                       (no '/' at the end; only for SEA_LEVEL==4)

#define Z_SLE_FACT 1.0d0
!                       Factor for conversion from computed
!                       sea-level-equivalent to sea-level forcing
!                       (only for SEA_LEVEL==4)

!  ------------------

#define SLIDE_LAW 1
!                       1 : Weertman-type sliding,
!                           full ice pressure in denominator
!                       2 : Weertman-type sliding,
!                           reduced pressure (ice minus water) in denominator

#define C_SLIDE 0.0d0
!                       Sliding coefficient, in m/[a*Pa^(p-q)]

#define GAMMA_SLIDE 0.0d0
!                       Sub-melt sliding coefficient, in K

#define P_WEERT 1
!                       Weertman exponent p for the basal shear stress
!                       (integer value)

#define Q_WEERT 0
!                       Weertman exponent q for the basal pressure
!                       (integer value)

!  ------------------

#define Q_GEO_MOD 1
!                         1 : Constant geothermal heat flux defined
!                             by parameter Q_GEO

#define Q_GEO 42.0d0
!                       Geothermal heat flux (in mW/m2)
!                       (only for Q_GEO_MOD==1)

!  ------------------

#define OUTPUT 1
!                         1 : Writing of time-slice data in files
!                             '.erg' with prescribed time step
!                         2 : Writing of time-slice data in files
!                             '.erg' at arbitrarily specified times

#define DTIME_OUT0 200000.0d0
!                             Time step (in a) for writing of
!                             time-slice data (only for OUTPUT==1)

#define N_OUTPUT 0
!                             Number of specified times for writing of
!                             time-slice data (only for OUTPUT==2,
!                             not more than 20)

#define TIME_OUT0_01 1.11d+11
#define TIME_OUT0_02 1.11d+11
#define TIME_OUT0_03 1.11d+11
#define TIME_OUT0_04 1.11d+11
#define TIME_OUT0_05 1.11d+11
#define TIME_OUT0_06 1.11d+11
#define TIME_OUT0_07 1.11d+11
#define TIME_OUT0_08 1.11d+11
#define TIME_OUT0_09 1.11d+11
#define TIME_OUT0_10 1.11d+11
#define TIME_OUT0_11 1.11d+11
#define TIME_OUT0_12 1.11d+11
#define TIME_OUT0_13 1.11d+11
#define TIME_OUT0_14 1.11d+11
#define TIME_OUT0_15 1.11d+11
#define TIME_OUT0_16 1.11d+11
#define TIME_OUT0_17 1.11d+11
#define TIME_OUT0_18 1.11d+11
#define TIME_OUT0_19 1.11d+11
#define TIME_OUT0_20 1.11d+11
!                             Times (in a) for writing of time-slice
!                             data (only for OUTPUT==2, in increasing
!                             order from #1 to #N_OUTPUT)

!  ------------------

#define OUTSER 3
!                        *1*: Only writing of usual time-series file
!                             '.ser'
!                         2 : Writing of usual time-series file '.ser'
!                             and of time-series file '.thk' for the
!                             complete ice-thickness field
!                         3 : Writing of usual time-series file '.ser'
!                             and of time-series file '.core' for the
!                             selected positions ("deep boreholes")

#define ERGDAT 1
!                         0 : Only 2-d fields written as time-slice
!                             data
!                        *1*: Full set of 2-d and 3-d fields written
!                             as time-slice data

#define NETCDF 1
!                         1 : Time-slice files unformatted
!                         2 : Time-slice files in format netCDF
!                         3 : Time-slice files both unformatted
!                             and in format netCDF

!  ------------------

#define NUMDIFF_ZS 0.0d0
!                       Spatial smoothing parameter for computation
!                       of zs (***use only in case of otherwise
!                       unrecoverable instability***)

#define NUMDIFF_H_T 0.15d0
!                       Spatial smoothing parameter for computation
!                       of H_t

#define TAU_CTS 0.0d0
!                       Numerical time lag (in a) for evolution
!                       of H_t

#define H_MIN 1.0d-03
!                       Minimum ice thickness interpreted as glaciation
!                       (in m)

#define VH_MAX 1.0d+04
!                       Lower (-VH_MAX) and upper (+VH_MAX) limits of
!                       horizontal velocities vx_c/t, vy_c/t (in m/a)

#define HD_MIN 1.0d-15
#define HD_MAX 500.0d0
!                       Lower and upper limits of free-surface
!                       diffusivity hdiff (in m^2/s)

#define QB_MIN 1.0d-15
#define QB_MAX 1.0d-07
!                       Lower and upper limits of the basal melting and
!                       drainage rates Q_bm, Q_tld and Q_b_tot (in m/s)

#define AGE_MIN 1.0d-15
#define AGE_MAX 1.0d+06
!                       Lower and upper limits of computed ages (in a)

#define MEAN_ACCUM 5.0d+02
!                       Mean accumulation rate over modelled ice sheet
!                       (in mm water equiv./a)
!                       [Only required for the numerical age gradient
!                       at the ice base, and, in case of CALCZS==3, for
!                       the convergence criterion of the SOR method.
!                       Need not be very precise, a rough estimate is
!                       sufficient.]

!  ------------------

#define NETCDF_INC '/usr/local/include/netcdf.inc'
!                       netCDF include file
