!-------- Specification of current simulation --------

#define RUNNAME 'nhem80_nt012_new'
!                      Name of simulation

#define SICO_VERSION '2.9'
!                      Version number of SICOPOLIS

#define INPATH '/uchi/greve/sico_exchange/sicopolis/sico_in'
!                      Directory for SICOPOLIS input
!                      (no '/' at the end!)

#define OUTPATH '/uchi/greve/sico_exchange/sicopolis/sico_out'
!                      Directory for SICOPOLIS output
!                      (no '/' at the end!)

!  ------------------

#define NHEM
!                      Simulated domain:
!                        ANT      - Antarctica
!                        GRL      - Greenland
!                        SCAND    - Scandinavia
!                        NHEM     - Northern hemisphere
!                        TIBET    - Tibet
!                        EMTP2SGE - EISMINT Phase 2 Simplified Geometry Experiment
!                        HEINO    - ISMIP HEINO
!                        NMARS    - North polar cap of Mars
!                        SMARS    - South polar cap of Mars

!  ------------------

#define GRID 1
!                         0 : Cartesian coordinates in the
!                             stereographic plane
!                        *1*: Stereographically expanded Cartesian
!                             coordinates on the surface of the earth
!                         2 : Latitude/longitude grid on the surface
!                             of the earth (not allowed for this application)

#define X0 -6240.0d0
!                       x coordinate (in km) of the origin point (i,j) = (0,0),
!                       for GRID==0 or GRID==1

#define Y0 -6240.0d0
!                       y coordinate (in km) of the origin point (i,j) = (0,0),
!                       for GRID==0 or GRID==1

#define DX 80.0d0
!                       Horizontal grid spacing in km, for GRID==0
!                       or GRID==1
!                       (80 km requires IMAX=156 and JMAX=156,
!                        40 km requires IMAX=312 and JMAX=312,
!                        20 km requires IMAX=624 and JMAX=624)

#define IMAX 156
!                       IMAX+1: number of grid points in x-direction
!                               (i=0...IMAX)

#define JMAX 156
!                       JMAX+1: number of grid points in y-direction
!                               (j=0...JMAX)

#define KCMAX 80
!                       KCMAX+1: number of grid points in z-direction
!                                in cold ice (kc=0...KCMAX)

#define KTMAX 10
!                       KTMAX+1: number of grid points in z-direction
!                                in temperate ice (kt=0...KTMAX)

#define KRMAX 10
!                       KRMAX+1: number of grid points in z-direction
!                                in the bedrock (kr=0...KRMAX)

#define DEFORM 2.0d0
!                       Exponential stretch parameter of the non-
!                       equidistant grid in z-direction in cold ice

!  ------------------

#define TIME_INIT0 -250000.0d0
!                             Initial time of simulation (in a)

#define TIME_END0 0.0d0
!                             Final time of simulation (in a)

#define DTIME0 5.0d0
!                       Time step (in a) for computation of velocity
!                       and topography

#define DTIME_TEMP0 5.0d0
!                       Time step (in a) for computation of
!                       temperature, water content and age of the ice

#define DTIME_WSS0 100.0d0
!                       Time step (in a) for computation of
!                       isostatic steady-state displacement of the lithosphere
!                       (only for REBOUND==2, ELRA model)

#define DTIME_SER0 100.0d0
!                             Time step (in a) for writing of data to
!                             the time-series files '.ser' and '.thk'

#define YEAR_SEC 31556926.0d0
!                       Conversion from years to seconds

!  ------------------

#define CALCMOD 1
!                         0 : Simulation in cold-ice mode with
!                             resetting of temperatures above melting
!                        *1*: Simulation in polythermal mode

!  ------------------

#define MARGIN 2
!                         1 : Ice extent restricted to land area
!                         2 : Ice can spread into the sea, constant value
!                             for the minimum bedrock elevation
!                         3 : Ice can spread into the sea, minimum bedrock
!                             elevation proportional to sea-level stand
!                         4 : Ice can spread into the sea, minimum bedrock
!                             elevation related to sea-level stand by the
!                             piecewise linear relation proposed by Zweck
!                             and Huybrechts (2005)

#define Z_MAR -500.0d0
!                         Minimum bedrock elevation (in m) allowed
!                         to glaciate (only for MARGIN==2)

#define FACT_Z_MAR 2.5d0  /* suitable value for MARGIN==3 */
!                         Minimum bedrock elevation allowed to glaciate:
!                         proportionality factor to sea-level stand
!                         (for MARGIN==3),
!                         modification factor for the piecewise linear
!                         relation by Zweck and Huybrechts (2005)
!                         (for MARGIN==4)

!  ------------------

#define FLOW_LAW 1
!                         1 : Glen's flow law with stress exponent n=3
!                         2 : Goldsby-Kohlstedt flow law with stress exponent
!                             n=1.8 and grain-size exponent p=1.4
!                         3 : Durham's flow law with stress exponent n=4

#define FIN_VISC 1
!                         1 : Unmodified flow law with infinite viscosity
!                             for low strain rates
!                         2 : Modified flow law with additional
!                             finite-viscosity parameter SIGMA_RES
!                             (defined in physical-parameter file)

#define GR_SIZE 1.0d-03
!                         Average grain size (in m; only for FLOW_LAW==2)

!  ------------------

#define ENHMOD 1
!                         1 : Flow enhancement factor enh=ENH
!                             everywhere
!                         2 : enh=1 for ice younger than 11 ka
!                             (Holocene ice), enh=ENH for ice older
!                             than 11 ka (Pleistocene ice); for present
!                             steady-state simulations
!                         3 : enh=1 for Holocene and Eemian ice,
!                             enh=ENH for Weichselian and pre-Eemian
!                             ice; for palaeoclimatic and
!                             greenhouse-warming scenarios

#define ENH 3.0d0
!                         Flow enhancement factor for glacial (for instance,
!                         Weichselian) ice

!  ------------------

#define DNS 1
!                         1 : Computation of vx and vy in the
!                             shallow-ice approximation
!                             (no deviatoric normal stresses)
!                         2 : Algorithm of Heinz Blatter for the
!                             computation of vx und vy
!                             (with deviatoric normal stresses);
!                             requires CALCMOD=0 und CALCZS=0
!                             (***under construction***)

!  ------------------

#define ANF_DAT 2
!                         1 : Present initial topography
!                         2 : Ice-free initial topography with
!                             relaxed lithosphere
!                         3 : Initial values from previous
!                             simulation

#define ZS_PRESENT_FILE   'nhem_e5_80_zs.dat'
!                             Name of the file containing the present-day
!                             surface topography

#define ZB_PRESENT_FILE   'nhem_e5_80_zb.dat'
!                             Name of the file containing the present-day
!                             bedrock topography (only for ANF_DAT==1)

#define ZB0_FILE          'nhem_e5_80_zb0.dat'
!                             Name of the file containing the topography
!                             of the relaxed bedrock

#define MASK_PRESENT_FILE 'nhem_e5_80_mask.dat'
!                             Name of the file containing the present-day
!                             ice-land-ocean mask

#define ANFDATNAME 'none'
!                             Initial-value file (only for ANF_DAT==3)

#define ANFDATPATH 'none'
!                             Directory for initial-value file
!                             (no '/' at the end!)

!  ------------------

#define REBOUND 1
!                         0 : No bedrock adjustment
!                         1 : Isostatic bedrock adjustment with local
!                             lithosphere and relaxing asthenosphere (LLRA model)
!                         2 : Isostatic bedrock adjustment with elastic
!                             lithosphere and relaxing asthenosphere (ELRA model)

#define FRAC_LLRA 1.0d0
!                             Fraction of isostatic compensation in the LLRA
!                             model (REBOUND==1). Range 0 <= FRAC_LLRA <= 1;
!                             0: no bedrock adjustment, 1: full adjustment.

#define Q_LITHO 1
!                         0 : No coupled heat-conducting bedrock
!                        *1*: Coupled heat-conducting bedrock

!  ------------------

#define ZS_EVOL 1
!                         0 : No evolution of the free surface, kept fixed on
!                             the initial topography
!                        *1*: Evolution of the free surface

#define CALCZS 3
!                         Solution of the ice-thickness equation:
!                         0 : Explicit scheme
!                         1 : ADI scheme
!                         2 : Over-implicit ADI scheme
!                         3 : Over-implicit scheme with iterative SOR solution
!                             of the resulting systems of linear equations 

#define OVI_WEIGHT 1.5d0
!                       Weighing parameter for the over-implicit
!                       schemes (only for CALCZS==2, 3)

#define OMEGA_SOR 1.0d0
!                       Relaxation parameter for the iterative SOR solver
!                       for systems of linear equations
!                       (0 < OMEGA_SOR < 2, only for CALCZS==3)

!  ------------------

#define ADV_HOR 2
!                         Discretization of horizontal advection terms in the
!                         3-d temperature and age equations:
!                         1 : Not defined
!                             (central differences would be unstable!)
!                         2 : Non-conservative first-order upstream
!                         3 : Non-conservative first-order upstream for
!                             temperature equation, conservative first-order
!                             upstream for age equation (by B. Muegge;
!                             requires ADV_VERT==3)
!                         4 : Conservative first-order upstream
!                             (requires also a conservative scheme for
!                              vertical advection which has not yet been
!                              implemented -> ***under construction***)

#define ADV_VERT 2
!                         Discretization of vertical advection terms in the
!                         3-d temperature and age equations:
!                         1 : Central differences (plus artificial diffusion
!                             for the age equation)
!                         2 : Non-conservative first-order upstream
!                         3 : Non-conservative first-order upstream for
!                             temperature equation, conservative first-order
!                             upstream for age equation (by B. Muegge;
!                             requires ADV_HOR==3)

#define AGEDIFF 5.0d-08
!                       Numerical age diffusivity
!                       (in m2/s, only for ADV_VERT==1)

#define M_AGE 200.0d0
!                       Parameter for the prescribed numerical age
!                       gradient at the ice base (only for ADV_VERT==1)

!  ------------------

#define TOPOGRAD 1
!                             Topography gradients at grid points
!                             with
!                         0 : second-order discretisation
!                        *1*: fourth-order discretisation

!  ------------------

#define TEMP_MM_PRESENT_FILE 'nhem80_tempmm.dat'
!                       Name of the file containing the present-day
!                       monthly-mean surface-temperature data

#define TSURFACE 4
!                         1 : delta_ts = DELTA_TS0, steady state
!                         3 : Sinusoidal air-temperature forcing
!                             between delta_ts = 0 C and delta_ts =
!                             -2*SINE_AMPLIT C with period
!                             SINE_PERIOD (in a)
!                         4 : delta_ts from ice-core data
!                             (e.g., GRIP, Vostok)
!                         5 : Surface temperature interpolated by using
!                             present values, LGM anomalies and a
!                             glacial index

#define DELTA_TS0 0.0d0
!                       Constant air-temperature deviation for steady
!                       states (only for TSURFACE==1)

#define SINE_AMPLIT 10.0d0
!                       Amplitude (in C) for sinusoidal air-temperature
!                       forcing (only for TSURFACE==3)

#define SINE_PERIOD 100000.0d0
!                       Period (in a) for sinusoidal air-temperature
!                       forcing (only for TSURFACE==3)

#define GRIP_TEMP_FILE 'grip95_temp_nosmooth_100ka.dat'
!                       Name of the file containing the ice-core
!                       air-temperature forcing (only for TSURFACE==4)

#define GRIP_TEMP_FACT 0.875d0
!                       Modification factor for ice-core air-temperature
!                       forcing (only for TSURFACE==4)

#define GLAC_IND_FILE 'grip_105ka_vostok_gi.dat'
!                       Name of the file containing the glacial-index
!                       forcing (only for TSURFACE==5)

#define TEMP_MM_ANOM_FILE 'none'
!                       Name of the file containing the LGM
!                       monthly-mean surface-temperature-anomaly data
!                       (difference LGM - present; only for TSURFACE==5)

#define TEMP_MM_ANOM_FACT 1.0d0
!                       Modification factor for the anomaly data of
!                       TEMP_MM_ANOM_FILE (for TSURFACE==5)

!  ------------------

#define PRECIP_MM_PRESENT_FILE 'nhem80_precmm.dat'
!                       Name of the file containing the present-day
!                       monthly-mean precipitation data

#define ACCSURFACE 2
!                         1 : Precipitation is constant factor ACCFACT
!                             times present distribution
!                         2 : Precipitation is coupled linearly to
!                             delta_ts, coupling parameter GAMMA_S
!                         3 : Precipitation is coupled exponentially to
!                             delta_ts, coupling parameter GAMMA_S
!                         5 : Precipitation interpolated by using
!                             present values, LGM anomalies and a
!                             glacial index (requires TSURFACE==5)

#define ACCFACT 1.0d0
!                       Constant ratio between actual and present
!                       precipitation (only for ACCSURFACE==1)

#define GAMMA_S 0.042857d0
!                       Parameter in the linear or exponential relation
!                       between precipitation and delta_ts
!                       (in 1/C, only for ACCSURFACE==2, 3)

#define PRECIP_MM_ANOM_FILE 'none'
!                       Name of the file containing the LGM
!                       monthly-mean precipitation-anomaly data
!                       (ratio LGM/present; only for ACCSURFACE==5)

#define PRECIP_MM_ANOM_FACT 1.0d0
!                       Modification factor for the anomaly data of
!                       PRECIP_MM_ANOM_FILE (for ACCSURFACE==5)

#define PRECIP_ANOM_INTERPOL 2
!                         1 : Interpolation with a linear function
!                             (for ACCSURFACE==5)
!                         2 : Interpolation with an exponential function
!                             (for ACCSURFACE==5)

!  ------------------

#define ABLSURFACE 1
!                         1 : Ablation parameterized
!                             by positive-degree-day (PDD) method.
!                             Rainfall assumed to run off instantaneously.
!                             Parameters defined in physical-parameter file.
!                         2 : Ablation parameterized
!                             by positive-degree-day (PDD) method.
!                             Rainfall assumed to contribute to formation 
!                             of superimposed ice.
!                             Parameters defined in physical-parameter file.
!                         3 : Ablation parameterized
!                             by linear-temperature-index (LTI) method.

#define LAMBDA_LTI 500.0d0
!                       Melting coefficient for the LTI method
!                       (in (mm WE)/(a*deg C), for ABLSURFACE==3)

#define TEMP_LTI -2.0d0
!                       Threshold summer temperature for the LTI method
!                       (in deg C, for ABLSURFACE==3)

!  ------------------

#define SEA_LEVEL 3
!                         1 : Constant sea level z_sl = Z_SL0
!                         2 : Saw-tooth-shaped sea-level forcing
!                             with glacial minimum z_sl = -130;
!                             for palaeoclimatic scenarios
!                         3 : Sea-level reconstruction from data
!                             (e.g., SPECMAP); for palaeoclimatic scenarios
!                         4 : Sea-level forcing by previously computed
!                             sea-level-equivalent scenario

#define Z_SL0 0.0d0
!                       Constant sea level
!                       (in m, only for SEA_LEVEL==1)

#define SEA_LEVEL_FILE 'specmap_zsl_782kyr.dat'
!                       Name of the file containing the sea-level
!                       forcing (only for SEA_LEVEL==3)

#define Z_SLE_FILE 'none'
!                       Name of the previously computed time-series
!                       file containing the sea-level-equivalent
!                       scenario (only for SEA_LEVEL==4)

#define Z_SLE_PATH 'none'
!                       Directory for file Z_SLE_FILE
!                       (no '/' at the end; only for SEA_LEVEL==4)

#define Z_SLE_FACT 1.0d0
!                       Factor for conversion from computed
!                       sea-level-equivalent to sea-level forcing
!                       (only for SEA_LEVEL==4)

!  ------------------

#define SLIDE_LAW 1
!                       1 : Weertman-type sliding,
!                           full ice pressure in denominator
!                       2 : Weertman-type sliding,
!                           reduced pressure (ice minus water) in denominator

#define C_SLIDE 11.2d0  /* for p=3, q=2 */
!                       Sliding coefficient, in m/[a*Pa^(p-q)]

#define GAMMA_SLIDE 0.0d0
!                       Sub-melt sliding coefficient, in K

#define P_WEERT 3
!                       Weertman exponent p for the basal shear stress
!                       (integer value)

#define Q_WEERT 2
!                       Weertman exponent q for the basal pressure
!                       (integer value)

!  ------------------

#define Q_GEO_MOD 1
!                         1 : Constant geothermal heat flux defined
!                             by parameter Q_GEO
!                         2 : Spatially varying geothermal heat flux
!                             read from file

#define Q_GEO 55.0d0
!                       Constant geothermal heat flux (for Q_GEO_MOD==1),
!                       in mW/m2

#define Q_GEO_FILE 'none'
!                       Name of the file containing the spatially varying
!                       geothermal heat flux (for Q_GEO_MOD==2)

!  ------------------

#define OUTPUT 2
!                         1 : Writing of time-slice data in files
!                             '.erg' with prescribed time step
!                         2 : Writing of time-slice data in files
!                             '.erg' at arbitrarily specified times

#define DTIME_OUT0 0.0d0
!                             Time step (in a) for writing of
!                             time-slice data (only for OUTPUT==1)

#define N_OUTPUT 4
!                             Number of specified times for writing of
!                             time-slice data (only for OUTPUT==2,
!                             not more than 20)

#define TIME_OUT0_01 -138000.0d0
#define TIME_OUT0_02 -127000.0d0
#define TIME_OUT0_03  -18000.0d0
#define TIME_OUT0_04       0.0d0
#define TIME_OUT0_05 1.11d+11
#define TIME_OUT0_06 1.11d+11
#define TIME_OUT0_07 1.11d+11
#define TIME_OUT0_08 1.11d+11
#define TIME_OUT0_09 1.11d+11
#define TIME_OUT0_10 1.11d+11
#define TIME_OUT0_11 1.11d+11
#define TIME_OUT0_12 1.11d+11
#define TIME_OUT0_13 1.11d+11
#define TIME_OUT0_14 1.11d+11
#define TIME_OUT0_15 1.11d+11
#define TIME_OUT0_16 1.11d+11
#define TIME_OUT0_17 1.11d+11
#define TIME_OUT0_18 1.11d+11
#define TIME_OUT0_19 1.11d+11
#define TIME_OUT0_20 1.11d+11
!                             Times (in a) for writing of time-slice
!                             data (only for OUTPUT==2, in increasing
!                             order from #1 to #N_OUTPUT)

!  ------------------

#define OUTSER 1
!                        *1*: Only writing of usual time-series file
!                             '.ser'
!                         2 : Writing of usual time-series file '.ser'
!                             and of time-series file '.thk' for the
!                             complete ice-thickness field

#define ERGDAT 1
!                         0 : Only 2-d fields written as time-slice
!                             data
!                        *1*: Full set of 2-d and 3-d fields written
!                             as time-slice data

#define NETCDF 1
!                         1 : Time-slice files unformatted
!                         2 : Time-slice files in format netCDF
!                         3 : Time-slice files both unformatted
!                             and in format netCDF

!  ------------------

#define NUMDIFF_ZS 0.0d0
!                       Spatial smoothing parameter for computation
!                       of zs (***use only in case of otherwise
!                       unrecoverable instability***)

#define NUMDIFF_H_T 0.15d0
!                       Spatial smoothing parameter for computation
!                       of H_t

#define TAU_CTS 0.0d0
!                       Numerical time lag (in a) for evolution
!                       of H_t

#define H_MIN 1.0d-03
!                       Minimum ice thickness interpreted as glaciation
!                       (in m)

#define VH_MAX 1.0d+04
!                       Lower (-VH_MAX) and upper (+VH_MAX) limits of
!                       horizontal velocities vx_c/t, vy_c/t (in m/a)

#define HD_MIN 1.0d-15
#define HD_MAX 500.0d0
!                       Lower and upper limits of free-surface
!                       diffusivity hdiff (in m^2/s)

#define QB_MIN 1.0d-15
#define QB_MAX 5.0d-07
!                       Lower and upper limits of the basal melting and
!                       drainage rates Q_bm, Q_tld and Q_b_tot (in m/s)

#define AGE_MIN 1.0d-15
#define AGE_MAX 1.0d+06
!                       Lower and upper limits of computed ages (in a)

#define MEAN_ACCUM 3.0d+02
!                       Mean accumulation rate over modelled ice sheet
!                       (in mm water equiv./a)
!                       [Only required for the numerical age gradient
!                       at the ice base, and, in case of CALCZS==3, for
!                       the convergence criterion of the SOR method.
!                       Need not be very precise, a rough estimate is
!                       sufficient.]

!  ------------------

#define NETCDF_INC '/usr/local/include/netcdf.inc'
!                       netCDF include file
