!-------- Specification of current simulation --------

#define RUNNAME 'smars'
!                      Name of simulation

#define SICO_VERSION '2.9'
!                      Version number of SICOPOLIS

#define INPATH '/uchi/greve/Documents/sicopolis/sico_in'
!                      Directory for SICOPOLIS input
!                      (no '/' at the end!)

#define OUTPATH '/data/greve/sico_out/smars'
!                      Directory for SICOPOLIS output
!                      (no '/' at the end!)

!  ------------------

#define SMARS
!                      Simulated domain:
!                        ANT      - Antarctica
!                        GRL      - Greenland
!                        SCAND    - Scandinavia
!                        NHEM     - Northern hemisphere
!                        TIBET    - Tibet
!                        EMTP2SGE - EISMINT Phase 2 Simplified Geometry Experiment
!                        HEINO    - ISMIP HEINO
!                        NMARS    - North polar cap of Mars
!                        SMARS    - South polar cap of Mars

!  ------------------

#define GRID 1
!                         0 : Cartesian coordinates in the
!                             stereographic plane
!                        *1*: Stereographically expanded Cartesian
!                             coordinates on the surface of the earth
!                         2 : Latitude/longitude grid on the surface
!                             of the earth (not allowed for this application)

#define X0 -1200.0d0
!                       x coordinate (in km) of the origin point (i,j) = (0,0),
!                       for GRID==0 or GRID==1

#define Y0 -1200.0d0
!                       y coordinate (in km) of the origin point (i,j) = (0,0),
!                       for GRID==0 or GRID==1

#define DX 20.0d0
!                       Horizontal grid spacing in km, for GRID==0
!                       or GRID==1
!                       (20 km requires IMAX=120 and JMAX=120,
!                        10 km requires IMAX=240 and JMAX=240,
!                         5 km requires IMAX=480 and JMAX=480)

#define IMAX 120
!                       IMAX+1: number of grid points in x-direction
!                               (i=0...IMAX)

#define JMAX 120
!                       JMAX+1: number of grid points in y-direction
!                               (j=0...JMAX)

#define KCMAX 50
!                       KCMAX+1: number of grid points in z-direction
!                                in cold ice (kc=0...KCMAX)

#define KTMAX 10
!                       KTMAX+1: number of grid points in z-direction
!                                in temperate ice (kt=0...KTMAX)

#define KRMAX 10
!                       KRMAX+1: number of grid points in z-direction
!                                in the bedrock (kr=0...KRMAX)

#define DEFORM 2.0d0
!                       Exponential stretch parameter of the non-
!                       equidistant grid in z-direction in cold ice

!  ------------------

#define TIME_INIT0 -10.0d+06
!                             Initial time of simulation (in a)

#define TIME_END0 0.0d0
!                             Final time of simulation (in a)

#define DTIME0 1000.0d0
!                       Time step (in a) for computation of velocity
!                       and topography

#define DTIME_TEMP0 1000.0d0
!                       Time step (in a) for computation of
!                       temperature, water content and age of the ice

#define DTIME_WSS0 10000.0d0
!                       Time step (in a) for computation of
!                       isostatic steady-state displacement of the lithosphere
!                       (only for REBOUND==2, ELRA model)

#define DTIME_SER0 1000.0d0
!                             Time step (in a) for writing of data to
!                             the time-series files '.ser' and '.thk'

#define YEAR_SEC 31556926.0d0
!                       Conversion from years to seconds

!  ------------------

#define CALCMOD 1
!                         0 : Simulation in cold-ice mode with
!                             resetting of temperatures above melting
!                        *1*: Simulation in polythermal mode

!  ------------------

#define MARGIN 1
!                         1 : Ice extent restricted to land area
!                         2 : Ice can spread into the sea, constant value
!                             for the minimum bedrock elevation
!                         3 : Ice can spread into the sea, minimum bedrock
!                             elevation proportional to sea-level stand

#define Z_MAR 0.0d0
!                             Minimum bedrock elevation (in m) allowed
!                             to glaciate (only for MARGIN==2)

#define FACT_Z_MAR 0.0d0
!                             Minimum bedrock elevation allowed to glaciate:
!                             proportionality factor to sea-level stand
!                             (only for MARGIN==3)

!  ------------------

#define FLOW_LAW 1
!                         1 : Glen's flow law with stress exponent n=3
!                         2 : Goldsby-Kohlstedt flow law with stress exponent
!                             n=1.8 and grain-size exponent p=1.4
!                         3 : Durham's flow law with stress exponent n=4

#define FIN_VISC 1
!                         1 : Unmodified flow law with infinite viscosity
!                             for low strain rates
!                         2 : Modified flow law with additional
!                             finite-viscosity parameter SIGMA_RES
!                             (defined in physical-parameter file)

#define GR_SIZE 1.0d-03
!                         Average grain size (in m; only for FLOW_LAW==2)

!  ------------------

#define FRAC_DUST 0.02d0
!                         Average volume fraction of dust in the ice.
!			  Should be within the range 0 <= FRAC_DUST <= 0.56.

!  ------------------

#define ENHMOD 1
!                         1 : Flow enhancement factor enh=ENH
!                             everywhere
!                         2 : enh=1 for ice younger than 11 ka
!                             (Holocene ice), enh=ENH for ice older
!                             than 11 ka (Pleistocene ice); for present
!                             steady-state simulations
!                         3 : enh=1 for Holocene and Eemian ice,
!                             enh=ENH for Weichselian and pre-Eemian
!                             ice; for palaeoclimatic and
!                             greenhouse-warming scenarios
!
!                         Note that if the dust content FRAC_DUST is not
!                         equal to zero, these values are multiplied by the
!                         hardening factor exp(-n*b*FRAC_DUST)
!                         [where n is the stress exponent, and b=2].

#define ENH 1.0d0
!                         Flow enhancement factor for glacial (for instance,
!                         Weichselian) ice

!  ------------------

#define DNS 1
!                         1 : Computation of vx and vy in the
!                             shallow-ice approximation
!                             (no deviatoric normal stresses)
!                         2 : Algorithm of Heinz Blatter for the
!                             computation of vx und vy
!                             (with deviatoric normal stresses);
!                             requires CALCMOD=0 und CALCZS=0
!                             (***under construction***)

!  ------------------

#define ANF_DAT 1
!                         1 : Present initial topography
!                               IMPORTANT: Make sure that the correct bedrock
!                               file smarsxx_zb.dat is specified below.
!                               It must fit the selected dust content
!                               FRAC_DUST, and, in case of REBOUND==0 or 1
!                               (LLRA model), the selected fraction of isostatic
!                               compensation FRAC_LLRA,
!                               or, in case of REBOUND==2 (ELRA model), the 
!                               selected flexural rigidity !!!!!!!!!!!!!!!!!!!!!
!                         2 : Ice-free initial topography with
!                             relaxed lithosphere
!                         3 : Initial values from previous
!                             simulation

#define ZS_PRESENT_FILE   'smars20_zs.dat'
!                             Name of the file containing the present-day
!                             surface topography

#define ZB_PRESENT_FILE   'smars20_zb_flexrig3_dust002.dat'
!                             Name of the file containing the present-day
!                             bedrock topography (only for ANF_DAT==1)

#define ZB0_FILE          'smars20_zb0_elra.dat'
!                             Name of the file containing the topography
!                             of the relaxed bedrock

#define MASK_PRESENT_FILE 'smars20_mask.dat'
!                             Name of the file containing the present-day
!                             ice-land-ocean mask

#define ANFDATNAME 'none'
!                             Initial-value file (only for ANF_DAT==3)

#define ANFDATPATH 'none'
!                             Directory for initial-value file
!                             (no '/' at the end!)

!  ------------------

#define REBOUND 2
!                         0 : No bedrock adjustment
!                         1 : Isostatic bedrock adjustment with local
!                             lithosphere and relaxing asthenosphere (LLRA model)
!                         2 : Isostatic bedrock adjustment with elastic
!                             lithosphere and relaxing asthenosphere (ELRA model)

#define FRAC_LLRA 0.65d0
!                             Fraction of isostatic compensation in the LLRA
!                             model (REBOUND==1). Range 0 <= FRAC_LLRA <= 1;
!                             0: no bedrock adjustment, 1: full adjustment.

#define Q_LITHO 1
!                         0 : No coupled heat-conducting bedrock
!                        *1*: Coupled heat-conducting bedrock

!  ------------------

#define ZS_EVOL 1
!                         0 : No evolution of the free surface, kept fixed on
!                             the initial topography
!                        *1*: Evolution of the free surface

#define CALCZS 3
!                             Solution of the ice-thickness equation:
!                         0 : Explicit scheme
!                         1 : ADI scheme
!                         2 : Over-implicit ADI scheme
!                         3 : Over-implicit scheme with iterative SOR solution
!                             of the resulting systems of linear equations 

#define OVI_WEIGHT 1.5d0
!                       Weighing parameter for the over-implicit
!                       schemes (only for CALCZS==2, 3)

#define OMEGA_SOR 1.0d0
!                       Relaxation parameter for the iterative SOR solver
!                       for systems of linear equations
!                       (0 < OMEGA_SOR < 2, only for CALCZS==3)

!  ------------------

#define ADV_HOR 2
!                         Discretization of horizontal advection terms in the
!                         3-d temperature and age equations:
!                         1 : Not defined
!                             (central differences would be unstable!)
!                         2 : Non-conservative first-order upstream
!                         3 : Non-conservative first-order upstream for
!                             temperature equation, conservative first-order
!                             upstream for age equation (by B. Muegge;
!                             requires ADV_VERT==3)
!                         4 : Conservative first-order upstream
!                             (requires also a conservative scheme for
!                              vertical advection which has not yet been
!                              implemented -> ***under construction***)

#define ADV_VERT 2
!                         Discretization of vertical advection terms in the
!                         3-d temperature and age equations:
!                         1 : Central differences (plus artificial diffusion
!                             for the age equation)
!                         2 : Non-conservative first-order upstream
!                         3 : Non-conservative first-order upstream for
!                             temperature equation, conservative first-order
!                             upstream for age equation (by B. Muegge;
!                             requires ADV_HOR==3)

#define AGEDIFF 5.0d-11
!                       Numerical age diffusivity
!                       (in m2/s, only for ADV_VERT==1)

#define M_AGE 200.0d0
!                       Parameter for the prescribed numerical age
!                       gradient at the ice base (only for ADV_VERT==1)

!  ------------------

#define TOPOGRAD 1
!                             Topography gradients at grid points
!                             with
!                         0 : second-order discretisation
!                        *1*: fourth-order discretisation

!  ------------------

#define TSURFACE 6
!                         1 : delta_ts = DELTA_TS0, steady state
!                         3 : Sinusoidal air-temperature forcing
!                             between delta_ts = 0 C and delta_ts =
!                             +2*SINE_AMPLIT C with period
!                             SINE_PERIOD (in a)
!                         4 : delta_ts from south-polar insolation based
!                             on simplified two-cycle obliquity (main cycle
!                             and first modulation)
!                         5 : delta_ts from south-polar insolation based
!                             on computed obliquity and eccentricity
!                             evolutions
!                         6 : Surface temperature computed by
!                             Bjoern Grieger's parameterisation

#define DELTA_TS0 0.0d0
!                       Constant air-temperature deviation for steady
!                       states (for TSURFACE==1);
!                       constant offset for Bjoern Grieger's parameterisation
!                       (for TSURFACE==6)

#define SINE_AMPLIT 15.0d0
!                       Amplitude (in C) for sinusoidal air-temperature
!                       forcing (only for TSURFACE==3)

#define SINE_PERIOD 1.3d+06
!                       Period (in a) for sinusoidal air-temperature
!                       forcing (only for TSURFACE==3)

#define TEMP0_MA_90S -125.0d0
!                       Present mean-annual air temperature (in C) at the
!                       south pole (only for TSURFACE==1, 2, 3)

#define C_MA 1.0d0
!                       Co-latitude factor in the parameterization of the
!                       present mean-annual air temperature
!                       (in C/deg co-lat.) (for TSURFACE==1, 2, 3, 4, 5)

#define GAMMA_MA 0.0d0
!                       Lapse rate in the parameterization of the
!                       present mean-annual air temperature (in C/m)
!                       (for TSURFACE==1, 2, 3, 4, 5)

#define INSOL_MA_90S_FILE 'mars_laskar_2004.dat'
!                       Name of the file containing the mean-annual south-polar
!                       insolation (for TSURFACE==5, 6)

#define ALBEDO 0.3d0
!                       Albedo of the residual cap (only for TSURFACE==4, 5, 6)
!                       Standard value for TSURFACE==4, 5:  0.7d0
!                                      for TSURFACE==   6:  0.3d0 (planetary mean)

!  ------------------

#define ACC_UNIT 2
!                         Unit of input accumulation data:
!                         1 : mm water equiv./a
!                         2 : mm (ice + dust) equiv./a

#define ACC_PRESENT 1
!                         Data for the present accumulation:
!                         1 : Spatially constant value

#define ACC_PRESENT_VAL 0.0d0
!                         Value of present accumulation,
!                         in mm water or (ice + dust) equiv./a
!                         (only for ACC_PRESENT==1)

#define ACCSURFACE 3
!                         1 : Accumulation coupled linearly to delta_ts,
!                             coupling parameter GAMMA_S
!                         2 : Accumulation coupled exponentially to delta_ts,
!                             coupling parameter GAMMA_S
!                         3 : Accumulation coupled to delta_ts via
!                             Clausius-Clapeyron relation.

#define GAMMA_S 0.0d0
!                       Parameter in the linear or exponential relation
!                       between accumulation and delta_ts
!                       (in 1/C, only for ACCSURFACE==1, 2)

#define ABLSURFACE 1
!                         1 : Ablation parameterized
!                             by equilibrium-line-distance approach.
!                             Equilibrium-line distance coupled linearly
!                             to delta_ts, coupling parameter GAMMA_ELD
!                         2 : Ablation parameterized
!                             by equilibrium-line-distance approach.
!                             Equilibrium-line distance coupled exponentially
!                             to delta_ts, coupling parameter GAMMA_ELD

#define ELD_0 6.5d+02
!                       Present equilibrium-line distance for the ablation
!                       parameterization (in km, only for ABLSURFACE==1, 2)

#define G_0 2.5d-06
!                       Accumulation-ablation-rate gradient,
!                       in [mm water or (ice + dust) equiv./a]/km
!                       (only for ABLSURFACE==1, 2)

#define GAMMA_ELD 0.0d0
!                       Parameter in the linear or exponential relation
!                       between equilibrium line distance (eld) and delta_ts
!                       (in 1/C, only for ABLSURFACE==1, 2)

#define GAMMA_G 1.0d0
!                       Parameter in the linear relation between
!                       accumulation-ablation-rate gradient (G_0)
!                       and accumulation ratio accum_now/accum_present
!                       (only for ABLSURFACE==1, 2)

!  ------------------

#define SEA_LEVEL 1
!                         1 : Constant sea level z_sl = Z_SL0
!                         2 : Saw-tooth-shaped sea-level forcing
!                             with glacial minimum z_sl = -130;
!                             for palaeoclimatic scenarios
!                         3 : Sea-level reconstruction from data
!                             (e.g., SPECMAP); for palaeoclimatic scenarios
!                         4 : Sea-level forcing by previously computed
!                             sea-level-equivalent scenario

#define Z_SL0 -1.11d+11
!                       Constant sea level
!                       (in m, only for SEA_LEVEL==1)

#define SEA_LEVEL_FILE 'none'
!                       Name of the file containing the sea-level
!                       forcing (only for SEA_LEVEL==3)

#define Z_SLE_FILE 'none'
!                       Name of the previously computed time-series
!                       file containing the sea-level-equivalent
!                       scenario (only for SEA_LEVEL==4)

#define Z_SLE_PATH 'none'
!                       Directory for file Z_SLE_FILE
!                       (no '/' at the end; only for SEA_LEVEL==4)

#define Z_SLE_FACT 1.0d0
!                       Factor for conversion from computed
!                       sea-level-equivalent to sea-level forcing
!                       (only for SEA_LEVEL==4)

!  ------------------

#define SLIDE_LAW 1
!                       1 : Weertman-type sliding,
!                           full ice pressure in denominator
!                       2 : Weertman-type sliding,
!                           reduced pressure (ice minus water) in denominator

#define C_SLIDE 0.0d0
!                       Sliding coefficient, in m/[a*Pa^(p-q)]

#define GAMMA_SLIDE 0.0d0
!                       Sub-melt sliding coefficient, in K

#define P_WEERT 1
!                       Weertman exponent p for the basal shear stress
!                       (integer value)

#define Q_WEERT 0
!                       Weertman exponent q for the basal pressure
!                       (integer value)

!  ------------------

#define Q_GEO_MOD 1
!                         1 : Constant geothermal heat flux defined
!                             by parameter Q_GEO
!                         2 : Spatially varying geothermal heat flux
!                             read from file

#define Q_GEO 35.0d0
!                       Constant geothermal heat flux (for Q_GEO_MOD==1),
!                       in mW/m2

#define Q_GEO_FILE 'smars20_qgeo_xxx.dat'
!                       Name of the file containing the spatially varying
!                       geothermal heat flux (for Q_GEO_MOD==2)

!  ------------------

#define CHASM 1
!                       1 : No special treatment of Chasma Australis
!                       2 : Special treatment of Chasma Australis

#define MASK_CHASM_FILE 'smars20_mask_chasm.dat'
!                       Name of the file containing the chasm mask
!                       (for CHASM==2)

#define TIME_CHASM_INIT0 -1.11d+11
!                       Initial time for active chasm area (in a)

#define TIME_CHASM_END0 1.11d+11
!                       Final time for active chasm area (in a)

#define Q_GEO_CHASM 150.0d0
!                       Geothermal heat flux for the chasm area, in mW/m2
!                       (for CHASM==2, active between TIME_CHASM_INIT0 and
!                       TIME_CHASM_END0)

#define EROSION_CHASM 0.0d0
!                       Wind erosion rate for the chasm area,
!                       in mm water or (ice + dust) equiv./a (for CHASM==2,
!                       active between TIME_CHASM_INIT0 and TIME_CHASM_END0)

!  ------------------

#define OUTPUT 2
!                         1 : Writing of time-slice data in files
!                             '.erg' with prescribed time step
!                         2 : Writing of time-slice data in files
!                             '.erg' at arbitrarily specified times

#define DTIME_OUT0 0.0d0
!                             Time step (in a) for writing of
!                             time-slice data (only for OUTPUT==1)

#define N_OUTPUT 5
!                             Number of specified times for writing of
!                             time-slice data (only for OUTPUT==2,
!                             not more than 20)

#define TIME_OUT0_01 -9.999d+06
#define TIME_OUT0_02 -7.5d+06
#define TIME_OUT0_03 -5.0d+06
#define TIME_OUT0_04 -2.5d+06
#define TIME_OUT0_05  0.0d0
#define TIME_OUT0_06 1.11d+11
#define TIME_OUT0_07 1.11d+11
#define TIME_OUT0_08 1.11d+11
#define TIME_OUT0_09 1.11d+11
#define TIME_OUT0_10 1.11d+11
#define TIME_OUT0_11 1.11d+11
#define TIME_OUT0_12 1.11d+11
#define TIME_OUT0_13 1.11d+11
#define TIME_OUT0_14 1.11d+11
#define TIME_OUT0_15 1.11d+11
#define TIME_OUT0_16 1.11d+11
#define TIME_OUT0_17 1.11d+11
#define TIME_OUT0_18 1.11d+11
#define TIME_OUT0_19 1.11d+11
#define TIME_OUT0_20 1.11d+11
!                             Times (in a) for writing of time-slice
!                             data (only for OUTPUT==2, in increasing
!                             order from #1 to #N_OUTPUT)

!  ------------------

#define OUTSER 1
!                        *1*: Only writing of usual time-series file
!                             '.ser'
!                         2 : Writing of usual time-series file '.ser'
!                             and of time-series file '.thk' for the
!                             complete ice-thickness field
#define ERGDAT 1
!                         0 : Only 2-d fields written as time-slice
!                             data
!                        *1*: Full set of 2-d and 3-d fields written
!                             as time-slice data
#define NETCDF 1
!                         1 : Time-slice files unformatted
!                         2 : Time-slice files in format netCDF
!                         3 : Time-slice files both unformatted
!                             and in format netCDF

!  ------------------

#define NUMDIFF_ZS 0.0d0
!                       Spatial smoothing parameter for computation
!                       of zs (***use only in case of otherwise
!                       unrecoverable instability***)

#define NUMDIFF_H_T 0.15d0
!                       Spatial smoothing parameter for computation
!                       of H_t

#define TAU_CTS 0.0d0
!                       Numerical time lag (in a) for evolution
!                       of H_t

#define H_MIN 1.0d-06
!                       Minimum ice thickness interpreted as glaciation
!                       (in m)

#define VH_MAX 10.0d0
!                       Lower (-VH_MAX) and upper (+VH_MAX) limits of
!                       horizontal velocities vx_c/t, vy_c/t (in m/a)

#define HD_MIN 1.0d-15
#define HD_MAX 20.0d0
!                       Lower and upper limits of free-surface
!                       diffusivity hdiff (in m^2/s)

#define QB_MIN 1.0d-15
#define QB_MAX 1.0d-08
!                       Lower and upper limits of the basal melting and
!                       drainage rates Q_bm, Q_tld and Q_b_tot (in m/s)

#define AGE_MIN 1.0d-15
#define AGE_MAX 1.0d+09
!                       Lower and upper limits of computed ages (in a)

#define MEAN_ACCUM 1.0d-01
!                       Mean accumulation rate over modelled ice sheet,
!                       in mm water or (ice + dust) equiv./a
!                       [Only required for the numerical age gradient
!                       at the ice base, and, in case of CALCZS==3, for
!                       the convergence criterion of the SOR method.
!                       Need not be very precise, a rough estimate is
!                       sufficient.]

!  ------------------

#define NETCDF_INC '/usr/local/include/netcdf.inc'
!                       netCDF include file
