!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  s i c o _ i n i t

!   Purpose    :  Initializations for SICOPOLIS.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine sico_init(delta_ts, glac_index, &
               sum_accum, mean_accum, mean_accum_inv, &
               dtime, dtime_temp, dtime_wss, dtime_out, dtime_ser, &
               time, time_init, time_end, time_output, &
               dxi, deta, dzeta_c, dzeta_t, dzeta_r, &
               dtt_2dxi, dtt_2deta, dtime_temp_inv, &
               griptemp, specmap_zsl, z_sle_forcing, time_forcing, &
               z_sl, z_mar, &
               grip_time_min, grip_time_stp, grip_time_max, &
               specmap_time_min, specmap_time_stp, specmap_time_max, &
               avxy3, avz3, aqxy1, azs2, azs3, &
               at1, at2_1, at2_2, at3_1, at3_2, at4_1, &
               at4_2, at5, at6, at7, &
               aw1, aw2, aw3, aw4, aw5, aw8, aw9, aw7, &
               ai1, ai2, ai3, ai4, atr1, am1, am2, atm1, atm2, &
               acb1, acb2, acb3, acb4, alb1, &
               aqbm1, aqbm2, aqbm3, aqbm4, aqtld, &
               ii, jj, nn, &
               ndat, ndata_forcing, n_output, &
               runname, zs_switch)

use sico_variables

implicit none
integer i, j, kc, kt, kr, m, n, ir, jr
integer ios, ios1, ios2, ios3, ios4
integer ndat, ndata_forcing
integer n_output
integer ii((IMAX+1)*(JMAX+1)), jj((IMAX+1)*(JMAX+1)), nn(0:JMAX,0:IMAX)
integer grip_time_min, grip_time_stp, grip_time_max, &
        specmap_time_min, specmap_time_stp, specmap_time_max
integer ndata_grip, ndata_specmap
double precision delta_ts, glac_index
double precision sum_accum, mean_accum, mean_accum_inv
double precision dtime0, dtime_temp0, dtime_wss0, dtime_out0, dtime_ser0
double precision dtime, dtime_temp, dtime_wss, dtime_out, dtime_ser
double precision time_init0, time_end0, time_output0(100)
double precision time, time_init, time_end, time_output(100)
double precision dxi, deta, dzeta_c, dzeta_t, dzeta_r
double precision dtt_2dxi, dtt_2deta, dtime_temp_inv
double precision griptemp(0:10000), specmap_zsl(0:10000), &
       z_sle_forcing(0:10000), time_forcing(0:10000), z_sl, z_mar
double precision avxy3(0:KCMAX), &
       avz3(0:KCMAX), &
       aqxy1(0:KCMAX), azs2, azs3, &
       at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), &
       at7, &
       aw1, aw2, aw3, aw4, aw5, aw8, aw9, aw7, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai3, ai4, &
       atr1, &
       am1, am2, &
       atm1(0:KCMAX), atm2(0:KTMAX), &
       acb1, acb2, acb3, acb4, &
       alb1, &
       aqbm1, aqbm2, aqbm3, aqbm4, aqtld
double precision kappa_val
double precision d_dummy
character (len=100) :: runname, anfdatname
character ch_dummy
logical zs_switch

!-------- Read physical parameters --------

call phys_para

!-------- Compatibility check of the SICOPOLIS version with the header file

if ( trim(VERSION) /= trim(SICO_VERSION) ) &
   stop ' SICOPOLIS version not compatible with header file!'

!-------- Compatibility check of the horizontal resolution with the
!         number of grid points --------

#if (GRID==0 || GRID==1)

if (abs(DX-80.0d0) < eps) then

   if ((IMAX /= 70).or.(JMAX /= 60)) then
      stop ' IMAX and/or JMAX wrong!'
   end if

else if (abs(DX-40.0d0) < eps) then

   if ((IMAX /= 140).or.(JMAX /= 120)) then
      stop ' IMAX and/or JMAX wrong!'
   end if


else if (abs(DX-20.0d0) < eps) then

   if ((IMAX /= 280).or.(JMAX /= 240)) then
      stop ' IMAX and/or JMAX wrong!'
   end if

else
      stop ' DX wrong!'
end if

#elif GRID==2

stop ' GRID==2 not allowed for this application!'

#endif

!-------- Compatibility check of surface-temperature and precipitation
!         determination by interpolation between present and LGM values
!         with a glacial index --------

#if ((TSURFACE == 5) && (ACCSURFACE != 5))
stop ' Options TSURFACE==5 and ACCSURFACE==5 must be used together!'
#endif

#if ((TSURFACE != 5) && (ACCSURFACE == 5))
stop ' Options TSURFACE==5 and ACCSURFACE==5 must be used together!'
#endif

!-------- Compatibility check of discretization schemes for the horizontal and
!         vertical advection terms in the temperature and age equations --------

#if ADV_HOR==1
stop ' Option ADV_HOR==1 (central differences) not defined!'
#endif

#if ((ADV_HOR == 3) && (ADV_VERT != 3))
stop ' Options ADV_HOR==3 and ADV_VERT==3 must be used together!'
#endif

#if ((ADV_HOR != 3) && (ADV_VERT == 3))
stop ' Options ADV_HOR==3 and ADV_VERT==3 must be used together!'
#endif

!-------- Setting of forcing flag --------

#if (TSURFACE <= 4)

forcing_flag = 1   ! forcing by delta_ts

#elif (TSURFACE == 5)

forcing_flag = 2   ! forcing by glac_index

#endif

!-------- Initialization of numerical time steps --------

dtime0      = DTIME0
dtime_temp0 = DTIME_TEMP0
#if REBOUND==2
dtime_wss0  = DTIME_WSS0
#endif

!-------- Further initializations --------

dzeta_c = 1.0d0/dble(KCMAX)
dzeta_t = 1.0d0/dble(KTMAX)
dzeta_r = 1.0d0/dble(KRMAX)
ndat = 1
zs_switch = .true.

!-------- Construction of a vector (with index n) from a 2-d array
!         (with indices i, j) by diagonal numbering --------

#if CALCZS==3

n=1

do m=0, IMAX+JMAX
   do i=m, 0, -1
      j = m-i
      if ((i.le.IMAX).and.(j.le.JMAX)) then
         ii(n)   = i
         jj(n)   = j
         nn(j,i) = n
         n=n+1
      end if
   end do
end do

#endif

!-------- Specification of current simulation --------

runname    = RUNNAME
anfdatname = ANFDATNAME
time_init0 = TIME_INIT0
time_end0  = TIME_END0
dtime_ser0 = DTIME_SER0

#if OUTPUT==1
dtime_out0 = DTIME_OUT0
#elif OUTPUT==2
n_output         = N_OUTPUT
time_output0( 1) = TIME_OUT0_01
time_output0( 2) = TIME_OUT0_02
time_output0( 3) = TIME_OUT0_03
time_output0( 4) = TIME_OUT0_04
time_output0( 5) = TIME_OUT0_05
time_output0( 6) = TIME_OUT0_06
time_output0( 7) = TIME_OUT0_07
time_output0( 8) = TIME_OUT0_08
time_output0( 9) = TIME_OUT0_09
time_output0(10) = TIME_OUT0_10
time_output0(11) = TIME_OUT0_11
time_output0(12) = TIME_OUT0_12
time_output0(13) = TIME_OUT0_13
time_output0(14) = TIME_OUT0_14
time_output0(15) = TIME_OUT0_15
time_output0(16) = TIME_OUT0_16
time_output0(17) = TIME_OUT0_17
time_output0(18) = TIME_OUT0_18
time_output0(19) = TIME_OUT0_19
time_output0(20) = TIME_OUT0_20
#endif

!-------- Write log file --------

open(10, iostat=ios, &
     file=OUTPATH//'/'//trim(runname)//'.log', &
     status='new')
if (ios /= 0) then
   write(6,1000) 'Error when opening the log file!'
   stop
end if

write(10,1000) 'Computational domain:'
#if defined(ANT)
write(10,1000) 'Antarctica'
#elif defined(GRL)
write(10,1000) 'Greenland'
#elif defined(SCAND)
write(10,1000) 'Scandinavia and Eurasia'
#elif defined(NHEM)
write(10,1000) 'Northern hemisphere'
#elif defined(EMTP2SGE)
write(10,1000) 'EISMINT Phase 2 Simplified Geometry Experiment'
#elif defined(NMARS)
write(10,1000) 'North polar cap of Mars'
#elif defined(SMARS)
write(10,1000) 'South polar cap of Mars'
#else
stop ' Subroutine sico_init: No valid domain specified!'
#endif
write(10,1000) ' '

write(10,1001) 'imax  =', IMAX
write(10,1001) 'jmax  =', JMAX
write(10,1001) 'kcmax =', KCMAX
write(10,1001) 'ktmax =', KTMAX
write(10,1001) 'krmax =', KRMAX
write(10,1000) ' '

write(10,1002) 'a =', DEFORM
write(10,1000) ' '

#if (GRID==0 || GRID==1)
write(10,1002) 'x0      =', X0
write(10,1002) 'y0      =', Y0
write(10,1002) 'dx      =', DX
#elif GRID==2
stop ' GRID==2 not allowed for this application!'
#endif
write(10,1000) ' '

write(10,1002) 'time_init  =', time_init0
write(10,1002) 'time_end   =', time_end0
write(10,1002) 'dtime      =', dtime0
write(10,1002) 'dtime_temp =', dtime_temp0
#if REBOUND==2
write(10,1002) 'dtime_wss  =', dtime_wss0
#endif
#if OUTPUT==1
write(10,1002) 'dtime_out  =', dtime_out0
#endif
write(10,1002) 'dtime_ser  =', dtime_ser0
write(10,1000) ' '

write(10,1000) 'zs_present file   = '//ZS_PRESENT_FILE
#if ANF_DAT==1
write(10,1000) 'zb_present file   = '//ZB_PRESENT_FILE
#endif
write(10,1000) 'zb0 file          = '//ZB0_FILE
write(10,1000) 'mask_present file = '//MASK_PRESENT_FILE
write(10,1000) ' '

#if (CALCZS==2 || CALCZS==3)
write(10,1002) 'ovi_weight =', OVI_WEIGHT
#if CALCZS==3
write(10,1002) 'omega_sor  =', OMEGA_SOR
#endif
write(10,1000) ' '
#endif

#if TSURFACE==1
write(10,1002) 'delta_ts0      =', DELTA_TS0
#elif TSURFACE==3
write(10,1002) 'sine_amplit    =', SINE_AMPLIT
write(10,1002) 'sine_period    =', SINE_PERIOD
#elif TSURFACE==4
write(10,1000) 'GRIP file      = '//GRIP_TEMP_FILE
write(10,1002) 'grip_temp_fact =', GRIP_TEMP_FACT
#elif TSURFACE==5
write(10,1000) 'Glacial-index file = '//GLAC_IND_FILE
write(10,1000) 'temp_ma_anom file  = '//TEMP_MA_ANOM_FILE
write(10,1002) 'temp_ma_anom fact  = ', TEMP_MA_ANOM_FACT
write(10,1000) 'temp_mj_anom file  = '//TEMP_MJ_ANOM_FILE
write(10,1002) 'temp_mj_anom fact  = ', TEMP_MJ_ANOM_FACT
#endif

write(10,1000) 'precip_present file = '//PRECIP_PRESENT_FILE
#if ACCSURFACE==1
write(10,1002) 'accfact        =', ACCFACT
#elif ( ACCSURFACE==2 || ACCSURFACE==3 )
write(10,1002) 'gamma_s        =', GAMMA_S
#elif ( ACCSURFACE==5 )
write(10,1000) 'precip_anom file    = '//PRECIP_ANOM_FILE
write(10,1002) 'precip_anom fact    = ', PRECIP_ANOM_FACT
#endif

#if ABLSURFACE==3
write(10,1002) 'lambda_lti     =', LAMBDA_LTI
write(10,1002) 'temp_lti       =', TEMP_LTI
#endif

#if SEA_LEVEL==1
write(10,1002) 'z_sl0          =', Z_SL0
#elif SEA_LEVEL==3
write(10,1000) 'sea-level file = '//SEA_LEVEL_FILE
#elif SEA_LEVEL==4
write(10,1000) 'z_sle file     = '//Z_SLE_FILE
write(10,1002) 'z_sle_fact     =', Z_SLE_FACT
#endif
write(10,1000) ' '

#if MARGIN==2
write(10,1002) 'z_mar          =', Z_MAR
write(10,1000) ' '
#elif ( MARGIN==3 || MARGIN==4 )
write(10,1002) 'fact_z_mar     =', FACT_Z_MAR
write(10,1000) ' '
#endif

write(10,1002) 'c_slide     =', C_SLIDE
write(10,1002) 'gamma_slide =', GAMMA_SLIDE
write(10,1001) 'p_weert     =', P_WEERT
write(10,1001) 'q_weert     =', Q_WEERT
write(10,1000) ' '

#if Q_GEO_MOD==1
write(10,1002) 'q_geo =', Q_GEO
#elif Q_GEO_MOD==2
write(10,1000) 'q_geo file = '//Q_GEO_FILE
#endif
write(10,1000) ' '

#if REBOUND==1
write(10,1002) 'frac_llra =', FRAC_LLRA
write(10,1000) ' '
#endif

#if FLOW_LAW==2
write(10,1002) 'gr_size   =', GR_SIZE
#endif
write(10,1002) 'enh       =', ENH
write(10,1000) ' '

write(10,1002) 'numdiff_zs  =', NUMDIFF_ZS
write(10,1002) 'numdiff_H_t =', NUMDIFF_H_T
write(10,1002) 'tau_cts     =', TAU_CTS
write(10,1002) 'H_min       =', H_MIN
write(10,1002) 'vh_max      =', VH_MAX
write(10,1002) 'hd_min      =', HD_MIN
write(10,1002) 'hd_max      =', HD_MAX
write(10,1002) 'qb_min      =', QB_MIN
write(10,1002) 'qb_max      =', QB_MAX
write(10,1002) 'age_min     =', AGE_MIN
write(10,1002) 'age_max     =', AGE_MAX
write(10,1002) 'mean_accum  =', MEAN_ACCUM
#if ADV_VERT==1
write(10,1002) 'age_diff    =', AGEDIFF
write(10,1002) 'm_age       =', M_AGE
#endif
write(10,1000) ' '

write(10,1001) 'GRID       =', GRID
write(10,1001) 'CALCMOD    =', CALCMOD
write(10,1001) 'FLOW_LAW   =', FLOW_LAW
write(10,1001) 'FIN_VISC   =', FIN_VISC
write(10,1001) 'ENHMOD     =', ENHMOD
write(10,1001) 'MARGIN     =', MARGIN
write(10,1001) 'ANF_DAT    =', ANF_DAT
write(10,1001) 'REBOUND    =', REBOUND
write(10,1001) 'Q_LITHO    =', Q_LITHO
write(10,1001) 'ZS_EVOL    =', ZS_EVOL
write(10,1001) 'CALCZS     =', CALCZS
write(10,1001) 'ADV_HOR    =', ADV_HOR
write(10,1001) 'ADV_VERT   =', ADV_VERT
write(10,1001) 'TOPOGRAD   =', TOPOGRAD
write(10,1001) 'TSURFACE   =', TSURFACE
write(10,1001) 'ACCSURFACE =', ACCSURFACE
#if ACCSURFACE==5
write(10,1001) 'PRECIP_ANOM_INTERPOL =', PRECIP_ANOM_INTERPOL
#endif
write(10,1001) 'ABLSURFACE =', ABLSURFACE
write(10,1001) 'SEA_LEVEL  =', SEA_LEVEL
write(10,1001) 'SLIDE_LAW  =', SLIDE_LAW
write(10,1001) 'Q_GEO_MOD  =', Q_GEO_MOD
write(10,1001) 'OUTPUT     =', OUTPUT
write(10,1001) 'OUTSER     =', OUTSER
write(10,1001) 'ERGDAT     =', ERGDAT
write(10,1001) 'NETCDF     =', NETCDF
write(10,1000) ' '
#if OUTPUT==2
write(10,1001) 'n_output =', n_output
do n=1, n_output
   write(10,1002) 'time_output =', time_output0(n)
end do
write(10,1000) ' '
#endif

write(10,1000) 'Program version and date: '//VERSION//' / '//DATE

 1000 format(a)
 1001 format(a,i4)
 1002 format(a,es12.4)

close(10, status='keep')

!-------- Conversion of time quantities --------

time_init  = time_init0*YEAR_SEC    ! a --> s
time_end   = time_end0*YEAR_SEC     ! a --> s
dtime      = dtime0*YEAR_SEC        ! a --> s
dtime_temp = dtime_temp0*YEAR_SEC   ! a --> s
#if REBOUND==2
dtime_wss  = dtime_wss0*YEAR_SEC    ! a --> s
#endif
dtime_ser  = dtime_ser0*YEAR_SEC    ! a --> s
#if OUTPUT==1
dtime_out  = dtime_out0*YEAR_SEC    ! a --> s
#elif OUTPUT==2
do n=1, n_output
   time_output(n) = time_output0(n)*YEAR_SEC  ! a --> s
end do
#endif

dtime_temp_inv = 1.0d0/dtime_temp

if (abs(dtime_temp/dtime-nint(dtime_temp/dtime)) > eps) &
   stop ' dtime_temp must be a multiple of dtime!'

#if REBOUND==2
if (abs(dtime_wss/dtime-nint(dtime_wss/dtime)) > eps) &
   stop ' dtime_wss must be a multiple of dtime!'
#endif

!-------- Reading of present mean-annual precipitation rate --------

#if (GRID==0 || GRID==1)

open(21, iostat=ios, &
     file=INPATH//'/ant/'//PRECIP_PRESENT_FILE, &
     recl=8192, status='old')

#elif GRID==2

stop ' GRID==2 not allowed for this application!'

#endif

if (ios /= 0) stop ' Error when opening the precip file!'

do n=1, 6; read(21,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(21,*) (precip_ma_present(j,i), i=0,IMAX)
end do

close(21, status='keep')

precip_ma_present = precip_ma_present *(1.0d-03/YEAR_SEC)*(RHO_W/RHO)
                                       ! mm/a water equiv. -> m/s ice equiv.

!-------- Present monthly precipitation rates
!         (assumed to be equal to the mean annual precipitation rate) --------

do n=1, 12   ! month counter
do i=0, IMAX
do j=0, JMAX
   precip_present(j,i,n) = precip_ma_present(j,i)
end do
end do
end do

!-------- Reading of LGM mean-annual precipitation-rate anomaly --------

#if ACCSURFACE==5

#if (GRID==0 || GRID==1)

open(21, iostat=ios, &
     file=INPATH//'/ant/'//PRECIP_ANOM_FILE, &
     recl=8192, status='old')

#endif

if (ios /= 0) stop ' Error when opening the precip anomaly file!'

do n=1, 6; read(21,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(21,*) (precip_ma_lgm_anom(j,i), i=0,IMAX)
end do

close(21, status='keep')

precip_ma_lgm_anom = precip_ma_lgm_anom * PRECIP_ANOM_FACT

!-------- LGM monthly precipitation-rate anomalies (assumed to be
!         equal to the mean annual precipitation-rate anomaly) --------

do n=1, 12   ! month counter
do i=0, IMAX
do j=0, JMAX

   precip_lgm_anom(j,i,n) = max(precip_ma_lgm_anom(j,i), eps)
                            ! positive values ensured

#if (PRECIP_ANOM_INTERPOL==1)
   gamma_precip_lgm_anom(j,i,n) = 0.0d0   ! dummy values
#elif (PRECIP_ANOM_INTERPOL==2)
   gamma_precip_lgm_anom(j,i,n) = -log(precip_lgm_anom(j,i,n))
#else
   stop ' Wrong value of switch PRECIP_ANOM_INTERPOL!'
#endif

end do
end do
end do

#endif

!-------- Mean accumulation --------

mean_accum = MEAN_ACCUM*(1.0d-03/YEAR_SEC)*(RHO_W/RHO)
                       ! mm/a water equiv. -> m/s ice equiv.

mean_accum_inv = 1.0d0/mean_accum

!-------- Read present topography mask --------

#if (GRID==0 || GRID==1)

open(24, iostat=ios, &
     file=INPATH//'/ant/'//MASK_PRESENT_FILE, &
     status='old')

#elif GRID==2

stop ' GRID==2 not allowed for this application!'

#endif

if (ios /= 0) stop ' Error when opening the mask file!'

do n=1, 6; read(24,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(24,2300) (maske_help(j,i), i=0,IMAX)
end do

close(24, status='keep')

2300 format(IMAX(i1),i1)

!-------- Present reference elevation (for precipitation data) --------

#if (GRID==0 || GRID==1)

open(21, iostat=ios, &
     file=INPATH//'/ant/'//ZS_PRESENT_FILE, &
     recl=8192, status='old')

#elif GRID==2

stop ' GRID==2 not allowed for this application!'

#endif

if (ios /= 0) stop ' Error when opening the zs file!'

do n=1, 6; read(21,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(21,*) (zs_ref(j,i), i=0,IMAX)
end do

close(21, status='keep')

!  ------ Reset bathymetry data (sea floor elevation) to the
!         sea surface

do i=0, IMAX
do j=0, JMAX
   if (maske_help(j,i) >= 2) zs_ref(j,i) = 0.0d0
end do
end do

!-------- Reading of LGM mean-annual and mean-July surface-temperature 
!         anomalies --------

#if TSURFACE==5

#if (GRID==0 || GRID==1)

open(21, iostat=ios, &
     file=INPATH//'/ant/'//TEMP_MA_ANOM_FILE, &
     recl=8192, status='old')

#endif

if (ios /= 0) stop ' Error when opening the temp_ma anomaly file!'

#if (GRID==0 || GRID==1)

open(22, iostat=ios, &
     file=INPATH//'/ant/'//TEMP_MJ_ANOM_FILE, &
     recl=8192, status='old')

#endif

if (ios /= 0) stop ' Error when opening the temp_mj anomaly file!'

do n=1, 6; read(21,'(a)') ch_dummy; end do
do n=1, 6; read(22,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(21,*) (temp_ma_lgm_anom(j,i), i=0,IMAX)
   read(22,*) (temp_mj_lgm_anom(j,i), i=0,IMAX)
end do

close(21, status='keep')
close(22, status='keep')

temp_ma_lgm_anom = temp_ma_lgm_anom * TEMP_MA_ANOM_FACT
temp_mj_lgm_anom = temp_mj_lgm_anom * TEMP_MJ_ANOM_FACT

#endif

!-------- Read data for delta_ts --------

griptemp = 0.0d0   ! Assignment of dummy values

#if TSURFACE==4

open(21, iostat=ios, &
     file=INPATH//'/general/'//GRIP_TEMP_FILE, &
     status='old')
if (ios /= 0) then
   write(6,1000) 'Error when opening the data file for delta_ts!'
   stop
end if

read(21,*) ch_dummy, grip_time_min, grip_time_stp, grip_time_max

if (ch_dummy /= '#') then
   write(6,*) 'grip_time_min, grip_time_stp, grip_time_max not defined in'
   write(6,*) 'data file for delta_ts!'
end if

ndata_grip = (grip_time_max-grip_time_min)/grip_time_stp

if (ndata_grip.gt.10000) &
   stop 'Too many data in data file for delta_ts!'

do n=0, ndata_grip
   read(21,*) d_dummy, griptemp(n)
end do

close(21, status='keep')

#endif

!-------- Read data for the glacial index --------

#if TSURFACE==5

open(21, iostat=ios, &
     file=INPATH//'/general/'//GLAC_IND_FILE, status='old')
if (ios /= 0) stop ' Error when opening the glacial-index file!'

read(21,*) ch_dummy, gi_time_min, gi_time_stp, gi_time_max

if (ch_dummy /= '#') then
   write(6,*) 'gi_time_min, gi_time_stp, gi_time_max not defined in'
   write(6,*) 'glacial-index file!'
end if

ndata_gi = (gi_time_max-gi_time_min)/gi_time_stp

allocate(glacial_index(0:ndata_gi))

do n=0, ndata_gi
   read(21,*) d_dummy, glacial_index(n)
end do

close(21, status='keep')

#endif

!-------- Read data for z_sl --------

specmap_zsl = 0.0d0   ! Assignment of dummy values

#if SEA_LEVEL==3

open(21, iostat=ios, &
     file=INPATH//'/general/'//SEA_LEVEL_FILE, &
     status='old')
if (ios /= 0) then
   write(6,1000) 'Error when opening the data file for z_sl!'
   stop
end if

read(21,*) ch_dummy, specmap_time_min, specmap_time_stp, specmap_time_max

if (ch_dummy /= '#') then
   write(6,*) 'specmap_time_min, specmap_time_stp, specmap_time_max'
   write(6,*) 'not defined in data file for z_sl!'
end if

ndata_specmap = (specmap_time_max-specmap_time_min)/specmap_time_stp

if (ndata_specmap.gt.10000) &
   stop 'Too many data in data file for z_sl!'

do n=0, ndata_specmap
   read(21,*) d_dummy, specmap_zsl(n)
end do

close(21, status='keep')

#endif

!-------- Read computed sea-level equivalents for z_sl --------

#if SEA_LEVEL==4

stop ' SEA_LEVEL==4 not allowed for this application!'

#else

ndata_forcing = 0   ! Assignment of dummy values

do n=0, 10000
   z_sle_forcing(n) = 0.0d0
   time_forcing(n)  = 0.0d0
end do

#endif

!-------- Determination of the geothermal heat flux --------

#if Q_GEO_MOD==1

!  ------ Constant value

do i=0, IMAX
do j=0, JMAX
   q_geo(j,i) = Q_GEO *1.0d-03   ! mW/m2 -> W/m2
end do
end do

#elif Q_GEO_MOD==2

!  ------ Read data from file

open(21, iostat=ios, &
     file=INPATH//'/ant/'//Q_GEO_FILE, &
     recl=8192, status='old')

if (ios /= 0) stop 'Error when opening the qgeo file!'

do n=1, 6; read(21,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(21,*) (q_geo(j,i), i=0,IMAX)
end do

close(21, status='keep')

do i=0, IMAX
do j=0, JMAX
   q_geo(j,i) = q_geo(j,i) *1.0d-03   ! mW/m2 -> W/m2
end do
end do

#endif

!-------- Reading of tabulated kei function--------

#if (REBOUND==0 || REBOUND==1)

kei = 0.0d0; n_data_kei = 0; kei_r_max = 0.0d0; kei_r_incr = 0.0d0
                                                    ! dummy values
#elif REBOUND==2

call read_kei

#endif

!-------- Definition of initial values --------

!  ------ Present topography

#if ANF_DAT==1

call topography1(dxi, deta)
call boundary(time_init, dxi, deta, delta_ts, glac_index, griptemp, &
              specmap_zsl, z_sl, z_mar, &
              grip_time_min, grip_time_stp, grip_time_max, &
              specmap_time_min, specmap_time_stp, specmap_time_max, &
              z_sle_forcing, time_forcing, ndata_forcing)

dtt_2dxi  = 0.5d0*dtime_temp/dxi
dtt_2deta = 0.5d0*dtime_temp/deta

do i=0, IMAX
do j=0, JMAX

   Q_bm(j,i)    = 0.0d0
   Q_tld(j,i)   = 0.0d0
   Q_b_tot(j,i) = 0.0d0

   do kc=0, KCMAX
      temp_c(kc,j,i) = -10.0d0
      age_c(kc,j,i)  = 15000.0d0*YEAR_SEC
   end do

   do kt=0, KTMAX
      omega_t(kt,j,i) = 0.0d0
      age_t(kt,j,i)   = 15000.0d0*YEAR_SEC
   end do

   do kr=0, KRMAX
      temp_r(kr,j,i) = -10.0d0+q_geo(j,i)*H_R/KAPPA_R &
                       *(1.0d0-dble(kr)/dble(KRMAX))
!           (linear temperature distribution according to the
!            geothermal heat flux)
   end do

end do
end do

call calc_enhance(time_init)

!  ------ Ice-free, relaxed bedrock

#elif ANF_DAT==2

call topography2(dxi, deta)
call boundary(time_init, dxi, deta, delta_ts, glac_index, griptemp, &
              specmap_zsl, z_sl, z_mar, &
              grip_time_min, grip_time_stp, grip_time_max, &
              specmap_time_min, specmap_time_stp, specmap_time_max, &
              z_sle_forcing, time_forcing, ndata_forcing)

dtt_2dxi  = 0.5d0*dtime_temp/dxi
dtt_2deta = 0.5d0*dtime_temp/deta

do i=0, IMAX
do j=0, JMAX

   Q_bm(j,i)    = 0.0d0
   Q_tld(j,i)   = 0.0d0
   Q_b_tot(j,i) = 0.0d0

   do kc=0, KCMAX
      temp_c(kc,j,i) = temp_s(j,i)
      age_c(kc,j,i)  = 15000.0d0*YEAR_SEC
   end do

   do kt=0, KTMAX
      omega_t(kt,j,i) = 0.0d0
      age_t(kt,j,i)   = 15000.0d0*YEAR_SEC
   end do

   do kr=0, KRMAX
      temp_r(kr,j,i) = temp_s(j,i)+q_geo(j,i)*H_R/KAPPA_R &
                       *(1.0d0-dble(kr)/dble(KRMAX))
!           (linear temperature distribution according to the
!            geothermal heat flux)
   end do

end do
end do

call calc_enhance(time_init)

!  ------ Read initial state from output of previous simulation

#elif ANF_DAT==3

call topography3(dxi, deta, anfdatname)
call boundary(time_init, dxi, deta, delta_ts, glac_index, griptemp, &
              specmap_zsl, z_sl, z_mar, &
              grip_time_min, grip_time_stp, grip_time_max, &
              specmap_time_min, specmap_time_stp, specmap_time_max, &
              z_sle_forcing, time_forcing, ndata_forcing)

dtt_2dxi  = 0.5d0*dtime_temp/dxi
dtt_2deta = 0.5d0*dtime_temp/deta

Q_b_tot = Q_bm + Q_tld

call calc_enhance(time_init)

#endif

!-------- Distance between grid points with delta_i=ir, delta_j=jr --------

#if (GRID==0 || GRID==1)

do ir=-IMAX, IMAX
do jr=-JMAX, JMAX
   dist_dxdy(jr,ir) = sqrt( (dble(ir)*dxi)**2 + (dble(jr)*deta)**2 )
                  ! distortion due to stereographic projection not accounted for
end do
end do

#endif

!-------- General abbreviations --------

ea = exp(DEFORM)

do kc=0, KCMAX
   zeta_c(kc) = kc*dzeta_c
   eaz_c(kc)  = exp(DEFORM*zeta_c(kc))
   eaz_c_quotient(kc) = (eaz_c(kc)-1.0d0)/(ea-1.0d0)
end do

do kt=0, KTMAX
   zeta_t(kt) = kt*dzeta_t
end do

at7 = 2.0d0/RHO*dtime_temp

aw1 = dtime_temp/dzeta_t
aw2 = dtime_temp/dzeta_t
aw3 = dtime_temp/dzeta_t
aw4 = dtime_temp/dzeta_t
aw5 = NUE/RHO*dtime_temp/(dzeta_t**2)
aw7 = 2.0d0/(RHO*L)*dtime_temp
aw8 = BETA**2/(RHO*L) &
      *(kappa_val(0.0d0)-kappa_val(-1.0d0))*dtime_temp
aw9 = BETA/L*dtime_temp

ai3 = AGEDIFF*dtime_temp/(dzeta_t**2)
ai4 = DEFORM*dzeta_c/(ea-1.0d0)

atr1 = KAPPA_R/(RHO_C_R*H_R**2)*dtime_temp/(dzeta_r**2)

am1 = DEFORM*BETA*dzeta_c/(ea-1.0d0)
am2 = DEFORM*L*RHO*dzeta_c/(ea-1.0d0)

acb1 = (ea-1.0d0)/DEFORM/dzeta_c
acb2 = KAPPA_R/H_R/dzeta_r
acb3 = RHO*G
acb4 = RHO*G

alb1 = H_R/KAPPA_R*dzeta_r

azs2     = dtime/(dxi*dxi)
azs3     = dtime/(deta*deta)

aqbm1 = (ea-1.0d0)/(RHO*L*DEFORM*dzeta_c)
aqbm2 = KAPPA_R/(RHO*L*H_R*dzeta_r)
aqbm3 = G/L
aqbm4 = BETA/(RHO*L)
aqtld = dzeta_t/dtime_temp

do kc=0, KCMAX
   avxy3(kc) = DEFORM*eaz_c(kc)/(ea-1.0d0)*dzeta_c

   avz3(kc)  = DEFORM*eaz_c(kc)/(ea-1.0d0)*dzeta_c

   aqxy1(kc) = DEFORM/(ea-1.0d0)*eaz_c(kc)*dzeta_c

   at1(kc)   = (ea-1.0d0)/(DEFORM*eaz_c(kc))*dtime_temp/dzeta_c
   at2_1(kc) = (ea-1.0d0)/(DEFORM*eaz_c(kc))*dtime_temp/dzeta_c
   at2_2(kc) = (eaz_c(kc)-1.0d0)/(DEFORM*eaz_c(kc)) &
               *dtime_temp/dzeta_c
   at3_1(kc) = (ea-1.0d0)/(DEFORM*eaz_c(kc))*dtime_temp/dzeta_c
   at3_2(kc) = (eaz_c(kc)-1.0d0)/(DEFORM*eaz_c(kc)) &
               *dtime_temp/dzeta_c
   at4_1(kc) = (ea-1.0d0)/(DEFORM*eaz_c(kc))*dtime_temp/dzeta_c
   at4_2(kc) = (eaz_c(kc)-1.0d0)/(DEFORM*eaz_c(kc)) &
               *dtime_temp/dzeta_c
   at5(kc)   = (ea-1.0d0)/(RHO*DEFORM*eaz_c(kc)) &
               *dtime_temp/dzeta_c
   if (kc /= KCMAX) then
      at6(kc) = (ea-1.0d0)/(DEFORM &
         *exp(DEFORM*0.5d0*(zeta_c(kc)+zeta_c(kc+1))))/dzeta_c
   else
      at6(kc) = 0.0d0
   end if

   ai1(kc) = AGEDIFF*(ea-1.0d0)/(DEFORM*eaz_c(kc)) &
             *dtime_temp/dzeta_c
   if (kc /= KCMAX) then
      ai2(kc) = (ea-1.0d0)/(DEFORM &
         *exp(DEFORM*0.5d0*(zeta_c(kc)+zeta_c(kc+1))))/dzeta_c
   else
      ai2(kc) = 0.0d0
   end if

   atm1(kc) = BETA*(1.0d0-eaz_c_quotient(kc))

end do

do kt=0, KTMAX
   atm2(kt) = 1.0d0-zeta_t(kt)
end do

!-------- Initial velocities --------

call calc_temp_melt(atm1, atm2)

call calc_vxy_b(z_sl)
call calc_vxy(avxy3, aqxy1, dzeta_t)

call calc_vz(dxi, deta, dzeta_c, dzeta_t, avz3)

!-------- Initialize time-series files --------

!  ------ Time-series file for the ice sheet on the whole

open(12, iostat=ios, &
     file=OUTPATH//'/'//trim(runname)//'.ser', &
     status='new')
if (ios /= 0) then
   write(6,1000) 'Error when opening the ser file!'
   stop
end if

if (forcing_flag == 1) then

   write(12,1102)
   write(12,1103)

   1102 format('        t(ka)  D_Ts(deg C)      z_sl(m)',/, &
               '                 H_max(km)   zs_max(km) V_g(10^6km3)', &
               ' V_t(10^3km3)  V_fw(km3/a)     z_sle(m)',/, &
               '              Aib(10^6km2) Atb(10^6km2)  V_bm(km3/a)', &
               ' V_tld(km3/a)   H_t_max(m)  vs_max(m/a)')
   1103 format('----------------------------------------------------', &
               '---------------------------------------')

else if (forcing_flag == 2) then

   write(12,1112)
   write(12,1113)

   1112 format('        t(ka)  glac_ind(1)      z_sl(m)',/, &
               '                 H_max(km)   zs_max(km) V_g(10^6km3)', &
               ' V_t(10^3km3)  V_fw(km3/a)     z_sle(m)',/, &
               '              Aib(10^6km2) Atb(10^6km2)  V_bm(km3/a)', &
               ' V_tld(km3/a)   H_t_max(m)  vs_max(m/a)')
   1113 format('----------------------------------------------------', &
               '---------------------------------------')

end if

!  ------ Time-series file for the ice-thickness field

#if OUTSER==2

open(13, iostat=ios, &
     file=OUTPATH//'/'//trim(runname)//'.thk', &
     status='new')
if (ios /= 0) stop 'Error when opening the thk file!'

if (forcing_flag == 1) then

 write(13,1104)
 write(13,1105)

 1104 format('t(ka)  D_Ts(deg C)  z_sl(m)',/, &
       'H(km): JMAX+1 records [j = JMAX (-1) 0] with',/, &
       '       IMAX+1 values [i = 0 (1) IMAX] in each record')
 1105 format('----------------------------------------------------')

else if (forcing_flag == 2) then

 write(13,1114)
 write(13,1115)

 1114 format('t(ka)  glac_ind(1)  z_sl(m)',/, &
       'H(km): JMAX+1 records [j = JMAX (-1) 0] with',/, &
       '       IMAX+1 values [i = 0 (1) IMAX] in each record')
 1115 format('----------------------------------------------------')

end if

#endif

!  ------ Time-series file for deep boreholes

#if OUTSER==3

n_core = 6   ! Vostok, Dome A, Dome C, Dome F, Kohnen, Byrd

allocate(lambda_core(n_core), phi_core(n_core), &
         x_core(n_core), y_core(n_core))

phi_core(1)    = -78.467d0 *pi_180    ! Geographical position of Vostok
lambda_core(1) = 106.8d0   *pi_180    ! at 78 28' S, 106 48' E,
                                      ! conversion deg -> rad
call num_coord(lambda_core(1), phi_core(1), x_core(1), y_core(1))
 
phi_core(2)    = -80.367d0 *pi_180    ! Geographical position of Dome A
lambda_core(2) =  77.35d0  *pi_180    ! at 80 22' S, 77 21' E,
                                      ! conversion deg -> rad
call num_coord(lambda_core(2), phi_core(2), x_core(2), y_core(2))

phi_core(3)    = -75.1d0 *pi_180      ! Geographical position of Dome C
lambda_core(3) = 123.4d0 *pi_180      ! at 75 06' S, 123 24' E,
                                      ! conversion deg -> rad
call num_coord(lambda_core(3), phi_core(3), x_core(3), y_core(3))

phi_core(4)    = -77.317d0 *pi_180    ! Geographical position of Dome F
lambda_core(4) =  39.7d0   *pi_180    ! at 77 19' S, 39 42' E,
                                      ! conversion deg -> rad
call num_coord(lambda_core(4), phi_core(4), x_core(4), y_core(4))

phi_core(5)    = -75.0d0   *pi_180    ! Geographical position of Kohnen
lambda_core(5) =   0.067d0 *pi_180    ! at 75 00' S, 00 04' E,
                                      ! conversion deg -> rad
call num_coord(lambda_core(5), phi_core(5), x_core(5), y_core(5))

phi_core(6)    =  -80.017d0 *pi_180   ! Geographical position of Byrd
lambda_core(6) = -119.517d0 *pi_180   ! at 80 01' S, 119 31' W,
                                      ! conversion deg -> rad
call num_coord(lambda_core(6), phi_core(6), x_core(6), y_core(6))

open(14, iostat=ios, &
     file=OUTPATH//'/'//trim(runname)//'.core', &
     status='new')
if (ios /= 0) stop 'Error when opening the core file!'

if (forcing_flag == 1) then

   write(14,1106)
   write(14,1107)

   1106 format('        t(ka)      D_Ts(C)      z_sl(m)',/, &
               '                 H_Vo (km)    H_DA (km)    H_DC (km)', &
               '    H_DF (km)    H_Ko (km)    H_By (km)',/, &
               '                v_Vo (m/a)   v_DA (m/a)   v_DC (m/a)', &
               '   v_DF (m/a)   v_Ko (m/a)   v_By (m/a)',/, &
               '                  T_Vo (C)     T_DA (C)     T_DC (C)', &
               '     T_DF (C)     T_Ko (C)     T_By (C)',/, &
               '              Rb_Vo (W/m2) Rb_DA (W/m2) Rb_DC (W/m2)', &
               ' Rb_DF (W/m2) Rb_Ko (W/m2) Rb_By (W/m2)')
   1107 format('----------------------------------------------------', &
               '---------------------------------------')

else if (forcing_flag == 2) then

   write(14,1116)
   write(14,1117)

   1116 format('        t(ka)  glac_ind(1)      z_sl(m)',/, &
               '                 H_Vo (km)    H_DA (km)    H_DC (km)', &
               '    H_DF (km)    H_Ko (km)    H_By (km)',/, &
               '                v_Vo (m/a)   v_DA (m/a)   v_DC (m/a)', &
               '   v_DF (m/a)   v_Ko (m/a)   v_By (m/a)',/, &
               '                  T_Vo (C)     T_DA (C)     T_DC (C)', &
               '     T_DF (C)     T_Ko (C)     T_By (C)',/, &
               '              Rb_Vo (W/m2) Rb_DA (W/m2) Rb_DC (W/m2)', &
               ' Rb_DF (W/m2) Rb_Ko (W/m2) Rb_By (W/m2)')
   1117 format('----------------------------------------------------', &
               '---------------------------------------')

end if

#endif

!-------- Output of the initial state --------

call output2(time_init, dxi, deta, delta_ts, glac_index, z_sl)
#if OUTSER==2
call output3(time_init, delta_ts, glac_index, z_sl)
#endif
#if OUTSER==3
call output4(time_init, dxi, deta, delta_ts, glac_index, z_sl)
#endif

end subroutine sico_init
