!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  b o u n d a r y

!   Purpose    :  Definition of the surface temperature (must be less
!                 than 0 deg C!) and of the accumulation-ablation
!                 function.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine boundary(time, dxi, deta, delta_ts, glac_index, griptemp, &
              specmap_zsl, z_sl, z_mar, &
              grip_time_min, grip_time_stp, grip_time_max, &
              specmap_time_min, specmap_time_stp, specmap_time_max, &
              z_sle_forcing, time_forcing, ndata_forcing)

use sico_variables

implicit none
integer i, j
integer mask_update
integer i_gr, i_kl
integer ndata_forcing
integer grip_time_min, grip_time_stp, grip_time_max, &
        specmap_time_min, specmap_time_stp, specmap_time_max
integer ndata_grip, ndata_specmap
double precision time, delta_ts, glac_index
double precision dxi, deta
double precision z_sl, z_sl_min, t1, t2, t3, t4, t5, t6, z_mar
double precision griptemp(0:10000), specmap_zsl(0:10000), time_gr, time_kl
double precision z_sle_present, z_sle_help, &
       z_sle_forcing(0:10000), time_forcing(0:10000)
double precision melt(0:JMAX,0:IMAX), melt_star(0:JMAX,0:IMAX)
double precision temp_ma(0:JMAX,0:IMAX), &
       temp_jja(0:JMAX,0:IMAX), temp_ampl(0:JMAX,0:IMAX)
double precision ET(0:JMAX,0:IMAX)
double precision gamma_t, temp_diff
double precision gamma_p, zs_thresh, temp_rain, temp_snow, precip_fact
double precision s_stat, beta1, beta2, Pmax, mu
double precision dist(0:JMAX,0:IMAX)
logical check_point(0:JMAX,0:IMAX)

!-------- Surface-temperature deviation from present values --------

#if TSURFACE==1
delta_ts = DELTA_TS0
!                           ! Steady state with prescribed constant
!                           ! air-temperature deviation
#elif TSURFACE==3
delta_ts = SINE_AMPLIT &
           *cos(2.0d0*pi*time/(SINE_PERIOD*YEAR_SEC)) &
           -SINE_AMPLIT
!                           ! Sinusoidal air-temperature forcing
#elif TSURFACE==4

!  ------ delta_ts from the GRIP record

ndata_grip = (grip_time_max-grip_time_min)/grip_time_stp

if (time/YEAR_SEC.lt.dble(grip_time_min)) then
   delta_ts = griptemp(0)
else if (time/YEAR_SEC.lt.dble(grip_time_max)) then

   i_kl = floor(((time/YEAR_SEC)-dble(grip_time_min))/dble(grip_time_stp))
   i_kl = max(i_kl, 0)

   i_gr = ceiling(((time/YEAR_SEC)-dble(grip_time_min))/dble(grip_time_stp))
   i_gr = min(i_gr, ndata_grip)

   if (i_kl.eq.i_gr) then

      delta_ts = griptemp(i_kl)

   else

      time_kl = (grip_time_min + i_kl*grip_time_stp) *YEAR_SEC
      time_gr = (grip_time_min + i_gr*grip_time_stp) *YEAR_SEC

      delta_ts = griptemp(i_kl) &
                +(griptemp(i_gr)-griptemp(i_kl)) &
                *(time-time_kl)/(time_gr-time_kl)
                 ! linear interpolation of the ice-core data

   end if

else
   delta_ts  = griptemp(ndata_grip)
end if

delta_ts = delta_ts * GRIP_TEMP_FACT
!                ! modification by constant factor

#elif TSURFACE==5
!     Enter here delta_ts scenario:

#endif

!-------- Sea level --------

#if SEA_LEVEL==1
!  ------ constant sea level
z_sl = Z_SL0

#elif SEA_LEVEL==2
!  ------ saw-tooth-shaped palaeoclimatic sea-level forcing

z_sl_min = -130.0d0

t1 = -250000.0d0 *YEAR_SEC
t2 = -140000.0d0 *YEAR_SEC
t3 = -125000.0d0 *YEAR_SEC
t4 =  -21000.0d0 *YEAR_SEC
t5 =   -8000.0d0 *YEAR_SEC
t6 =       0.0d0 *YEAR_SEC

if (time.lt.t1) then
   z_sl = 0.0d0
else if (time.lt.t2) then
   z_sl = z_sl_min*(time-t1)/(t2-t1)
else if (time.lt.t3) then
   z_sl = -z_sl_min*(time-t3)/(t3-t2)
else if (time.lt.t4) then
   z_sl = z_sl_min*(time-t3)/(t4-t3)
else if (time.lt.t5) then
   z_sl = -z_sl_min*(time-t5)/(t5-t4)
else if (time.lt.t6) then
   z_sl = 0.0d0
else
   z_sl = 0.0d0
end if

#elif SEA_LEVEL==3

!  ------ z_sl from the SPECMAP record

ndata_specmap = (specmap_time_max-specmap_time_min)/specmap_time_stp

if (time/YEAR_SEC.lt.dble(specmap_time_min)) then
   z_sl = specmap_zsl(0)
else if (time/YEAR_SEC.lt.dble(specmap_time_max)) then

   i_kl = floor(((time/YEAR_SEC)-dble(specmap_time_min))/dble(specmap_time_stp))
   i_kl = max(i_kl, 0)

   i_gr = ceiling(((time/YEAR_SEC)-dble(specmap_time_min))/dble(specmap_time_stp))
   i_gr = min(i_gr, ndata_specmap)

   if (i_kl.eq.i_gr) then

      z_sl = specmap_zsl(i_kl)

   else

      time_kl = (specmap_time_min + i_kl*specmap_time_stp) *YEAR_SEC
      time_gr = (specmap_time_min + i_gr*specmap_time_stp) *YEAR_SEC

      z_sl = specmap_zsl(i_kl) &
                +(specmap_zsl(i_gr)-specmap_zsl(i_kl)) &
                *(time-time_kl)/(time_gr-time_kl)
                 ! linear interpolation of the sea-level data

   end if

else
   z_sl  = specmap_zsl(ndata_specmap)
end if

#elif SEA_LEVEL==4

stop ' SEA_LEVEL==4 not allowed for this application!'

#endif

!  ------ Minimum bedrock elevation for extent of marine ice

#if MARGIN==2
z_mar = Z_MAR
#elif MARGIN==3
z_mar = FACT_Z_MAR*z_sl
#endif

!  ------ Update of the mask according to the sea level

!    ---- Check all sea and floating-ice points and their direct
!         neighbours

do i=0, IMAX
do j=0, JMAX
   check_point(j,i) = .false.
end do
end do

do i=1, IMAX-1
do j=1, JMAX-1
   if (maske(j,i).ge.2) then
      check_point(j  ,i  ) = .true.
      check_point(j  ,i+1) = .true.
      check_point(j  ,i-1) = .true.
      check_point(j+1,i  ) = .true.
      check_point(j-1,i  ) = .true.
   end if
end do
end do

do i=1, IMAX-1
do j=1, JMAX-1
   if (check_point(j,i)) then
      maske_neu(j,i) = mask_update(z_sl, i, j)
   end if
end do
end do

!    ---- Assign new values of the mask

do i=1, IMAX-1
do j=1, JMAX-1
   if (check_point(j,i)) then
      maske(j,i) = maske_neu(j,i)
   end if
end do
end do

!-------- Surface air temperature and accumulation-ablation function --------

do i=0, IMAX
do j=0, JMAX
   dist(j,i) = sqrt( (xi(i)-x_hat)**2 + (eta(j)-y_hat)**2 )
end do
end do

do i=0, IMAX
do j=0, JMAX

   temp_s(j,i) = temp_min + s_t*dist(j,i)
   temp_s(j,i) = temp_s(j,i) + delta_ts
                 ! Correction with temperature deviation delta_ts
   if (temp_s(j,i).gt. -0.001d0) temp_s(j,i) = -0.001d0
                               ! Cut-off of positive air temperatures

   accum_present(j,i) = b_max
   accum(j,i)         = accum_present(j,i)

   as_perp(j,i) = s_b*(eld-dist(j,i))
   if (as_perp(j,i).gt.accum(j,i)) as_perp(j,i) = accum(j,i)

end do
end do

return
end
