!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  t o p o g r a p h y 2

!   Purpose    :  Definition of the initial surface and bedrock
!                 topography (including gradients) and of the
!                 horizontal grid spacings dxi, deta.
!                 For present ice-free initial topography with
!                 relaxed bedrock.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine topography2(dxi, deta)

use sico_variables

implicit none
integer i, j
integer ios, nmuell
double precision dxi, deta
double precision dmuell
double precision xi0, eta0
character chmuell

!-------- Set topography --------

zb0 = 0.0d0

maske = 1

maske(0,:)    = 2
maske(JMAX,:) = 2

maske(:,0)    = 2
maske(:,IMAX) = 2


!-------- Further stuff --------

#if (GRID==0 || GRID==1)

dxi  = DX *1000.0d0   ! km -> m
deta = DX *1000.0d0   ! km -> m

xi0  = X0 *1000.0d0   ! km -> m
eta0 = Y0 *1000.0d0   ! km -> m

#elif GRID==2

stop ' GRID==2 not allowed for this application!'

#endif

do i=0, IMAX
do j=0, JMAX

   zs(j,i) = zb0(j,i)
   zb(j,i) = zb0(j,i)
   if (maske(j,i).eq.0) maske(j,i) = 1
   if (maske(j,i).eq.3) maske(j,i) = 2

!   zs(j,i) = zs(j,i)*1000.0d0
!   zb(j,i) = zb(j,i)*1000.0d0    ! km --> m
!   zb0(j,i) = zb0(j,i)*1000.0d0

   xi(i)  = xi0+dble(i)*dxi
   eta(j) = eta0+dble(j)*deta

   call geo_coord(phi(j,i), lambda(j,i), xi(i), eta(j))

   zm(j,i) = zb0(j,i)
   n_cts(j,i) = -1

   H_c(j,i) = 0.0d0
   H_t(j,i) = 0.0d0

   dzs_dtau(j,i)  = 0.0d0
   dzm_dtau(j,i)  = 0.0d0
   dzb_dtau(j,i)  = 0.0d0
   dH_c_dtau(j,i) = 0.0d0
   dH_t_dtau(j,i) = 0.0d0

end do
end do

!-------- Metric tensor, gradients of the topography --------

call metric

#if TOPOGRAD==0
call topograd_1(dxi, deta, 1)
#elif TOPOGRAD==1
call topograd_2(dxi, deta, 1)
#endif

 1000 format(a)

!-------- Corresponding area of grid points --------

do i=0, IMAX
do j=0, JMAX
   area(j,i) = sq_g11_g(j,i)*sq_g22_g(j,i)*dxi*deta
end do
end do

return
end
