!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Function :  c _ v a l

!   Purpose  :  Specific heat of ice:
!               Linear interpolation of tabulated values in C(.).

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

double precision function c_val(temp_val)

use sico_variables

implicit none
double precision, intent(in) :: temp_val

double precision :: c_ice

!-------- Specific heat of pure ice --------

#if defined(FRAC_DUST)
c_ice = C(floor(temp_val)) &
        +(C(floor(temp_val)+1)-C(floor(temp_val))) &
         *(temp_val-dble(floor(temp_val)))
#else
c_val = C(floor(temp_val)) &
        +(C(floor(temp_val)+1)-C(floor(temp_val))) &
         *(temp_val-dble(floor(temp_val)))
#endif

!-------- If dust is present (polar caps of Mars):
!         Specific heat of ice-dust mixture --------

#if defined(FRAC_DUST)
c_val = rho_inv * ( (1.0d0-FRAC_DUST)*RHO_I*c_ice + FRAC_DUST*RHO_C*C_C )
#endif

end function c_val
