!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ a g e _ c

!   Purpose    :  Computation of age for cold ice column.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_age_c(dtime_temp, i, j)

use sico_variables

implicit none
integer i, j, kc, kt
double precision g11_g22_g_inv(0:JMAX,0:IMAX)
double precision d_sq_g11_g22_g_dxi(0:JMAX,0:IMAX), &
                   d_sq_g11_g22_g_deta(0:JMAX,0:IMAX)
!-
double precision eaz_c_half(0:KCMAX)
double precision vx_c_g(0:KCMAX,0:JMAX,0:IMAX), &
                   vy_c_g(0:KCMAX,0:JMAX,0:IMAX), &
                   vz_c_g(0:KCMAX,0:JMAX,0:IMAX)
double precision vx_c_half(0:KCMAX,0:JMAX,0:IMAX), &
                   vy_c_half(0:KCMAX,0:JMAX,0:IMAX)
double precision age_c_half(0:KCMAX,0:JMAX,0:IMAX)
double precision T_c_mean(0:KCMAX,0:JMAX,0:IMAX), &
                   X_c_mean(0:KCMAX,0:JMAX,0:IMAX), &
                   Y_c_mean(0:KCMAX,0:JMAX,0:IMAX)
double precision T_c_mean_quarter(0:JMAX,0:IMAX), &
                   X_c_mean_quarter(0:JMAX,0:IMAX), &
                   Y_c_mean_quarter(0:JMAX,0:IMAX)
double precision T_c(0:KCMAX,0:JMAX,0:IMAX), &
                   X_c(0:KCMAX,0:JMAX,0:IMAX), &
                   Y_c(0:KCMAX,0:JMAX,0:IMAX)
double precision T_c_0(0:JMAX,0:IMAX), &
                   X_c_0(0:JMAX,0:IMAX), &
                   Y_c_0(0:JMAX,0:IMAX)
double precision vx_c_quarter(0:JMAX,0:IMAX), &
                   vy_c_quarter(0:JMAX,0:IMAX)
double precision age_c_quarter(0:JMAX,0:IMAX)
double precision adv_c_x(0:KCMAX,0:JMAX,0:IMAX), &
                   adv_c_y(0:KCMAX,0:JMAX,0:IMAX), &
                   adv_c_z(0:KCMAX,0:JMAX,0:IMAX)
double precision adv_c_z_0(0:JMAX,0:IMAX)
double precision mean_c_t(0:KCMAX,0:JMAX,0:IMAX), &
                   mean_c_f(0:KCMAX,0:JMAX,0:IMAX), &
                   mean_c_g(0:KCMAX,0:JMAX,0:IMAX), &
                   mean_c_h(0:KCMAX,0:JMAX,0:IMAX)
double precision mean_c_t_quarter(0:JMAX,0:IMAX), &
                   mean_c_f_quarter(0:JMAX,0:IMAX), &
                   mean_c_g_quarter(0:JMAX,0:IMAX), &
                   mean_c_h_quarter(0:JMAX,0:IMAX)
double precision add_t_c_1(0:KCMAX,0:JMAX,0:IMAX), &
                   add_t_c_2(0:KCMAX,0:JMAX,0:IMAX), &
                   add_t_c_3(0:KCMAX,0:JMAX,0:IMAX)
double precision add_t_c_1_0(0:JMAX,0:IMAX), &
                   add_t_c_2_0(0:JMAX,0:IMAX), &
                   add_t_c_3_0(0:JMAX,0:IMAX)
double precision eaz_c_quarter
!-
double precision dtime_temp, dxi, deta, dzeta_c, dtau
double precision, parameter :: zero=0.0d0

!-------- Check for boundary points --------

if ((i.eq.0).or.(i.eq.IMAX).or.(j.eq.0).or.(j.eq.JMAX)) &
   stop ' Boundary points not allowed in routine calc_temp1!'

!-------- Some definitions --------

dxi  = DX *1000.0d0
deta = DX *1000.0d0
dzeta_c = 1.0d0/dble(KCMAX)
dtau = dtime_temp

g11_g22_g_inv(j,i) =  1.0d0 / ( sq_g11_g(j,i)**2 * sq_g22_g(j,i)**2 )

d_sq_g11_g22_g_dxi(j,i)  = ( sq_g11_g(j,i+1) * sq_g22_g(j,i+1)        &
                             - sq_g11_g(j,i-1) * sq_g22_g(j,i-1) )    &
                           / ( 2.0d0 * dxi )

d_sq_g11_g22_g_deta(j,i) = ( sq_g11_g(j+1,i) * sq_g22_g(j+1,i)        &
                             - sq_g11_g(j-1,i) * sq_g22_g(j-1,i) )    &
                           / ( 2.0d0 * deta )

eaz_c_quarter = exp(DEFORM*(0.25d0)*dzeta_c)

!-------- Further definitions --------

do kc=0, KCMAX-1

vx_c_half(kc,j,i) = ( vz_c(kc,j,i) + vz_c(kc,j,i-1)                    &
                      + vz_c(kc+1,j,i) + vz_c(kc+1,j,i-1) ) * 0.25d0
vy_c_half(kc,j,i) = ( vz_c(kc,j,i) + vz_c(kc,j-1,i)                    &
                      + vz_c(kc+1,j,i) + vz_c(kc+1,j-1,i) ) * 0.25d0

age_c_half(kc,j,i) = ( age_c(kc,j,i) + age_c(kc+1,j,i) ) * 0.5d0
eaz_c_half(kc) = exp(DEFORM*(kc+0.5d0)*dzeta_c)

T_c(kc,j,i) = ( (ea-1.0d0)/(H_c(j,i)*DEFORM*eaz_c_half(kc))*dzm_dtau(j,i)&
              + (eaz_c_half(kc)-1.0d0)/(H_c(j,i)*DEFORM*eaz_c_half(kc))  &
                * dH_c_dtau(j,i) )
X_c(kc,j,i) = insq_g11_g(j,i)                                          &
            * ( (ea-1.0d0)/(H_c(j,i)*DEFORM*eaz_c_half(kc))*dzm_dxi(j,i) &
                + (eaz_c_half(kc)-1.0d0)/(H_c(j,i)*DEFORM*eaz_c_half(kc))&
                  * dH_c_dxi(j,i) )
Y_c(kc,j,i) = insq_g22_g(j,i)                                          &
            * ( (ea-1.0d0)/(H_c(j,i)*DEFORM*eaz_c_half(kc))*dzm_deta(j,i)&
                + (eaz_c_half(kc)-1.0d0)/(H_c(j,i)*DEFORM*eaz_c_half(kc))&
                  * dH_c_deta(j,i))

end do

!-------- vz at the grid point --------

   vz_c_g(0,j,i) = vz_c(0,j,i)
do kc=1, KCMAX
   vz_c_g(kc,j,i) = ( vz_c(kc,j,i) + vz_c(kc-1,j,i) ) * 0.5d0
end do

do kc=0, KCMAX

!-------- vx and vy at grid points --------

vx_c_g(kc,j,i) = ( vx_c(kc,j,i) + vx_c(kc,j,i-1) ) * 0.5d0
vy_c_g(kc,j,i) = ( vy_c(kc,j,i) + vy_c(kc,j-1,i) ) * 0.5d0

!-------- Abbreviations --------

T_c_mean(kc,j,i) = ( (ea-1.0d0)/(H_c(j,i)*eaz_c(kc))*dzm_dtau(j,i)       &
                   - 1.0d0/(H_c(j,i)*eaz_c(kc))*dH_c_dtau(j,i) )
X_c_mean(kc,j,i) = insq_g11_g(j,i)                                     &
                 * ( (ea-1.0d0)/(H_c(j,i)*eaz_c(kc))*dzm_dxi(j,i)        &
                     - 1.0d0/(H_c(j,i)*eaz_c(kc))*dH_c_dxi(j,i) )
Y_c_mean(kc,j,i) = insq_g22_g(j,i)                                     &
                 * ( (ea-1.0d0)/(H_c(j,i)*eaz_c(kc))*dzm_deta(j,i)       &
                     - 1.0d0/(H_c(j,i)*eaz_c(kc))*dH_c_deta(j,i) )

!-------- The terms of the mean values --------

mean_c_t(kc,j,i) = dtau * T_c_mean(kc,j,i) * age_c(kc,j,i)

mean_c_f(kc,j,i) = dtau * ( X_c_mean(kc,j,i)                           &
                         - d_sq_g11_g22_g_dxi(j,i)                     &
                           * sq_g22_g(j,i) * g11_g22_g_inv(j,i) )      & 
                 * age_c(kc,j,i) * vx_c_g(kc,j,i)

mean_c_g(kc,j,i) = dtau * ( Y_c_mean(kc,j,i)                           &
                         - d_sq_g11_g22_g_deta(j,i)                    &
                           * sq_g11_g(j,i) * g11_g22_g_inv(j,i) )      & 
                 * age_c(kc,j,i) * vy_c_g(kc,j,i)

mean_c_h(kc,j,i) = dtau * ( ea-1 ) / ( H_c(j,i) * eaz_c(kc) )          &
                 * age_c(kc,j,i) * vz_c_g(kc,j,i)

!-------- The advection terms --------

adv_c_x(kc,j,i) = dtau/(2.0d0*dxi)                                       &          
                 * (   age_c(kc,j,i)*insq_g11_g(j,i)                   &
                       * (abs(vx_c(kc,j,i))+vx_c(kc,j,i))              &
                     - age_c(kc,j,i+1)*insq_g11_g(j,i+1)               &
                       * (abs(vx_c(kc,j,i))-vx_c(kc,j,i))              &
                     - age_c(kc,j,i-1)*insq_g11_g(j,i-1)               &
                       * (abs(vx_c(kc,j,i-1))+vx_c(kc,j,i-1))          &
                     + age_c(kc,j,i)*insq_g11_g(j,i)                   &
                       * (abs(vx_c(kc,j,i-1))-vx_c(kc,j,i-1)) )

adv_c_y(kc,j,i) = dtau/(2.0d0*deta)                                      &
                 * (   age_c(kc,j,i)*insq_g22_g(j,i)                   &
                       * (abs(vy_c(kc,j,i))+vy_c(kc,j,i))              &
                     - age_c(kc,j+1,i)*insq_g22_g(j+1,i)               &
                       * (abs(vy_c(kc,j,i))-vy_c(kc,j,i))              &
                     - age_c(kc,j-1,i)*insq_g22_g(j-1,i)               &
                       * (abs(vy_c(kc,j-1,i))+vy_c(kc,j-1,i))          &
                     + age_c(kc,j,i)*insq_g22_g(j,i)                   &
                       * (abs(vy_c(kc,j-1,i))-vy_c(kc,j-1,i)) )

end do


do kc=1, KCMAX-1

!-------- The vertical advection term --------

adv_c_z(kc,j,i) = dtau/(2.0d0*dzeta_c) * (ea - 1.0d0)/(H_c(j,i)*DEFORM)    &
                 * (   age_c(kc,j,i)/eaz_c(kc)                         &
                       * (abs(vz_c(kc,j,i))+vz_c(kc,j,i))              &
                     - age_c(kc+1,j,i)/eaz_c(kc+1)                     &
                       * (abs(vz_c(kc,j,i))-vz_c(kc,j,i))              &
                     - age_c(kc-1,j,i)/eaz_c(kc-1)                     &
                       * (abs(vz_c(kc-1,j,i))+vz_c(kc-1,j,i))          &
                     + age_c(kc,j,i)/eaz_c(kc)                         &
                       * (abs(vz_c(kc-1,j,i))-vz_c(kc-1,j,i)) )

!-------- Additional terms --------

add_t_c_1(kc,j,i) = dtau/dzeta_c                                       &
                  * ( T_c(kc,j,i) * age_c_half(kc,j,i)                 &
                      - T_c(kc-1,j,i) * age_c_half(kc-1,j,i) )
add_t_c_2(kc,j,i) = dtau/dzeta_c                                       &
               * ( X_c(kc,j,i)*age_c_half(kc,j,i)*vx_c_half(kc,j,i)    &
                   - X_c(kc-1,j,i)*age_c_half(kc-1,j,i)                &
                     *vx_c_half(kc-1,j,i) )
add_t_c_3(kc,j,i) = dtau/dzeta_c                                       &
               * ( Y_c(kc,j,i)*age_c_half(kc,j,i)*vy_c_half(kc,j,i)    &
                   - Y_c(kc-1,j,i)*age_c_half(kc-1,j,i)                &
                     *vy_c_half(kc-1,j,i) )

end do


!-------- Abbreviations at kc=0 --------

T_c_0(j,i) = ( (ea-1.0d0)/(H_c(j,i)*DEFORM)*dzm_dtau(j,i) )
X_c_0(j,i) = insq_g11_g(j,i)                                           &
            * ( (ea-1.0d0)/(H_c(j,i)*DEFORM)*dzm_dxi(j,i) )
Y_c_0(j,i) = insq_g22_g(j,i)                                           &
            * ( (ea-1.0d0)/(H_c(j,i)*DEFORM)*dzm_deta(j,i) )

age_c_quarter(j,i) = (3.0d0*age_c(0,j,i)+age_c(1,j,i))/4.0d0

vx_c_quarter(j,i) = (3.0d0*vx_c_g(0,j,i)+vx_c_g(1,j,i))/4.0d0
vy_c_quarter(j,i) = (3.0d0*vy_c_g(0,j,i)+vy_c_g(1,j,i))/4.0d0

T_c_mean_quarter(j,i) = ( (ea-1.0d0)/(H_c(j,i)*eaz_c_quarter)*dzm_dtau(j,i)    &
                         - 1.0d0/(H_c(j,i)*eaz_c_quarter)*dH_c_dtau(j,i) )
X_c_mean_quarter(j,i) = insq_g11_g(j,i)                                      &
                      * ( (ea-1.0d0)/(H_c(j,i)*eaz_c_quarter)*dzm_dxi(j,i)     &
                         - 1.0d0/(H_c(j,i)*eaz_c_quarter)*dH_c_dxi(j,i) )
Y_c_mean_quarter(j,i) = insq_g22_g(j,i)                                      &
                      * ( (ea-1.0d0)/(H_c(j,i)*eaz_c_quarter)*dzm_deta(j,i)    &
                         - 1.0d0/(H_c(j,i)*eaz_c_quarter)*dH_c_deta(j,i) )

!-------- The vertical advection term at kc=  --------

adv_c_z_0(j,i) = (2.0d0*dtau)/dzeta_c * (ea - 1.0d0)/(H_c(j,i)*DEFORM)     &
                 * (   age_c(0,j,i)/2.0d0                                &
                       * (abs(vz_c(0,j,i))+vz_c(0,j,i))                &
                     - age_c(1,j,i)/(2.0d0*eaz_c(1))                     &
                       * (abs(vz_c(0,j,i))-vz_c(0,j,i))                &
                     - age_c(0,j,i)                                    &
                       * vz_c(0,j,i))

!-------- The terms of the mean values at kc=0 --------

mean_c_t_quarter(j,i) = dtau * T_c_mean_quarter(j,i) * age_c_quarter(j,i)

mean_c_f_quarter(j,i) = dtau * ( X_c_mean_quarter(j,i)                       &
                         - d_sq_g11_g22_g_dxi(j,i)                           &
                           * sq_g22_g(j,i) * g11_g22_g_inv(j,i) )            & 
                 * age_c_quarter(j,i) * vx_c_quarter(j,i)

mean_c_g_quarter(j,i) = dtau * ( Y_c_mean_quarter(j,i)                       &
                         - d_sq_g11_g22_g_deta(j,i)                          &
                           * sq_g11_g(j,i) * g11_g22_g_inv(j,i) )            & 
                 * age_c_quarter(j,i) * vy_c_quarter(j,i)

mean_c_h_quarter(j,i) = dtau * ( ea-1)/( H_c(j,i)*eaz_c_quarter )            &
                 * age_c_quarter(j,i) * vz_c_g(0,j,i)

!-------- Additional terms at kc=0 --------

add_t_c_1_0(j,i) = 2.0d0*dtau/dzeta_c                                    &
                 * ( T_c(0,j,i)*age_c_half(0,j,i)                      &
                     - T_c_0(j,i) * age_c(0,j,i) )
add_t_c_2_0(j,i) = 2.0d0*dtau/dzeta_c                                    &
                 * ( X_c(0,j,i)*age_c_half(0,j,i)*vx_c_half(0,j,i)     &
                     - X_c_0(j,i)*age_c(0,j,i)*vx_c_g(0,j,i) )
add_t_c_3_0(j,i) = 2.0d0*dtau/dzeta_c                                    &
                 * ( Y_c(0,j,i)*age_c_half(0,j,i)*vy_c_half(0,j,i)     &
                     - Y_c_0(j,i)*age_c(0,j,i)*vy_c_g(0,j,i) )

!-------- 

!-------- Solve the age equation at the free surface --------

kc=KCMAX

        age_c_neu(kc,j,i) = 0.0d0

!--------

do kc=1, KCMAX-1

        age_c_neu(kc,j,i) = age_c(kc,j,i) + dtau                              &
                            - adv_c_x(kc,j,i)                                 &
                            - adv_c_y(kc,j,i)                                 &
                            - adv_c_z(kc,j,i)                                 &
                            + mean_c_t(kc,j,i)                                &
                            + mean_c_f(kc,j,i)                                &
                            + mean_c_g(kc,j,i)                                &
                            - mean_c_h(kc,j,i)                                &
                            + add_t_c_1(kc,j,i)                               &
                            + add_t_c_2(kc,j,i)                               &
                            + add_t_c_3(kc,j,i)

end do

!-------- Solve the age equation at bottom layer --------

kc=0

        age_c_neu(0,j,i) = age_c(0,j,i) + dtau                                &
                           - adv_c_x(0,j,i)                                   &
                           - adv_c_y(0,j,i)                                   &
                           - adv_c_z_0(j,i)                                   &
                           + mean_c_t_quarter(j,i)                            &
                           + mean_c_f_quarter(j,i)                            &
                           + mean_c_g_quarter(j,i)                            &
                           - mean_c_h_quarter(j,i)                            &
                           + add_t_c_1_0(j,i)                                 &
                           + add_t_c_2_0(j,i)                                 &
                           + add_t_c_3_0(j,i)

!-------- Assign the result,
!         restriction to interval [0, AGE_MAX yr] --------

do kc=0, KCMAX

   if ( (maske_neu(j,i).eq.1).or.(maske_neu(j,i).eq.2) ) &
                            age_c_neu(kc,j,i) = 0.0d0
   if (age_c_neu(kc,j,i).lt.zero) &
                            age_c_neu(kc,j,i) = 0.0d0
   if (age_c_neu(kc,j,i).gt.(AGE_MAX*YEAR_SEC)) &
                            age_c_neu(kc,j,i) = AGE_MAX*YEAR_SEC

end do

!-------- Age of the ice in the non-existing temperate layer --------

do kt=0, KTMAX
   age_t_neu(kt,j,i) = age_c_neu(0,j,i)
end do

return
end
