!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ e n h a n c e

!   Purpose    :  Computation of the flow enhancement factor.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_enhance(time)

use sico_variables

implicit none
integer :: i, j, kc, kt
double precision :: time
double precision :: age_trans, date_trans1, date_trans2, date_trans3
double precision :: enh_mult

#if ENHMOD==1

do i=0, IMAX
do j=0, JMAX

   do kt=0, KTMAX
      enh_t(kt,j,i) = ENH
   end do

   do kc=0, KCMAX
      enh_c(kc,j,i) = ENH
   end do

end do
end do

#elif ENHMOD==2

age_trans = 11000.0d0*YEAR_SEC

do i=0, IMAX
do j=0, JMAX

   do kt=0, KTMAX
      if (age_t(kt,j,i).lt.age_trans) then
         enh_t(kt,j,i) = 1.0d0 ! Holocene ice
      else
         enh_t(kt,j,i) = ENH   ! pre-Holocene ice
      end if
   end do

   do kc=0, KCMAX
      if (age_c(kc,j,i).lt.age_trans) then
         enh_c(kc,j,i) = 1.0d0 ! Holocene ice
      else
         enh_c(kc,j,i) = ENH   ! Weichselian ice
      end if
   end do

end do
end do

#elif ENHMOD==3

date_trans1 = -132000.0d0*YEAR_SEC
date_trans2 = -114500.0d0*YEAR_SEC
date_trans3 =  -11000.0d0*YEAR_SEC

do i=0, IMAX
do j=0, JMAX

   do kt=0, KTMAX
      if ( (time-age_t(kt,j,i)).lt.date_trans1 ) then
         enh_t(kt,j,i) = ENH   ! pre-Eemian ice
      else if ( ((time-age_t(kt,j,i)).ge.date_trans1).and. &
                ((time-age_t(kt,j,i)).lt.date_trans2) ) then
         enh_t(kt,j,i) = 1.0d0 ! Eemian ice
      else if ( ((time-age_t(kt,j,i)).ge.date_trans2).and. &
                ((time-age_t(kt,j,i)).lt.date_trans3) ) then
         enh_t(kt,j,i) = ENH   ! Weichselian ice
      else
         enh_t(kt,j,i) = 1.0d0 ! Holocene ice
      end if
   end do

   do kc=0, KCMAX
      if ( (time-age_c(kc,j,i)).lt.date_trans1 ) then
         enh_c(kc,j,i) = ENH   ! pre-Eemian ice
      else if ( ((time-age_c(kc,j,i)).ge.date_trans1).and. &
                ((time-age_c(kc,j,i)).lt.date_trans2) ) then
         enh_c(kc,j,i) = 1.0d0 ! Eemian ice
      else if ( ((time-age_c(kc,j,i)).ge.date_trans2).and. &
                ((time-age_c(kc,j,i)).lt.date_trans3) ) then
         enh_c(kc,j,i) = ENH   ! Weichselian ice
      else
         enh_c(kc,j,i) = 1.0d0 ! Holocene ice
      end if
   end do

end do
end do

#endif

!-------- Modification due to dust content (Martian polar caps only) --------

#if ( defined(NMARS) \
      || defined(SMARS) )   /* Martian ice sheet */

#if (FLOW_LAW==1)
   enh_mult = exp(-3.0d0*2.0d0*FRAC_DUST)
#elif (FLOW_LAW==2)
   enh_mult = exp(-1.8d0*2.0d0*FRAC_DUST)
#elif (FLOW_LAW==3)
   enh_mult = exp(-4.0d0*2.0d0*FRAC_DUST)
#endif

enh_t = enh_t * enh_mult
enh_c = enh_c * enh_mult

#endif

end subroutine calc_enhance
