!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ q b m

!   Purpose    :  Computation of the basal melting rate Q_bm.
!                 Summation of Q_bm and Q_tld.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_qbm(aqbm1, aqbm2, aqbm3, aqbm4)

use sico_variables

implicit none
double precision, intent(in) :: aqbm1, aqbm2, aqbm3, aqbm4

integer :: i, j
double precision :: kappa_val

!-------- Computation of Q_bm --------

do i=1, IMAX-1
do j=1, JMAX-1

   if ( (maske(j,i) == 0).or.(maske(j,i) == 3) ) then
!                                ! glaciated land or floating ice --------

      if (n_cts(j,i) == -1) then
         Q_bm(j,i) = 0.0d0
      else if (n_cts(j,i) == 0) then
         Q_bm(j,i) = aqbm1*(temp_c(1,j,i)-temp_c(0,j,i))/H_c(j,i) &
                     *kappa_val(temp_c(0,j,i)) &
            -aqbm2*(temp_r(KRMAX,j,i)-temp_r(KRMAX-1,j,i)) &
            -aqbm3*H_c(j,i)*0.5d0*(vx_t(0,j,i)+vx_t(0,j,i-1)) &
                  *dzs_dxi_g(j,i) &
            -aqbm3*H_c(j,i)*0.5d0*(vy_t(0,j,i)+vy_t(0,j-1,i)) &
                  *dzs_deta_g(j,i)
      else   ! n_cts(j,i) == 1
         Q_bm(j,i) = aqbm4 &
                     *kappa_val(temp_t_m(0,j,i)) &
            -aqbm2*(temp_r(KRMAX,j,i)-temp_r(KRMAX-1,j,i)) &
            -aqbm3*(H_c(j,i)+H_t(j,i)) &
                  *0.5d0*(vx_t(0,j,i)+vx_t(0,j,i-1)) &
                  *dzs_dxi_g(j,i) &
            -aqbm3*(H_c(j,i)+H_t(j,i)) &
                  *0.5d0*(vy_t(0,j,i)+vy_t(0,j-1,i)) &
                  *dzs_deta_g(j,i)
      end if

   else   ! maske(j,i) == (1 or 2)
      Q_bm(j,i) = 0.0d0
   end if

end do
end do

do i=0, IMAX
   Q_bm(0,i)    = 0.0d0
   Q_bm(JMAX,i) = 0.0d0
end do

do j=0, JMAX
   Q_bm(j,0)    = 0.0d0
   Q_bm(j,IMAX) = 0.0d0
end do

!-------- Sum of Q_bm and Q_tld --------

do i=0, IMAX
do j=0, JMAX
   Q_b_tot(j,i) = Q_bm(j,i)+Q_tld(j,i)
end do
end do

!-------- Limitation of Q_bm, Q_tld and Q_b_tot --------

do i=0, IMAX
do j=0, JMAX
   if (Q_bm(j,i)    < QB_MIN) Q_bm(j,i)    = 0.0d0
   if (Q_bm(j,i)    > QB_MAX) Q_bm(j,i)    = QB_MAX
   if (Q_tld(j,i)   < QB_MIN) Q_tld(j,i)   = 0.0d0
   if (Q_tld(j,i)   > QB_MAX) Q_tld(j,i)   = QB_MAX
   if (Q_b_tot(j,i) < QB_MIN) Q_b_tot(j,i) = 0.0d0
   if (Q_b_tot(j,i) > QB_MAX) Q_b_tot(j,i) = QB_MAX
end do
end do

end subroutine calc_qbm
