!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ t e m p 1

!   Purpose    :  Computation of temperature and age for cold ice
!                 column.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_temp1(at1, at2_1, at2_2, at3_1, at3_2, &
   at4_1, at4_2, at5, at6, at7, atr1, acb1, acb2, &
   acb3, acb4, alb1, ai1, ai2, ai4, &
   mean_accum_inv, &
   dtime_temp, dtt_2dxi, dtt_2deta, i, j)

use sico_variables

implicit none
integer i, j, kc, kt, kr
double precision at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), at7, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai4, &
       atr1, acb1, acb2, acb3, acb4, alb1
double precision ct1(0:KCMAX), ct2(0:KCMAX), ct3(0:KCMAX), ct4(0:KCMAX), &
       ct5(0:KCMAX), ct6(0:KCMAX), ct7(0:KCMAX), ctr1, &
       ccb1, ccb2, ccb3, ccb4, clb1
double precision ct1_sg(0:KCMAX), ct2_sg(0:KCMAX), ct3_sg(0:KCMAX), &
       ct4_sg(0:KCMAX), adv_vert_sg(0:KCMAX), abs_adv_vert_sg(0:KCMAX)
double precision ci1(0:KCMAX), ci2(0:KCMAX)
double precision ftx_c_l(0:KCMAX), ftx_c_r(0:KCMAX), &
                 fty_c_l(0:KCMAX), fty_c_r(0:KCMAX), &
                 fax_c_l(0:KCMAX), fax_c_r(0:KCMAX), &
                 fay_c_l(0:KCMAX), fay_c_r(0:KCMAX)
double precision temp_c_help(0:KCMAX)
double precision mean_accum_inv
double precision dtt_2dxi, dtt_2deta, dtime_temp
double precision dtt_dxi, dtt_deta
double precision ratefac, creep, kappa_val, c_val
double precision lgs_a0(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_a1(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_a2(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_x(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_b(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX)
double precision, parameter :: zero=0.0d0

!-------- Check for boundary points --------

if ((i.eq.0).or.(i.eq.IMAX).or.(j.eq.0).or.(j.eq.JMAX)) &
   stop ' Boundary points not allowed in routine calc_temp1!'

!-------- Abbreviations --------

ctr1 = atr1

ccb1 = acb1 &
   *kappa_val(temp_c(0,j,i)) &
   /H_c(j,i)
ccb2 = acb2
ccb3 = acb3*0.5d0*(vx_t(0,j,i)+vx_t(0,j,i-1)) &
       *H_c(j,i)*dzs_dxi_g(j,i)
ccb4 = acb4*0.5d0*(vy_t(0,j,i)+vy_t(0,j-1,i)) &
       *H_c(j,i)*dzs_deta_g(j,i)

clb1 = alb1*q_geo(j,i)

#if ADV_VERT==1

do kc=1, KCMAX
   ct1(kc) = at1(kc)/H_c(j,i)*0.5d0*(vz_c(kc,j,i)+vz_c(kc-1,j,i))
end do

#elif (ADV_VERT==2 || ADV_VERT==3)

do kc=0, KCMAX-1
   ct1_sg(kc) = 0.5d0*(at1(kc)+at1(kc+1))/H_c(j,i)*vz_c(kc,j,i)
end do

#endif

do kc=0, KCMAX

   ct2(kc) = ( at2_1(kc)*dzm_dtau(j,i) &
           +at2_2(kc)*dH_c_dtau(j,i) )/H_c(j,i)
   ct3(kc) = ( at3_1(kc)*dzm_dxi_g(j,i) &
           +at3_2(kc)*dH_c_dxi_g(j,i) )/H_c(j,i) &
          *0.5d0*(vx_c(kc,j,i)+vx_c(kc,j,i-1)) *insq_g11_g(j,i) ! new factor !!!
   ct4(kc) = ( at4_1(kc)*dzm_deta_g(j,i) &
            +at4_2(kc)*dH_c_deta_g(j,i) )/H_c(j,i) &
          *0.5d0*(vy_c(kc,j,i)+vy_c(kc,j-1,i)) *insq_g22_g(j,i) ! new factor !!!
   ct5(kc) = at5(kc) &
             /c_val(temp_c(kc,j,i)) &
             /H_c(j,i)
   ct7(kc) = at7 &
             /c_val(temp_c(kc,j,i)) &
             *enh_c(kc,j,i) &
             *ratefac(temp_c(kc,j,i), temp_c_m(kc,j,i)) &
             *creep(sigma_c(kc,j,i)) &
             *sigma_c(kc,j,i)*sigma_c(kc,j,i)
   ci1(kc) = ai1(kc)/H_c(j,i)

!  ------ Fluxes for horizontal advection of temperature and age

#if ADV_HOR==4
   if (vx_c(kc,j,i-1).ge.zero) then
      ftx_c_l(kc) = temp_c(kc,j,i-1)*vx_c(kc,j,i-1)
      fax_c_l(kc) =  age_c(kc,j,i-1)*vx_c(kc,j,i-1)
   else
      ftx_c_l(kc) = temp_c(kc,j,i)*vx_c(kc,j,i-1)
      fax_c_l(kc) =  age_c(kc,j,i)*vx_c(kc,j,i-1)
   end if

   if (vx_c(kc,j,i).ge.zero) then
      ftx_c_r(kc) = temp_c(kc,j,i)*vx_c(kc,j,i)
      fax_c_r(kc) =  age_c(kc,j,i)*vx_c(kc,j,i)
   else
      ftx_c_r(kc) = temp_c(kc,j,i+1)*vx_c(kc,j,i)
      fax_c_r(kc) =  age_c(kc,j,i+1)*vx_c(kc,j,i)
   end if

   if (vy_c(kc,j-1,i).ge.zero) then
      fty_c_l(kc) = temp_c(kc,j-1,i)*vy_c(kc,j-1,i)
      fay_c_l(kc) =  age_c(kc,j-1,i)*vy_c(kc,j-1,i)
   else
      fty_c_l(kc) = temp_c(kc,j,i)*vy_c(kc,j-1,i)
      fay_c_l(kc) =  age_c(kc,j,i)*vy_c(kc,j-1,i)
   end if

   if (vy_c(kc,j,i).ge.zero) then
      fty_c_r(kc) = temp_c(kc,j,i)*vy_c(kc,j,i)
      fay_c_r(kc) =  age_c(kc,j,i)*vy_c(kc,j,i)
   else
      fty_c_r(kc) = temp_c(kc,j+1,i)*vy_c(kc,j,i)
      fay_c_r(kc) =  age_c(kc,j+1,i)*vy_c(kc,j,i)
   end if
#endif

end do

#if (ADV_VERT==2 || ADV_VERT==3)

do kc=0, KCMAX-1
   ct2_sg(kc) = 0.5d0*(ct2(kc)+ct2(kc+1))
   ct3_sg(kc) = 0.5d0*(ct3(kc)+ct3(kc+1))
   ct4_sg(kc) = 0.5d0*(ct4(kc)+ct4(kc+1))
   adv_vert_sg(kc) = ct1_sg(kc)-ct2_sg(kc)-ct3_sg(kc)-ct4_sg(kc)
   abs_adv_vert_sg(kc) = abs(adv_vert_sg(kc))
end do

#endif

do kc=0, KCMAX-1
   temp_c_help(kc) = 0.5d0*(temp_c(kc,j,i)+temp_c(kc+1,j,i))
   ct6(kc) = at6(kc) &
    *kappa_val(temp_c_help(kc)) &
    /H_c(j,i)
   ci2(kc) = ai2(kc)/H_c(j,i)
end do

#if ADV_HOR==4
dtt_dxi  = 2.0d0*dtt_2dxi
dtt_deta = 2.0d0*dtt_2deta
#endif

!-------- Set up the temperature equations (ice and bedrock
!         simultaneously) --------

kr=0
lgs_a1(kr) = 1.0d0
lgs_a2(kr) = -1.0d0
lgs_b(kr)    = clb1

#if Q_LITHO==1
!   (coupled heat-conducting bedrock)

do kr=1, KRMAX-1
   lgs_a0(kr) = - ctr1
   lgs_a1(kr) = 1.0d0 + 2.0d0*ctr1
   lgs_a2(kr) = - ctr1
   lgs_b(kr)    = temp_r(kr,j,i)
end do

#elif Q_LITHO==0
!   (no coupled heat-conducting bedrock)

do kr=1, KRMAX-1
   lgs_a0(kr) = 1.0d0
   lgs_a1(kr) = 0.0d0
   lgs_a2(kr) = -1.0d0
   lgs_b(kr)  = 2.0d0*clb1
end do

#endif

kr=KRMAX
kc=0
lgs_a0(kr) = ccb2
lgs_a1(kr) = -(ccb1+ccb2)
lgs_a2(kr) = ccb1
lgs_b(kr)  = ccb3+ccb4

do kc=1, KCMAX-1

#if ADV_VERT==1

   lgs_a0(KRMAX+kc) = -0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ct5(kc)*ct6(kc-1)
   lgs_a1(KRMAX+kc) = 1.0d0+ct5(kc)*(ct6(kc)+ct6(kc-1))
   lgs_a2(KRMAX+kc) = 0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ct5(kc)*ct6(kc)

#elif (ADV_VERT==2 || ADV_VERT==3)

   lgs_a0(KRMAX+kc) &
         = -0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1)) &
           -ct5(kc)*ct6(kc-1)
   lgs_a1(KRMAX+kc) &
         = 1.0d0 &
           +0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1)) &
           -0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  ) &
           +ct5(kc)*(ct6(kc)+ct6(kc-1))
   lgs_a2(KRMAX+kc) &
         =  0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  ) &
           -ct5(kc)*ct6(kc)

#endif

#if (ADV_HOR==2 || ADV_HOR==3)

   lgs_b(KRMAX+kc) = temp_c(kc,j,i) + ct7(kc) &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(temp_c(kc,j,i+1)-temp_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(temp_c(kc,j,i)-temp_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(temp_c(kc,j+1,i)-temp_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(temp_c(kc,j,i)-temp_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_HOR==4

   lgs_b(KRMAX+kc) = temp_c(kc,j,i) + ct7(kc) &
                     -dtt_dxi *(ftx_c_r(kc)-ftx_c_l(kc)) &
                     -dtt_deta*(fty_c_r(kc)-fty_c_l(kc))

#endif

end do

kc=KCMAX
lgs_a0(KRMAX+kc) = 0.0d0
lgs_a1(KRMAX+kc) = 1.0d0
lgs_b(KRMAX+kc)  = temp_s(j,i)

!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KCMAX+KRMAX)

!-------- Assign the result --------

do kr=0, KRMAX
   temp_r_neu(kr,j,i) = lgs_x(kr)
end do

do kc=0, KCMAX
   temp_c_neu(kc,j,i) = lgs_x(KRMAX+kc)
end do

!-------- Set water content in the non-existing temperate layer
!         to zero --------

do kt=0, KTMAX
   omega_t_neu(kt,j,i) = 0.0d0
end do

!-------- Water drainage from the non-existing temperate layer --------

Q_tld(j,i) = 0.0d0

!-------- Set up the equations for the age of the cold ice --------

#if (ADV_HOR==3 && ADV_VERT==3)

call calc_age_c(dtime_temp, i, j)   ! Algorithm by B. Muegge

#else

#if ADV_VERT==1

kc=0
lgs_a1(kc) = 1.0d0
lgs_a2(kc) = -1.0d0
lgs_b(kc)  = M_AGE*H_c(j,i)*ai4*mean_accum_inv

#elif ADV_VERT==2

kc=0
lgs_a1(kc) = 1.0d0 - adv_vert_sg(kc)
lgs_a2(kc) = adv_vert_sg(kc)
lgs_b(kc) = age_c(kc,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(age_c(kc,j,i+1)-age_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(age_c(kc,j,i)-age_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(age_c(kc,j+1,i)-age_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(age_c(kc,j,i)-age_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#endif

do kc=1, KCMAX-1

#if ADV_VERT==1

   lgs_a0(kc) = -0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ci1(kc)*ci2(kc-1)
   lgs_a1(kc) = 1.0d0+ci1(kc)*(ci2(kc)+ci2(kc-1))
   lgs_a2(kc) = 0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ci1(kc)*ci2(kc)

#elif ADV_VERT==2

   lgs_a0(kc) = -0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1))
   lgs_a1(kc) =  1.0d0 &
                +0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1)) &
                -0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  )
   lgs_a2(kc) =  0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  )

#endif

#if ADV_HOR==2

   lgs_b(kc) = age_c(kc,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(age_c(kc,j,i+1)-age_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(age_c(kc,j,i)-age_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(age_c(kc,j+1,i)-age_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(age_c(kc,j,i)-age_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_HOR==4

   lgs_b(kc) = age_c(kc,j,i) + dtime_temp &
               -dtt_dxi *(fax_c_r(kc)-fax_c_l(kc)) &
               -dtt_deta*(fay_c_r(kc)-fay_c_l(kc))

#endif

end do

#if ADV_VERT==1

kc=KCMAX
if (as_perp(j,i).ge.zero) then
   lgs_a0(kc) = 0.0d0
   lgs_a1(kc) = 1.0d0
   lgs_b(kc)  = 0.0d0
else
   lgs_a0(kc) = -1.0d0
   lgs_a1(kc) = 1.0d0
   lgs_b(kc)  = 0.0d0
end if

#elif ADV_VERT==2

kc=KCMAX
if (as_perp(j,i).ge.zero) then
   lgs_a0(kc) = 0.0d0
   lgs_a1(kc) = 1.0d0
   lgs_b(kc)  = 0.0d0
else
   lgs_a0(kc) = -adv_vert_sg(kc-1)
   lgs_a1(kc) = 1.0d0 + adv_vert_sg(kc-1)
   lgs_b(kc) = age_c(kc,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(age_c(kc,j,i+1)-age_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(age_c(kc,j,i)-age_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(age_c(kc,j+1,i)-age_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(age_c(kc,j,i)-age_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )
end if

#endif


!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KCMAX)

!-------- Assign the result,
!         restriction to interval [0, AGE_MAX yr] --------

do kc=0, KCMAX

   age_c_neu(kc,j,i) = lgs_x(kc)

   if (age_c_neu(kc,j,i).lt.(AGE_MIN*YEAR_SEC)) &
                            age_c_neu(kc,j,i) = 0.0d0
   if (age_c_neu(kc,j,i).gt.(AGE_MAX*YEAR_SEC)) &
                            age_c_neu(kc,j,i) = AGE_MAX*YEAR_SEC

end do

!-------- Age of the ice in the non-existing temperate layer --------

do kt=0, KTMAX
   age_t_neu(kt,j,i) = age_c_neu(0,j,i)
end do

#endif

end subroutine calc_temp1
