!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ t e m p 3

!   Purpose    :  Computation of temperature, water content and age
!                 for ice column with temperate base overlain by a
!                 temperate-ice layer.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
   at4_1, at4_2, at5, at6, at7, atr1, am1, am2, alb1, &
   aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
   ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
   dtime_temp, dtt_2dxi, dtt_2deta, i, j)

use sico_variables

implicit none
integer i, j, kc, kt, kr
double precision at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), at7, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai3, &
       atr1, am1, am2, alb1
double precision ct1(0:KCMAX), ct2(0:KCMAX), ct3(0:KCMAX), ct4(0:KCMAX), &
       ct5(0:KCMAX), ct6(0:KCMAX), ct7(0:KCMAX), ctr1, cm1, cm2, &
       clb1
double precision ct1_sg(0:KCMAX), ct2_sg(0:KCMAX), ct3_sg(0:KCMAX), &
       ct4_sg(0:KCMAX), adv_vert_sg(0:KCMAX), abs_adv_vert_sg(0:KCMAX)
double precision ci1(0:KCMAX), ci2(0:KCMAX), ci3
double precision aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld
double precision cw1(0:KTMAX), cw2(0:KTMAX), cw3(0:KTMAX), cw4(0:KTMAX), &
       cw5, cw7(0:KTMAX), cw8, cw9(0:KTMAX)
double precision cw1_sg(0:KTMAX), cw2_sg(0:KTMAX), cw3_sg(0:KTMAX), &
       cw4_sg(0:KTMAX), adv_vert_w_sg(0:KTMAX), abs_adv_vert_w_sg(0:KTMAX)
double precision ftx_c_l(0:KCMAX), ftx_c_r(0:KCMAX), &
                 fty_c_l(0:KCMAX), fty_c_r(0:KCMAX), &
                 fax_c_l(0:KCMAX), fax_c_r(0:KCMAX), &
                 fay_c_l(0:KCMAX), fay_c_r(0:KCMAX)
double precision fwx_t_l(0:KTMAX), fwx_t_r(0:KTMAX), &
                 fwy_t_l(0:KTMAX), fwy_t_r(0:KTMAX), &
                 fax_t_l(0:KTMAX), fax_t_r(0:KTMAX), &
                 fay_t_l(0:KTMAX), fay_t_r(0:KTMAX)
double precision sigma_c_help(0:KCMAX), sigma_t_help(0:KTMAX), &
       temp_c_help(0:KCMAX)
double precision dtt_2dxi, dtt_2deta, dtime_temp, dzeta_t
double precision dtt_dxi, dtt_deta
double precision mean_accum_inv
double precision ratefac, ratefac_t, creep, kappa_val, c_val
double precision lgs_a0(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_a1(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_a2(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_x(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_b(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX)
double precision, parameter :: zero=0.0d0

!-------- Check for boundary points --------

if ((i.eq.0).or.(i.eq.IMAX).or.(j.eq.0).or.(j.eq.JMAX)) &
   stop ' Boundary points not allowed in routine calc_temp3!'

!-------- Abbreviations --------

ctr1 = atr1
cm1  = am1*H_c_neu(j,i)
clb1 = alb1*q_geo(j,i)

#if ADV_VERT==1

do kc=1, KCMAX
   ct1(kc) = at1(kc)/H_c_neu(j,i)*0.5d0*(vz_c(kc,j,i)+vz_c(kc-1,j,i))
end do

#elif (ADV_VERT==2 || ADV_VERT==3)

do kc=0, KCMAX-1
   ct1_sg(kc) = 0.5d0*(at1(kc)+at1(kc+1))/H_c_neu(j,i)*vz_c(kc,j,i)
end do

#endif

do kc=0, KCMAX

   ct2(kc) = ( at2_1(kc)*dzm_dtau(j,i) &
           +at2_2(kc)*dH_c_dtau(j,i) )/H_c_neu(j,i)
   ct3(kc) = ( at3_1(kc)*dzm_dxi_g(j,i) &
           +at3_2(kc)*dH_c_dxi_g(j,i) )/H_c_neu(j,i) &
          *0.5d0*(vx_c(kc,j,i)+vx_c(kc,j,i-1)) *insq_g11_g(j,i) ! new factor !!!
   ct4(kc) = ( at4_1(kc)*dzm_deta_g(j,i) &
            +at4_2(kc)*dH_c_deta_g(j,i) )/H_c_neu(j,i) &
          *0.5d0*(vy_c(kc,j,i)+vy_c(kc,j-1,i)) *insq_g22_g(j,i) ! new factor !!!
   ct5(kc) = at5(kc) &
             /c_val(temp_c(kc,j,i)) &
             /H_c_neu(j,i)

   sigma_c_help(kc) &
           = RHO*G*H_c_neu(j,i)*(1.0d0-eaz_c_quotient(kc)) &
             *(dzs_dxi_g(j,i)**2+dzs_deta_g(j,i)**2)**0.5d0
   ct7(kc) = at7 &
             /c_val(temp_c(kc,j,i)) &
             *enh_c(kc,j,i) &
             *ratefac(temp_c(kc,j,i), temp_c_m(kc,j,i)) &
             *creep(sigma_c_help(kc)) &
             *sigma_c_help(kc)*sigma_c_help(kc)

   ci1(kc) = ai1(kc)/H_c_neu(j,i)

!  ------ Fluxes for horizontal advection of temperature and age

#if ADV_HOR==4
   if (vx_c(kc,j,i-1).ge.zero) then
      ftx_c_l(kc) = temp_c(kc,j,i-1)*vx_c(kc,j,i-1)
      fax_c_l(kc) =  age_c(kc,j,i-1)*vx_c(kc,j,i-1)
   else
      ftx_c_l(kc) = temp_c(kc,j,i)*vx_c(kc,j,i-1)
      fax_c_l(kc) =  age_c(kc,j,i)*vx_c(kc,j,i-1)
   end if

   if (vx_c(kc,j,i).ge.zero) then
      ftx_c_r(kc) = temp_c(kc,j,i)*vx_c(kc,j,i)
      fax_c_r(kc) =  age_c(kc,j,i)*vx_c(kc,j,i)
   else
      ftx_c_r(kc) = temp_c(kc,j,i+1)*vx_c(kc,j,i)
      fax_c_r(kc) =  age_c(kc,j,i+1)*vx_c(kc,j,i)
   end if

   if (vy_c(kc,j-1,i).ge.zero) then
      fty_c_l(kc) = temp_c(kc,j-1,i)*vy_c(kc,j-1,i)
      fay_c_l(kc) =  age_c(kc,j-1,i)*vy_c(kc,j-1,i)
   else
      fty_c_l(kc) = temp_c(kc,j,i)*vy_c(kc,j-1,i)
      fay_c_l(kc) =  age_c(kc,j,i)*vy_c(kc,j-1,i)
   end if

   if (vy_c(kc,j,i).ge.zero) then
      fty_c_r(kc) = temp_c(kc,j,i)*vy_c(kc,j,i)
      fay_c_r(kc) =  age_c(kc,j,i)*vy_c(kc,j,i)
   else
      fty_c_r(kc) = temp_c(kc,j+1,i)*vy_c(kc,j,i)
      fay_c_r(kc) =  age_c(kc,j+1,i)*vy_c(kc,j,i)
   end if
#endif

end do

#if (ADV_VERT==2 || ADV_VERT==3)

do kc=0, KCMAX-1
   ct2_sg(kc) = 0.5d0*(ct2(kc)+ct2(kc+1))
   ct3_sg(kc) = 0.5d0*(ct3(kc)+ct3(kc+1))
   ct4_sg(kc) = 0.5d0*(ct4(kc)+ct4(kc+1))
   adv_vert_sg(kc) = ct1_sg(kc)-ct2_sg(kc)-ct3_sg(kc)-ct4_sg(kc)
   abs_adv_vert_sg(kc) = abs(adv_vert_sg(kc))
end do

#endif

do kc=0, KCMAX-1
   temp_c_help(kc) = 0.5d0*(temp_c(kc,j,i)+temp_c(kc+1,j,i))
   ct6(kc) = at6(kc) &
    *kappa_val(temp_c_help(kc)) &
    /H_c_neu(j,i)
   ci2(kc) = ai2(kc)/H_c_neu(j,i)
end do

cw5 = aw5/(H_t_neu(j,i)**2)
cw8 = aw8
ci3 = ai3/(H_t_neu(j,i)**2)

#if ADV_VERT==1

do kt=1, KTMAX
   cw1(kt) = aw1/H_t_neu(j,i)*0.5d0*(vz_t(kt,j,i)+vz_t(kt-1,j,i))
end do

#elif (ADV_VERT==2 || ADV_VERT==3)

do kt=0, KTMAX-1
   cw1_sg(kt) = aw1/H_t_neu(j,i)*vz_t(kt,j,i)
end do

#endif

do kt=1, KTMAX
   cw9(kt) = aw9 &
             *c_val(temp_t_m(kt,j,i)) &
    *( dzs_dtau(j,i) &
      +0.5d0*(vx_t(kt,j,i)+vx_t(kt,j,i-1))*dzs_dxi_g(j,i) &
      +0.5d0*(vy_t(kt,j,i)+vy_t(kt,j-1,i))*dzs_deta_g(j,i) &
      -0.5d0*(vz_t(kt,j,i)+vz_t(kt-1,j,i)) )
end do

do kt=0, KTMAX

   cw2(kt) = aw2*(dzb_dtau(j,i)+zeta_t(kt)*dH_t_dtau(j,i)) &
          /H_t_neu(j,i)
   cw3(kt) = aw3*(dzb_dxi_g(j,i)+zeta_t(kt)*dH_t_dxi_g(j,i)) &
             /H_t_neu(j,i) &
             *0.5d0*(vx_t(kt,j,i)+vx_t(kt,j,i-1)) *insq_g11_g(j,i) ! new factor !!!
   cw4(kt) = aw4*(dzb_deta_g(j,i)+zeta_t(kt)*dH_t_deta_g(j,i)) &
             /H_t_neu(j,i) &
             *0.5d0*(vy_t(kt,j,i)+vy_t(kt,j-1,i)) *insq_g22_g(j,i) ! new factor !!!

   sigma_t_help(kt) &
           = sigma_c_help(0) &
             + RHO*G*H_t_neu(j,i)*(1.0d0-zeta_t(kt)) &
               *(dzs_dxi_g(j,i)**2+dzs_deta_g(j,i)**2)**0.5d0
   cw7(kt) = aw7 &
             *enh_t(kt,j,i) &
             *ratefac_t(omega_t(kt,j,i)) &
             *creep(sigma_t_help(kt)) &
             *sigma_t_help(kt)*sigma_t_help(kt)

!  ------ Fluxes for horizontal advection of water content and age

#if ADV_HOR==4
   if (vx_t(kt,j,i-1).ge.zero) then
      fwx_t_l(kt) = omega_t(kt,j,i-1)*vx_t(kt,j,i-1)
      fax_t_l(kt) =   age_t(kt,j,i-1)*vx_t(kt,j,i-1)
   else
      fwx_t_l(kt) = omega_t(kt,j,i)*vx_t(kt,j,i-1)
      fax_t_l(kt) =   age_t(kt,j,i)*vx_t(kt,j,i-1)
   end if

   if (vx_t(kt,j,i).ge.zero) then
      fwx_t_r(kt) = omega_t(kt,j,i)*vx_t(kt,j,i)
      fax_t_r(kt) =   age_t(kt,j,i)*vx_t(kt,j,i)
   else
      fwx_t_r(kt) = omega_t(kt,j,i+1)*vx_t(kt,j,i)
      fax_t_r(kt) =   age_t(kt,j,i+1)*vx_t(kt,j,i)
   end if

   if (vy_t(kt,j-1,i).ge.zero) then
      fwy_t_l(kt) = omega_t(kt,j-1,i)*vy_t(kt,j-1,i)
      fay_t_l(kt) =   age_t(kt,j-1,i)*vy_t(kt,j-1,i)
   else
      fwy_t_l(kt) = omega_t(kt,j,i)*vy_t(kt,j-1,i)
      fay_t_l(kt) =   age_t(kt,j,i)*vy_t(kt,j-1,i)
   end if

   if (vy_t(kt,j,i).ge.zero) then
      fwy_t_r(kt) = omega_t(kt,j,i)*vy_t(kt,j,i)
      fay_t_r(kt) =   age_t(kt,j,i)*vy_t(kt,j,i)
   else
      fwy_t_r(kt) = omega_t(kt,j+1,i)*vy_t(kt,j,i)
      fay_t_r(kt) =   age_t(kt,j+1,i)*vy_t(kt,j,i)
   end if
#endif

end do

#if (ADV_VERT==2 || ADV_VERT==3)

do kt=0, KTMAX-1
   cw2_sg(kt) = 0.5d0*(cw2(kt)+cw2(kt+1))
   cw3_sg(kt) = 0.5d0*(cw3(kt)+cw3(kt+1))
   cw4_sg(kt) = 0.5d0*(cw4(kt)+cw4(kt+1))
   adv_vert_w_sg(kt) = cw1_sg(kt)-cw2_sg(kt)-cw3_sg(kt)-cw4_sg(kt)
   abs_adv_vert_w_sg(kt) = abs(adv_vert_w_sg(kt))
end do

#endif

#if ADV_HOR==4
dtt_dxi  = 2.0d0*dtt_2dxi
dtt_deta = 2.0d0*dtt_2deta
#endif

!-------- Set up the equations for the bedrock temperature --------

kr=0
lgs_a1(kr) = 1.0d0
lgs_a2(kr) = -1.0d0
lgs_b(kr)    = clb1

#if Q_LITHO==1
!   (coupled heat-conducting bedrock)

do kr=1, KRMAX-1
   lgs_a0(kr) = - ctr1
   lgs_a1(kr) = 1.0d0 + 2.0d0*ctr1
   lgs_a2(kr) = - ctr1
   lgs_b(kr)    = temp_r(kr,j,i)
end do

#elif Q_LITHO==0
!   (no coupled heat-conducting bedrock)

do kr=1, KRMAX-1
   lgs_a0(kr) = 1.0d0
   lgs_a1(kr) = 0.0d0
   lgs_a2(kr) = -1.0d0
   lgs_b(kr)  = 2.0d0*clb1
end do

#endif

kr=KRMAX
lgs_a0(kr) = 0.0d0
lgs_a1(kr) = 1.0d0
lgs_b(kr)   = temp_t_m(0,j,i)

!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KRMAX)

!-------- Assign the result --------

do kr=0, KRMAX
   temp_r_neu(kr,j,i) = lgs_x(kr)
end do

!-------- Set up the equations for the water content in
!         temperate ice --------

kt=0
lgs_a1(kt) = 1.0d0
lgs_a2(kt) = -1.0d0
lgs_b(kt)  = 0.0d0

do kt=1, KTMAX-1

#if ADV_VERT==1

   lgs_a0(kt) = -0.5d0*(cw1(kt)-cw2(kt)-cw3(kt)-cw4(kt)) - cw5
   lgs_a1(kt) = 1.0d0 + 2.0d0*cw5
   lgs_a2(kt) = 0.5d0*(cw1(kt)-cw2(kt)-cw3(kt)-cw4(kt)) - cw5

#elif (ADV_VERT==2 || ADV_VERT==3)

   lgs_a0(kt) &
         = -0.5d0*(adv_vert_w_sg(kt-1)+abs_adv_vert_w_sg(kt-1)) &
           -cw5
   lgs_a1(kt) &
         = 1.0d0 &
           +0.5d0*(adv_vert_w_sg(kt-1)+abs_adv_vert_w_sg(kt-1)) &
           -0.5d0*(adv_vert_w_sg(kt)  -abs_adv_vert_w_sg(kt)  ) &
           +2.0d0*cw5
   lgs_a2(kt) &
         =  0.5d0*(adv_vert_w_sg(kt)  -abs_adv_vert_w_sg(kt)  ) &
           -cw5

#endif

#if (ADV_HOR==2 || ADV_HOR==3)

   lgs_b(kt) = omega_t(kt,j,i) + cw7(kt) + cw8 + cw9(kt) &
       -dtt_2dxi* &
          ( (vx_t(kt,j,i)-abs(vx_t(kt,j,i))) &
            *(omega_t(kt,j,i+1)-omega_t(kt,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_t(kt,j,i-1)+abs(vx_t(kt,j,i-1))) &
            *(omega_t(kt,j,i)-omega_t(kt,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_t(kt,j,i)-abs(vy_t(kt,j,i))) &
            *(omega_t(kt,j+1,i)-omega_t(kt,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_t(kt,j-1,i)+abs(vy_t(kt,j-1,i))) &
            *(omega_t(kt,j,i)-omega_t(kt,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_HOR==4

   lgs_b(kt) = omega_t(kt,j,i) + cw7(kt) + cw8 + cw9(kt) &
               -dtt_dxi *(fwx_t_r(kt)-fwx_t_l(kt)) &
               -dtt_deta*(fwy_t_r(kt)-fwy_t_l(kt))

#endif

end do

kt=KTMAX

if (am_perp(j,i).ge.zero) then   ! melting condition
   lgs_a0(kt) = 0.0d0
   lgs_a1(kt) = 1.0d0
   lgs_b(kt)  = 0.0d0
else   ! am_perp(j,i).lt.0.0, freezing condition
   lgs_a0(kt) = -1.0d0
   lgs_a1(kt) = 1.0d0
   lgs_b(kt)  = 0.0d0
end if

!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KTMAX)

!-------- Assign the result, compute the water drainage --------

Q_tld(j,i) = 0.0d0

do kt=0, KTMAX

   if (lgs_x(kt).lt.zero) then
      omega_t_neu(kt,j,i) = 0.0d0   ! (as a precaution)
   else if (lgs_x(kt).lt.OMEGA_MAX) then
      omega_t_neu(kt,j,i) = lgs_x(kt)
   else
      omega_t_neu(kt,j,i) = OMEGA_MAX
      Q_tld(j,i) = Q_tld(j,i) &
                     +aqtld*H_t_neu(j,i)*(lgs_x(kt)-OMEGA_MAX)
   end if

end do

!-------- Set up the equations for the ice temperature --------

!  ------ Abbreviation for the jump of the temperature gradient with
!         the new omega

if (am_perp(j,i).ge.zero) then   ! melting condition
   cm2 = 0.0d0
else   ! am_perp(j,i).lt.0.0, freezing condition
   cm2  = am2*H_c_neu(j,i)*omega_t_neu(KTMAX,j,i)*am_perp(j,i) &
          /kappa_val(temp_c(0,j,i))
end if

kc=0
lgs_a1(kc) = 1.0d0
lgs_a2(kc) = -1.0d0
lgs_b(kc)  = -cm1-cm2

do kc=1, KCMAX-1

#if ADV_VERT==1

   lgs_a0(kc) = -0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ct5(kc)*ct6(kc-1)
   lgs_a1(kc) = 1.0d0+ct5(kc)*(ct6(kc)+ct6(kc-1))
   lgs_a2(kc) = 0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ct5(kc)*ct6(kc)

#elif (ADV_VERT==2 || ADV_VERT==3)

   lgs_a0(kc) &
         = -0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1)) &
           -ct5(kc)*ct6(kc-1)
   lgs_a1(kc) &
         = 1.0d0 &
           +0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1)) &
           -0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  ) &
           +ct5(kc)*(ct6(kc)+ct6(kc-1))
   lgs_a2(kc) &
         =  0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  ) &
           -ct5(kc)*ct6(kc)

#endif

#if (ADV_HOR==2 || ADV_HOR==3)

   lgs_b(kc) = temp_c(kc,j,i) + ct7(kc) &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(temp_c(kc,j,i+1)-temp_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(temp_c(kc,j,i)-temp_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(temp_c(kc,j+1,i)-temp_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(temp_c(kc,j,i)-temp_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_HOR==4

   lgs_b(kc)  = temp_c(kc,j,i) + ct7(kc) &
               -dtt_dxi *(ftx_c_r(kc)-ftx_c_l(kc)) &
               -dtt_deta*(fty_c_r(kc)-fty_c_l(kc))

#endif

end do

kc=KCMAX
lgs_a0(kc) = 0.0d0
lgs_a1(kc) = 1.0d0
lgs_b(kc)  = temp_s(j,i)

!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KCMAX)

!-------- Assign the result --------

do kc=0, KCMAX
   temp_c_neu(kc,j,i) = lgs_x(kc)
end do

!-------- Set up the equations for the age (cold and temperate ice
!         simultaneously) --------

#if (ADV_HOR==3 && ADV_VERT==3)

call calc_age_t(dtime_temp, i, j)   ! Algorithm by B. Muegge

#else

#if ADV_VERT==1

kt=0
lgs_a1(kt) = 1.0d0
lgs_a2(kt) = -1.0d0
lgs_b(kt)  = M_AGE*H_t(j,i)*dzeta_t*mean_accum_inv

#elif ADV_VERT==2

kt=0
lgs_a1(kt) = 1.0d0 - adv_vert_w_sg(kt)
lgs_a2(kt) = adv_vert_w_sg(kt)
lgs_b(kt) = age_t(kt,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_t(kt,j,i)-abs(vx_t(kt,j,i))) &
            *(age_t(kt,j,i+1)-age_t(kt,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_t(kt,j,i-1)+abs(vx_t(kt,j,i-1))) &
            *(age_t(kt,j,i)-age_t(kt,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_t(kt,j,i)-abs(vy_t(kt,j,i))) &
            *(age_t(kt,j+1,i)-age_t(kt,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_t(kt,j-1,i)+abs(vy_t(kt,j-1,i))) &
            *(age_t(kt,j,i)-age_t(kt,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#endif

do kt=1, KTMAX-1

#if ADV_VERT==1

   lgs_a0(kt) = -0.5d0*(cw1(kt)-cw2(kt)-cw3(kt)-cw4(kt)) - ci3
   lgs_a1(kt) = 1.0d0 + 2.0d0*ci3
   lgs_a2(kt) = 0.5d0*(cw1(kt)-cw2(kt)-cw3(kt)-cw4(kt)) - ci3

#elif ADV_VERT==2

   lgs_a0(kt) = -0.5d0*(adv_vert_w_sg(kt-1)+abs_adv_vert_w_sg(kt-1))
   lgs_a1(kt) = 1.0d0 &
               +0.5d0*(adv_vert_w_sg(kt-1)+abs_adv_vert_w_sg(kt-1)) &
               -0.5d0*(adv_vert_w_sg(kt)  -abs_adv_vert_w_sg(kt)  )
   lgs_a2(kt) =  0.5d0*(adv_vert_w_sg(kt)  -abs_adv_vert_w_sg(kt)  )

#endif

#if ADV_HOR==2

   lgs_b(kt) = age_t(kt,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_t(kt,j,i)-abs(vx_t(kt,j,i))) &
            *(age_t(kt,j,i+1)-age_t(kt,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_t(kt,j,i-1)+abs(vx_t(kt,j,i-1))) &
            *(age_t(kt,j,i)-age_t(kt,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_t(kt,j,i)-abs(vy_t(kt,j,i))) &
            *(age_t(kt,j+1,i)-age_t(kt,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_t(kt,j-1,i)+abs(vy_t(kt,j-1,i))) &
            *(age_t(kt,j,i)-age_t(kt,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_HOR==4

   lgs_b(kt) = age_t(kt,j,i) + dtime_temp &
               -dtt_dxi *(fax_t_r(kt)-fax_t_l(kt)) &
               -dtt_deta*(fay_t_r(kt)-fay_t_l(kt))

#endif

end do

#if ADV_VERT==1

kt=KTMAX
kc=0

lgs_a0(kt) = -0.5d0*(cw1(kt)-cw2(kt)-cw3(kt)-cw4(kt)) - ci3
lgs_a1(kt) = 1.0d0 + 2.0d0*ci3
lgs_a2(kt) = 0.5d0*(cw1(kt)-cw2(kt)-cw3(kt)-cw4(kt)) - ci3
lgs_b(kt) = age_t(kt,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_t(kt,j,i)-abs(vx_t(kt,j,i))) &
            *(age_t(kt,j,i+1)-age_t(kt,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_t(kt,j,i-1)+abs(vx_t(kt,j,i-1))) &
            *(age_t(kt,j,i)-age_t(kt,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_t(kt,j,i)-abs(vy_t(kt,j,i))) &
            *(age_t(kt,j+1,i)-age_t(kt,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_t(kt,j-1,i)+abs(vy_t(kt,j-1,i))) &
            *(age_t(kt,j,i)-age_t(kt,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_VERT==2

kt=KTMAX
kc=0

if (adv_vert_sg(kc).le.zero) then

   lgs_a0(KTMAX+kc) = 0.0d0
   lgs_a1(KTMAX+kc) = 1.0d0 - adv_vert_sg(kc)
   lgs_a2(KTMAX+kc) = adv_vert_sg(kc)
   lgs_b(KTMAX+kc) = age_c(kc,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(age_c(kc,j,i+1)-age_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(age_c(kc,j,i)-age_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(age_c(kc,j+1,i)-age_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(age_c(kc,j,i)-age_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

else if (adv_vert_w_sg(kt-1).ge.zero) then

   lgs_a0(kt) = -adv_vert_w_sg(kt-1)
   lgs_a1(kt) = 1.0d0 + adv_vert_w_sg(kt-1)
   lgs_a2(kt) = 0.0d0
   lgs_b(kt) = age_t(kt,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_t(kt,j,i)-abs(vx_t(kt,j,i))) &
            *(age_t(kt,j,i+1)-age_t(kt,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_t(kt,j,i-1)+abs(vx_t(kt,j,i-1))) &
            *(age_t(kt,j,i)-age_t(kt,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_t(kt,j,i)-abs(vy_t(kt,j,i))) &
            *(age_t(kt,j+1,i)-age_t(kt,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_t(kt,j-1,i)+abs(vy_t(kt,j-1,i))) &
            *(age_t(kt,j,i)-age_t(kt,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

else

   lgs_a0(kt) = -0.5d0
   lgs_a1(kt) = 1.0d0
   lgs_a2(kt) = -0.5d0
   lgs_b(kt)  = 0.0d0
   ! Makeshift: Average of age_c(kc=1) and age_t(kt=KTMAX-1)

end if

#endif

do kc=1, KCMAX-1

#if ADV_VERT==1

   lgs_a0(KTMAX+kc) = -0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ci1(kc)*ci2(kc-1)
   lgs_a1(KTMAX+kc) = 1.0d0+ci1(kc)*(ci2(kc)+ci2(kc-1))
   lgs_a2(KTMAX+kc) = 0.5d0*(ct1(kc)-ct2(kc)-ct3(kc)-ct4(kc)) &
                      -ci1(kc)*ci2(kc)

#elif ADV_VERT==2

   lgs_a0(KTMAX+kc) = -0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1))
   lgs_a1(KTMAX+kc) =  1.0d0 &
                      +0.5d0*(adv_vert_sg(kc-1)+abs_adv_vert_sg(kc-1)) &
                      -0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  )
   lgs_a2(KTMAX+kc) =  0.5d0*(adv_vert_sg(kc)  -abs_adv_vert_sg(kc)  )

#endif

#if ADV_HOR==2

   lgs_b(KTMAX+kc) = age_c(kc,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(age_c(kc,j,i+1)-age_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(age_c(kc,j,i)-age_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(age_c(kc,j+1,i)-age_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(age_c(kc,j,i)-age_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )

#elif ADV_HOR==4

   lgs_b(KTMAX+kc) = age_c(kc,j,i) + dtime_temp &
                     -dtt_dxi *(fax_c_r(kc)-fax_c_l(kc)) &
                     -dtt_deta*(fay_c_r(kc)-fay_c_l(kc))

#endif

end do

#if ADV_VERT==1

kc=KCMAX
if (as_perp(j,i).ge.zero) then
   lgs_a0(KTMAX+kc) = 0.0d0
   lgs_a1(KTMAX+kc) = 1.0d0
   lgs_b(KTMAX+kc)  = 0.0d0
else
   lgs_a0(KTMAX+kc) = -1.0d0
   lgs_a1(KTMAX+kc) = 1.0d0
   lgs_b(KTMAX+kc)  = 0.0d0
end if

#elif ADV_VERT==2

kc=KCMAX
if (as_perp(j,i).ge.zero) then
   lgs_a0(KTMAX+kc) = 0.0d0
   lgs_a1(KTMAX+kc) = 1.0d0
   lgs_b(KTMAX+kc)  = 0.0d0
else
   lgs_a0(KTMAX+kc) = -adv_vert_sg(kc-1)
   lgs_a1(KTMAX+kc) = 1.0d0 + adv_vert_sg(kc-1)
   lgs_b(KTMAX+kc) = age_c(kc,j,i) + dtime_temp &
       -dtt_2dxi* &
          ( (vx_c(kc,j,i)-abs(vx_c(kc,j,i))) &
            *(age_c(kc,j,i+1)-age_c(kc,j,i)) &
            *insq_g11_sgx(j,i) &
           +(vx_c(kc,j,i-1)+abs(vx_c(kc,j,i-1))) &
            *(age_c(kc,j,i)-age_c(kc,j,i-1)) &
            *insq_g11_sgx(j,i-1) ) &
       -dtt_2deta* &
          ( (vy_c(kc,j,i)-abs(vy_c(kc,j,i))) &
            *(age_c(kc,j+1,i)-age_c(kc,j,i)) &
            *insq_g22_sgy(j,i) &
           +(vy_c(kc,j-1,i)+abs(vy_c(kc,j-1,i))) &
            *(age_c(kc,j,i)-age_c(kc,j-1,i)) &
            *insq_g22_sgy(j-1,i) )
end if

#endif

!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KCMAX+KTMAX)

!-------- Assign the result,
!         restriction to interval [0, AGE_MAX yr] --------

do kt=0, KTMAX

   age_t_neu(kt,j,i) = lgs_x(kt)

   if (age_t_neu(kt,j,i).lt.(AGE_MIN*YEAR_SEC)) &
                            age_t_neu(kt,j,i) = 0.0d0
   if (age_t_neu(kt,j,i).gt.(AGE_MAX*YEAR_SEC)) &
                            age_t_neu(kt,j,i) = AGE_MAX*YEAR_SEC

end do

do kc=0, KCMAX

   age_c_neu(kc,j,i) = lgs_x(KTMAX+kc)

   if (age_c_neu(kc,j,i).lt.(AGE_MIN*YEAR_SEC)) &
                            age_c_neu(kc,j,i) = 0.0d0
   if (age_c_neu(kc,j,i).gt.(AGE_MAX*YEAR_SEC)) &
                            age_c_neu(kc,j,i) = AGE_MAX*YEAR_SEC

end do

#endif

end subroutine calc_temp3
