!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ t e m p _ c o l d

!   Purpose    :  Computation of temperature and age in cold-ice mode.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_temp_cold(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 acb1, acb2, acb3, acb4, &
                 ai1, ai2, ai3, ai4, mean_accum_inv, &
                 dxi, deta, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta)

use sico_variables

implicit none
integer i, j, kc, kr, ii, jj
double precision at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), at7, &
       acb1, acb2, acb3, acb4, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai3, ai4, &
       atr1, am1, am2, alb1
double precision aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld
double precision temp_c_help(0:KCMAX)
double precision dtt_2dxi, dtt_2deta, dtime_temp, dxi, deta, dzeta_t
double precision mean_accum_inv

!-------- Computation loop for temperature and age --------

do i=1, IMAX-1   ! skipping domain margins (T.Zwinger 7.12.01 -> V1.1)
do j=1, JMAX-1   ! skipping domain margins (T.Zwinger 7.12.01 -> V1.1)

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
!                                ! glaciated land or floating ice

      n_cts_neu(j,i) = -1
      zm_neu(j,i)  = zb(j,i)
      H_c_neu(j,i) = H_c(j,i)
      H_t_neu(j,i) = H_t(j,i)

      call calc_temp1(at1, at2_1, at2_2, at3_1, at3_2, &
           at4_1, at4_2, at5, at6, at7, atr1, acb1, acb2, &
           acb3, acb4, alb1, ai1, ai2, ai4, &
           mean_accum_inv, &
           dtime_temp, dtt_2dxi, dtt_2deta, i, j)

!  ------ Reset temperatures above melting to the melting point,
!         look for the CTS

      kc_cts(j,i) = 0

      if (temp_c_neu(0,j,i).gt.temp_c_m(0,j,i)) then
         n_cts_neu(j,i)        = 0
         kc_cts(j,i)           = 0
         temp_c_neu(0,j,i)     = temp_c_m(0,j,i)
         temp_r_neu(KRMAX,j,i) = temp_c_m(0,j,i)
      end if

      do kc=1, KCMAX
         if (temp_c_neu(kc,j,i).gt.temp_c_m(kc,j,i)) then
            kc_cts(j,i)        = kc
            temp_c_neu(kc,j,i) = temp_c_m(kc,j,i)
         end if
      end do

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      kc_cts(j,i)    = 0
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

end do
end do 

!-------- Extrapolate values on margins (T.Zwinger 7.12.01 -> V1.1) --------

!  ------ Lower left corner

i=0
j=0

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i+1
   jj=j+1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = n_cts_neu(jj,ii)
   kc_cts(j,i)    = kc_cts(jj,ii)
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   kc_cts(j,i)    = 0
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Lower right corner

i=IMAX
j=0

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i-1
   jj=j+1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = n_cts_neu(jj,ii)
   kc_cts(j,i)    = kc_cts(jj,ii)
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   kc_cts(j,i)    = 0
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Upper left corner

i=0
j=JMAX

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i+1
   jj=j-1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = n_cts_neu(jj,ii)
   kc_cts(j,i)    = kc_cts(jj,ii)
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   kc_cts(j,i)    = 0
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Upper right corner

i=IMAX
j=JMAX

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i-1
   jj=j-1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = n_cts_neu(jj,ii)
   kc_cts(j,i)    = kc_cts(jj,ii)
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   kc_cts(j,i)    = 0
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Lower and upper margins

do i=1, IMAX-1

!    ---- Lower margin

   j=0

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i
      jj=j+1

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = n_cts_neu(jj,ii)
      kc_cts(j,i)    = kc_cts(jj,ii)
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      kc_cts(j,i)    = 0
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

!    ---- Upper margin

   j=JMAX

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i
      jj=j-1

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = n_cts_neu(jj,ii)
      kc_cts(j,i)    = kc_cts(jj,ii)
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      kc_cts(j,i)    = 0
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

end do

!  ------ Left and right margins

do j=1, JMAX-1

!    ---- Left margin

   i=0

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i+1
      jj=j

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = n_cts_neu(jj,ii)
      kc_cts(j,i)    = kc_cts(jj,ii)
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      kc_cts(j,i)    = 0
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

!    ---- Right margin

   i=IMAX

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i-1
      jj=j

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = n_cts_neu(jj,ii)
      kc_cts(j,i)    = kc_cts(jj,ii)
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      kc_cts(j,i)    = 0
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

end do

return
end
