!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ t e m p _ p o l y

!   Purpose    :  Computation of temperature, water content and age
!                 in polythermal mode.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_temp_poly(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 acb1, acb2, acb3, acb4, &
                 ai1, ai2, ai3, ai4, mean_accum_inv, &
                 dxi, deta, dzeta_t, &
                 dtime_temp, dtime_temp_inv, dtt_2dxi, dtt_2deta)

use sico_variables

implicit none
integer i, j, kc, kt, kr, ii, jj
double precision at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), at7, &
       acb1, acb2, acb3, acb4, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai3, ai4, &
       atr1, am1, am2, alb1
double precision aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld
double precision temp_c_help(0:KCMAX)
double precision dtt_2dxi, dtt_2deta, dtime_temp, dxi, deta, dzeta_t
double precision mean_accum_inv
double precision dtime_temp_inv
double precision fact_thick
double precision time_lag_cts
double precision Vol_t, Vol_t_smooth, korrfakt_t, &
       dH_t_smooth(0:JMAX,0:IMAX)

!-------- Computation loop for temperature, water content and age --------

do i=1, IMAX-1   ! skipping domain margins (22.1.02 -> V1.1)
do j=1, JMAX-1   ! skipping domain margins (22.1.02 -> V1.1)

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
!                                ! glaciated land or floating ice --------

!  ------ Old vertical column cold

      if (n_cts(j,i).eq.-1) then

         n_cts_neu(j,i) = n_cts(j,i)
         zm_neu(j,i)  = zb(j,i)
         H_c_neu(j,i) = H_c(j,i)
         H_t_neu(j,i) = H_t(j,i)

         call calc_temp1(at1, at2_1, at2_2, at3_1, at3_2, &
           at4_1, at4_2, at5, at6, at7, atr1, acb1, acb2, &
           acb3, acb4, alb1, ai1, ai2, ai4, &
           mean_accum_inv, &
           dtime_temp, dtt_2dxi, dtt_2deta, i, j)

!    ---- Check whether base has become temperate

         if (temp_c_neu(0,j,i).gt.temp_c_m(0,j,i)) then

            n_cts_neu(j,i) = 0

            call calc_temp2(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, alb1, &
                 ai1, ai2, ai4, mean_accum_inv, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

         end if

!    ---- Check whether even temperate layer has formed

         if ( &
              ( n_cts_neu(j,i).eq.0 ).and. &
              ( (temp_c_neu(1,j,i)-temp_c_neu(0,j,i)) &
                .gt.(am1*H_c_neu(j,i)) ) &
            ) then

            n_cts_neu(j,i) = 1
            zm_neu(j,i)  = zb(j,i)+0.001d0
            H_c_neu(j,i) = H_c(j,i)-0.001d0
            H_t_neu(j,i) = H_t(j,i)+0.001d0
!                 ! CTS preliminarily positioned 1 mm above ice base --------

            call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

            call shift_cts_upward(at1, at2_1, at2_2, at3_1, at3_2, &
              at4_1, at4_2, at5, at6, at7, atr1, am1, am2, alb1, &
              aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
              ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
              dtime_temp, dtt_2dxi, dtt_2deta, dtime_temp_inv, &
              i, j)

         end if

!  ------ Old vertical column with temperate base

      else if (n_cts(j,i).eq.0) then

         n_cts_neu(j,i) = n_cts(j,i)
         zm_neu(j,i)  = zb(j,i)
         H_c_neu(j,i) = H_c(j,i)
         H_t_neu(j,i) = H_t(j,i)

         call calc_temp2(at1, at2_1, at2_2, at3_1, at3_2, &
              at4_1, at4_2, at5, at6, at7, atr1, alb1, &
              ai1, ai2, ai4, mean_accum_inv, &
              dtime_temp, dtt_2dxi, dtt_2deta, i, j)

!    ---- Check whether temperate base becomes cold

         if ( (temp_c_neu(1,j,i)-temp_c_neu(0,j,i)) &
               .lt. (am1*H_c(j,i)) ) then

            n_cts_neu(j,i) = -1

            call calc_temp1(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, acb1, acb2, &
                 acb3, acb4, alb1, ai1, ai2, ai4, &
                 mean_accum_inv, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

            if (temp_c_neu(0,j,i).ge.temp_c_m(0,j,i)) then

               n_cts_neu(j,i) = 0

               call calc_temp2(at1, at2_1, at2_2, at3_1, at3_2, &
                    at4_1, at4_2, at5, at6, at7, atr1, alb1, &
                    ai1, ai2, ai4, mean_accum_inv, &
                    dtime_temp, dtt_2dxi, dtt_2deta, i, j)

            end if

         end if

!    ---- Check whether temperate layer has formed

         if ( &
              ( n_cts_neu(j,i).eq.0 ).and. &
              ( (temp_c_neu(1,j,i)-temp_c_neu(0,j,i)) &
                .gt.(am1*H_c_neu(j,i)) ) &
            ) then

            n_cts_neu(j,i) = 1
            zm_neu(j,i)  = zb(j,i)+0.001d0
            H_c_neu(j,i) = H_c(j,i)-0.001d0
            H_t_neu(j,i) = H_t(j,i)+0.001d0
!                 ! CTS preliminarily positioned 1 mm above ice base --------

            call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

            call shift_cts_upward(at1, at2_1, at2_2, at3_1, at3_2, &
              at4_1, at4_2, at5, at6, at7, atr1, am1, am2, alb1, &
              aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
              ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
              dtime_temp, dtt_2dxi, dtt_2deta, dtime_temp_inv, &
              i, j)

         end if

!  ------ Old vertical column with temperate base and temperate layer

      else   ! n_cts(j,i).eq.1

         n_cts_neu(j,i) = n_cts(j,i)
         zm_neu(j,i)  = zm(j,i)
         H_c_neu(j,i) = H_c(j,i)
         H_t_neu(j,i) = H_t(j,i)

         call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

         if ( (temp_c_neu(0,j,i)-(-BETA*H_c_neu(j,i))).gt.0.0d0 ) &
         then
            call shift_cts_upward(at1, at2_1, at2_2, at3_1, at3_2, &
              at4_1, at4_2, at5, at6, at7, atr1, am1, am2, alb1, &
              aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
              ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
              dtime_temp, dtt_2dxi, dtt_2deta, dtime_temp_inv, &
              i, j)
         else
            call shift_cts_downward(at1, at2_1, at2_2, at3_1, at3_2, &
              at4_1, at4_2, at5, at6, at7, atr1, am1, am2, alb1, &
              aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
              ai1, ai2, ai3, ai4, mean_accum_inv, dzeta_t, &
              dtime_temp, dtt_2dxi, dtt_2deta, dtime_temp_inv, &
              i, j)
         end if

      end if

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      zm_neu(j,i)  = zb(j,i)
      H_c_neu(j,i) = H_c(j,i)
      H_t_neu(j,i) = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

end do
end do   ! End of computation loop

!-------- Extrapolate values on margins (22.1.02 -> V1.1) --------

!  ------ Lower left corner

i=0
j=0

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i+1
   jj=j+1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kt=0,KTMAX
      omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
      age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)
   zm_neu(j,i)  = zb(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Lower right corner

i=IMAX
j=0

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i-1
   jj=j+1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kt=0,KTMAX
      omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
      age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)
   zm_neu(j,i)  = zb(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Upper left corner

i=0
j=JMAX

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i+1
   jj=j-1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kt=0,KTMAX
      omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
      age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)
   zm_neu(j,i)  = zb(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Upper right corner

i=IMAX
j=JMAX

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                              ! glaciated land or floating ice
   ii=i-1
   jj=j-1

   do kc=0,KCMAX
      temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
      age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
   end do

   do kt=0,KTMAX
      omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
      age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
   end do

   do kr=0,KRMAX
      temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
   end do

   n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)
   zm_neu(j,i)  = zb(j,i)

else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

   n_cts_neu(j,i) = -1
   zm_neu(j,i)    = zb(j,i)
   H_c_neu(j,i)   = H_c(j,i)
   H_t_neu(j,i)   = H_t(j,i)

   call calc_temp_r(atr1, alb1, i, j)

end if

!  ------ Lower and upper margins

do i=1, IMAX-1

!    ---- Lower margin

   j=0

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i
      jj=j+1

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kt=0,KTMAX
         omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
         age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)
      zm_neu(j,i)  = zb(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

!    ---- Upper margin

   j=JMAX

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i
      jj=j-1

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kt=0,KTMAX
         omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
         age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)
      zm_neu(j,i)  = zb(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

end do

!  ------ Left and right margins

do j=1, JMAX-1

!    ---- Left margin

   i=0

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i+1
      jj=j

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kt=0,KTMAX
         omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
         age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)
      zm_neu(j,i)  = zb(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

!    ---- Right margin

   i=IMAX

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
                                 ! glaciated land or floating ice
      ii=i-1
      jj=j

      do kc=0,KCMAX
         temp_c_neu(kc,j,i) = temp_c_neu(kc,jj,ii)   ! set cold-ice temperature
         age_c_neu(kc,j,i)  = age_c_neu(kc,jj,ii)    ! set cold-ice age
      end do

      do kt=0,KTMAX
         omega_t_neu(kt,j,i) = omega_t_neu(kt,jj,ii) ! set temp.-ice water content
         age_t_neu(kt,j,i)   = age_t_neu(kt,jj,ii)   ! set temp.-ice age
      end do

      do kr=0,KRMAX
         temp_r_neu(kr,j,i) = temp_r_neu(kr,jj,ii)   ! set bedrock temperature
      end do

      n_cts_neu(j,i) = min(n_cts_neu(jj,ii),0)   ! temperate layer excluded
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)
      zm_neu(j,i)  = zb(j,i)

   else   ! maske(j,i).eq.1,2 (ice-free land or sea point)

      n_cts_neu(j,i) = -1
      zm_neu(j,i)    = zb(j,i)
      H_c_neu(j,i)   = H_c(j,i)
      H_t_neu(j,i)   = H_t(j,i)

      call calc_temp_r(atr1, alb1, i, j)

   end if

end do

!-------- Smoothing of H_t_neu with numerical diffusion --------

!  ------ Volume of temperate ice without smoothing

Vol_t = 0.0d0
do i=0, IMAX   ! extended to domain margins (22.1.02 -> V1.1)
do j=0, JMAX   ! extended to domain margins (22.1.02 -> V1.1)
   if (n_cts_neu(j,i).eq.1) then
      Vol_t = Vol_t + H_t_neu(j,i)*area(j,i)
   end if
end do
end do

!  ------ Smoothing

do i=1, IMAX-1
do j=1, JMAX-1
   if (n_cts_neu(j,i).ne.-1) then

      dH_t_smooth(j,i) = NUMDIFF_H_T* ( -4.0d0*H_t_neu(j,i) &
                             +H_t_neu(j,i+1)+H_t_neu(j,i-1) &
                             +H_t_neu(j+1,i)+H_t_neu(j-1,i) )
      if (dH_t_smooth(j,i).gt.0.001d0) n_cts_neu(j,i) = 1

   end if
end do
end do

do i=1, IMAX-1
do j=1, JMAX-1
   if (n_cts_neu(j,i).eq.1) then
      H_t_neu(j,i) = H_t_neu(j,i) + dH_t_smooth(j,i)
   end if
end do
end do

!  ------ Volume of temperate ice with smoothing

Vol_t_smooth = 0.0d0
do i=0, IMAX   ! extended to domain margins (22.1.02 -> V1.1)
do j=0, JMAX   ! extended to domain margins (22.1.02 -> V1.1)
   if (n_cts_neu(j,i).eq.1) then
      Vol_t_smooth = Vol_t_smooth + H_t_neu(j,i)*area(j,i)
   end if
end do
end do

!  ------ Correction so that volume is not changed by the smoothing

if (Vol_t_smooth.gt.0.0d0) then

   korrfakt_t = Vol_t/Vol_t_smooth
   do i=0, IMAX   ! extended to domain margins (22.1.02 -> V1.1)
   do j=0, JMAX   ! extended to domain margins (22.1.02 -> V1.1)
      if (n_cts_neu(j,i).eq.1) then
         H_t_neu(j,i) = H_t_neu(j,i)*korrfakt_t
!               zm_neu(j,i)  = zb(j,i) + H_t_neu(j,i)
!               H_c_neu(j,i) = zs(j,i) - zm_neu(j,i)
      end if
   end do
   end do

end if

!-------- Numerical time lag for evolution of H_t_neu --------

time_lag_cts = TAU_CTS*YEAR_SEC   ! yr --> s

do i=0, IMAX   ! extended to domain margins (22.1.02 -> V1.1)
do j=0, JMAX   ! extended to domain margins (22.1.02 -> V1.1)

   if (n_cts_neu(j,i).eq.1) then

      H_t_neu(j,i) = ( time_lag_cts*H_t(j,i) &
                       + dtime_temp*H_t_neu(j,i) ) &
                     /(time_lag_cts+dtime_temp)

      zm_neu(j,i)  = zb(j,i) + H_t_neu(j,i)
      H_c_neu(j,i) = zs(j,i) - zm_neu(j,i)

   end if

end do
end do

return
end
