!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ t e m p _ r

!   Purpose    :  Computation of temperature for ice-free column.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_temp_r(atr1, alb1, i, j)

use sico_variables

implicit none
integer i, j, kc, kt, kr
double precision atr1, alb1, ctr1, clb1
double precision lgs_a0(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_a1(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_a2(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_x(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX), &
                 lgs_b(0:KCMAX+KTMAX+KRMAX+IMAX+JMAX)

!-------- Abbreviations --------

ctr1 = atr1
clb1 = alb1*q_geo(j,i)

!-------- Set up the equations for the bedrock temperature --------

kr=0
lgs_a1(kr) = 1.0d0
lgs_a2(kr) = -1.0d0
lgs_b(kr)    = clb1

#if Q_LITHO==1
!   (coupled heat-conducting bedrock)

do kr=1, KRMAX-1
   lgs_a0(kr) = - ctr1
   lgs_a1(kr) = 1.0d0 + 2.0d0*ctr1
   lgs_a2(kr) = - ctr1
   lgs_b(kr)    = temp_r(kr,j,i)
end do

#elif Q_LITHO==0
!   (no coupled heat-conducting bedrock)

do kr=1, KRMAX-1
   lgs_a0(kr) = 1.0d0
   lgs_a1(kr) = 0.0d0
   lgs_a2(kr) = -1.0d0
   lgs_b(kr)  = 2.0d0*clb1
end do

#endif

kr=KRMAX
lgs_a0(kr) = 0.0d0
lgs_a1(kr) = 1.0d0
lgs_b(kr)   = temp_s(j,i)

!-------- Solve system of linear equations --------

call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, KRMAX)

!-------- Assign the result --------

do kr=0, KRMAX
   temp_r_neu(kr,j,i) = lgs_x(kr)
end do

!-------- Water content and age in the non-existing temperate layer --------

do kt=0, KTMAX
   omega_t_neu(kt,j,i) = 0.0d0
   age_t_neu(kt,j,i)   = 0.0d0
end do

!-------- Temperature and age in the non-existing cold layer --------

do kc=0, KCMAX
   temp_c_neu(kc,j,i) = temp_s(j,i)
   age_c_neu(kc,j,i)  = 0.0d0
end do

return
end
