!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ t o p

!   Purpose    :  Computation of the topography (including gradients).

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_top(time, time_init, z_sl, z_mar, dtime, dxi, deta, &
   dzeta_t, mean_accum, azs2, azs3, &
   ii, jj, nn, itercount, iter_wss, zs_switch)

use sico_variables

implicit none
integer i, j, kc, kt
integer in, jn
integer k, nc, nr
integer ier
integer :: itercount, iter_wss
integer, parameter :: nmax=(IMAX+1)*(JMAX+1), n_sprs=10*(IMAX+1)*(JMAX+1)
integer, allocatable, dimension(:) :: lgs_ija
integer ii((IMAX+1)*(JMAX+1)), jj((IMAX+1)*(JMAX+1)), nn(0:JMAX,0:IMAX)
double precision time, time_init, z_sl, z_mar, dtime, dxi, deta, dzeta_t
double precision mean_accum, eps_sor
double precision azs2, azs3, czs2(0:JMAX,0:IMAX), czs3(0:JMAX,0:IMAX)
double precision tldt_inv, rho_rho_a
double precision dtime_inv, dxi_inv, deta_inv
double precision dt_dxi, dt_deta
double precision Vol, Vol_smooth, korrfakt, dzs_smooth(0:JMAX,0:IMAX), &
       H_help(0:JMAX,0:IMAX)
double precision, allocatable, dimension(:) :: lgs_a0, lgs_a1, lgs_a2, &
                                               lgs_sa, lgs_x, lgs_b
logical zs_switch

double precision, parameter :: eps_vol = 1.0d0

double precision, parameter :: H_isol_max = 1.0d+03
!                 ! Thickness limit of isolated glaciated points

tldt_inv = 1.0d0/(TIME_LAG+dtime)
rho_rho_a = RHO/RHO_A

dtime_inv = 1.0d0/dtime
dxi_inv   = 1.0d0/dxi
deta_inv  = 1.0d0/deta

dt_dxi  = dtime/dxi
dt_deta = dtime/deta

!-------- Compute zb_neu and its time derivative --------

#if REBOUND==0

do i=0, IMAX
do j=0, JMAX
   zb_neu(j,i)   = zb(j,i)
   dzb_dtau(j,i) = (zb_neu(j,i)-zb(j,i))*dtime_inv
end do
end do

#elif REBOUND==1

do i=0, IMAX
do j=0, JMAX
   zb_neu(j,i)   = tldt_inv*( TIME_LAG*zb(j,i) &
                   + dtime*(zb0(j,i)-FRAC_LLRA*rho_rho_a*(H_c(j,i)+H_t(j,i))) )
   dzb_dtau(j,i) = (zb_neu(j,i)-zb(j,i))*dtime_inv
end do
end do

#elif REBOUND==2

if ((mod(itercount, iter_wss)==0).or.(itercount==1)) then
   write(6,*) ' -------- Computation of wss'
   call calc_elra(z_sl, dxi, deta)   ! Update of the steady-state displacement
                                     ! of the lithosphere (wss)
end if

do i=0, IMAX
do j=0, JMAX
   zb_neu(j,i) = tldt_inv * ( TIME_LAG*zb(j,i) + dtime*(zb0(j,i)-wss(j,i)) )
   dzb_dtau(j,i) = (zb_neu(j,i)-zb(j,i))*dtime_inv
end do
end do

#endif

!-------- Compute zs_neu --------

!  ------ Explicit scheme

#if CALCZS==0

do i=1, IMAX-1
do j=1, JMAX-1

#if MARGIN==1
   if (maske(j,i).ne.2) then   ! no sea point
#endif

      zs_neu(j,i) = zs(j,i) &
            -dt_dxi*insq_g11_g(j,i)*insq_g22_g(j,i) &
             *(qx(j,i)*sq_g22_sgx(j,i) &
              -qx(j,i-1)*sq_g22_sgx(j,i-1)) &
            -dt_deta*insq_g11_g(j,i)*insq_g22_g(j,i) &
             *(qy(j,i)*sq_g11_sgy(j,i) &
              -qy(j-1,i)*sq_g11_sgy(j-1,i)) &
            +dtime*( as_perp(j,i)+dzb_dtau(j,i)-Q_b_tot(j,i) )

#if MARGIN==1
   else   ! maske(j,i).eq.2, sea point
      zs_neu(j,i) = zb_neu(j,i)
   end if
#endif

end do
end do

do i=0, IMAX
   zs_neu(0,i)    = zb_neu(0,i)
   zs_neu(JMAX,i) = zb_neu(JMAX,i)
end do

do j=0, JMAX
   zs_neu(j,0)    = zb_neu(j,0)
   zs_neu(j,IMAX) = zb_neu(j,IMAX)
end do

!  ------ ADI scheme / over-implicit ADI scheme

#elif ( CALCZS==1 || CALCZS==2 )

!    ---- Abbreviations

do i=0, IMAX-1
do j=0, JMAX
   czs2(j,i) = azs2*0.5d0*(h_diff(j,i)+h_diff(j,i+1)) &
               *sq_g22_sgx(j,i)*insq_g11_sgx(j,i)
end do
end do

do i=0, IMAX
do j=0, JMAX-1
   czs3(j,i) = azs3*0.5d0*(h_diff(j,i)+h_diff(j+1,i)) &
               *sq_g11_sgy(j,i)*insq_g22_sgy(j,i)
end do
end do

!    ---- Switch direction for ADI scheme

zs_switch = .not.zs_switch

!    ---- zs_neu with ADI scheme, implicit in i-direction

if (zs_switch) then   ! zs_switch = .true.

   allocate(lgs_a0(0:IMAX), lgs_a1(0:IMAX), lgs_a2(0:IMAX), &
            lgs_x(0:IMAX), lgs_b(0:IMAX))

!        Boundaries j=0, j=JMAX:

   do i=0, IMAX
      zs_neu(0,i)    = zb_neu(0,i)
      zs_neu(JMAX,i) = zb_neu(JMAX,i)
   end do

   do j=1, JMAX-1

      i=0
      lgs_a1(i) = 1.0d0
      lgs_a2(i) = 0.0d0
      lgs_b(i)  = zb_neu(j,i)

      do i=1, IMAX-1

#if MARGIN==1
         if (maske(j,i).ne.2) then   ! no sea point
#endif

#if CALCZS==1
!               ! ADI scheme
            lgs_a0(i) = -czs2(j,i-1) &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a1(i) = 1.0d0 + (czs2(j,i)+czs2(j,i-1)) &
                              *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a2(i) = -czs2(j,i) &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_b(i)  = zs(j,i) + dtime*( as_perp(j,i) &
                        +dzb_dtau(j,i)-Q_b_tot(j,i) ) &
                        + ( czs3(j,i)*(zs(j+1,i)-zs(j,i)) &
                           -czs3(j-1,i)*(zs(j,i)-zs(j-1,i)) ) &
                          *insq_g11_g(j,i)*insq_g22_g(j,i)
#elif CALCZS==2
!               ! over-implicit ADI scheme
            lgs_a0(i) = -czs2(j,i-1)*OVI_WEIGHT &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a1(i) = 1.0d0 + (czs2(j,i)+czs2(j,i-1)) &
                              *OVI_WEIGHT &
                              *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a2(i) = -czs2(j,i)*OVI_WEIGHT &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_b(i)  = zs(j,i) + dtime*( as_perp(j,i) &
                        +dzb_dtau(j,i)-Q_b_tot(j,i) ) &
                        + ( czs3(j,i)*(zs(j+1,i)-zs(j,i)) &
                           -czs3(j-1,i)*(zs(j,i)-zs(j-1,i)) ) &
                          *insq_g11_g(j,i)*insq_g22_g(j,i) &
                        + ( czs2(j,i)*(zs(j,i+1)-zs(j,i)) &
                           -czs2(j,i-1)*(zs(j,i)-zs(j,i-1)) ) &
                          *(1.0d0-OVI_WEIGHT) &
                          *insq_g11_g(j,i)*insq_g22_g(j,i)
#endif

#if MARGIN==1
         else   ! maske(j,i).eq.2, sea point

            lgs_a0(i) = 0.0d0
            lgs_a1(i) = 1.0d0
            lgs_a2(i) = 0.0d0
            lgs_b(i)  = zb_neu(j,i)

         end if
#endif

      end do

      i=IMAX
      lgs_a0(i) = 0.0d0
      lgs_a1(i) = 1.0d0
      lgs_b(i)  = zb_neu(j,i)

      call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, IMAX)

      do i=0, IMAX
         zs_neu(j,i) = lgs_x(i)
      end do

   end do

   deallocate(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b)

!    ---- zs_neu with ADI scheme, implicit in j-direction

else   ! zs_switch = .false.

   allocate(lgs_a0(0:JMAX), lgs_a1(0:JMAX), lgs_a2(0:JMAX), &
            lgs_x(0:JMAX), lgs_b(0:JMAX))

!        Boundaries i=0, i=IMAX:

   do j=0, JMAX
      zs_neu(j,0)    = zb_neu(j,0)
      zs_neu(j,IMAX) = zb_neu(j,IMAX)
   end do

   do i=1, IMAX-1

      j=0
      lgs_a1(j) = 1.0d0
      lgs_a2(j) = 0.0d0
      lgs_b(j)  = zb_neu(j,i)

      do j=1, JMAX-1

#if MARGIN==1
         if (maske(j,i).ne.2) then   ! no sea point
#endif

#if CALCZS==1
!               ! ADI scheme
            lgs_a0(j) = -czs3(j-1,i) &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a1(j) = 1.0d0 + (czs3(j,i)+czs3(j-1,i)) &
                              *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a2(j) = -czs3(j,i) &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_b(j)  = zs(j,i) + dtime*( as_perp(j,i) &
                        +dzb_dtau(j,i)-Q_b_tot(j,i) ) &
                        + ( czs2(j,i)*(zs(j,i+1)-zs(j,i)) &
                           -czs2(j,i-1)*(zs(j,i)-zs(j,i-1)) ) &
                          *insq_g11_g(j,i)*insq_g22_g(j,i)
#elif CALCZS==2
!               ! over-implicit ADI scheme
            lgs_a0(j) = -czs3(j-1,i)*OVI_WEIGHT &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a1(j) = 1.0d0 + (czs3(j,i)+czs3(j-1,i)) &
                              *OVI_WEIGHT &
                              *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_a2(j) = -czs3(j,i)*OVI_WEIGHT &
                         *insq_g11_g(j,i)*insq_g22_g(j,i)
            lgs_b(j)  = zs(j,i) + dtime*( as_perp(j,i) &
                        +dzb_dtau(j,i)-Q_b_tot(j,i) ) &
                        + ( czs2(j,i)*(zs(j,i+1)-zs(j,i)) &
                           -czs2(j,i-1)*(zs(j,i)-zs(j,i-1)) ) &
                          *insq_g11_g(j,i)*insq_g22_g(j,i) &
                        + ( czs3(j,i)*(zs(j+1,i)-zs(j,i)) &
                           -czs3(j-1,i)*(zs(j,i)-zs(j-1,i)) ) &
                          *(1.0d0-OVI_WEIGHT) &
                          *insq_g11_g(j,i)*insq_g22_g(j,i)
#endif

#if MARGIN==1
         else   ! maske(j,i).eq.2, sea point

            lgs_a0(j) = 0.0d0
            lgs_a1(j) = 1.0d0
            lgs_a2(j) = 0.0d0
            lgs_b(j)  = zb_neu(j,i)

         end if
#endif

      end do

      j=JMAX
      lgs_a0(j) = 0.0d0
      lgs_a1(j) = 1.0d0
      lgs_b(j)  = zb_neu(j,i)

      call tri_sle(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b, JMAX)

      do j=0, JMAX
         zs_neu(j,i) = lgs_x(j)
      end do

   end do

   deallocate(lgs_a0, lgs_a1, lgs_a2, lgs_x, lgs_b)

end if

!  ------ Over-implicit scheme

#elif CALCZS==3

!    ---- Abbreviations

do i=0, IMAX-1
do j=0, JMAX
   czs2(j,i) = azs2*0.5d0*(h_diff(j,i)+h_diff(j,i+1)) &
               *sq_g22_sgx(j,i)*insq_g11_sgx(j,i)
end do
end do

do i=0, IMAX
do j=0, JMAX-1
   czs3(j,i) = azs3*0.5d0*(h_diff(j,i)+h_diff(j+1,i)) &
               *sq_g11_sgy(j,i)*insq_g22_sgy(j,i)
end do
end do

!    ---- Assemble system of linear equations
!         [matrix in row-indexed compact storage,
!         represented by arrays lgs_sa (values) and lgs_ija (indices);
!         see Press et al., 'Numerical Recipes in Fortran 77']

allocate(lgs_sa(n_sprs), lgs_ija(n_sprs), lgs_x(nmax), lgs_b(nmax))

lgs_sa  = 0.0d0
lgs_ija = 0

k = nmax+1

lgs_ija(1)=nmax+2   ! index to 1st row off-diagonal element, if any

do nr=1, nmax   ! loop over rows

   i = ii(nr)
   j = jj(nr)

!      -- Matrix

   if ( &
        (i.ne.0).and.(i.ne.IMAX).and.(j.ne.0).and.(j.ne.JMAX) &   ! inner point
#if MARGIN==1
        .and.(maske(j,i).ne.2) &   ! no sea point
#endif
      ) then

      lgs_sa(nr) = 1.0d0 &
                   + (czs2(j,i)+czs2(j,i-1)+czs3(j,i)+czs3(j-1,i)) &
                     *OVI_WEIGHT &
                     *insq_g11_g(j,i)*insq_g22_g(j,i)   ! diagonal element

      nc = nn(j-1,i)   ! yields smallest nc
      k  = k+1
      if (k.gt.n_sprs) stop 'n_sprs too small!'
      lgs_sa(k) = -czs3(j-1,i)*OVI_WEIGHT &
                   *insq_g11_g(j,i)*insq_g22_g(j,i)   ! off-diagonal element
      lgs_ija(k) = nc

      nc = nn(j,i-1)   ! yields second smallest nc
      k  = k+1
      if (k.gt.n_sprs) stop 'n_sprs too small!'
      lgs_sa(k) = -czs2(j,i-1)*OVI_WEIGHT &
                   *insq_g11_g(j,i)*insq_g22_g(j,i)   ! off-diagonal element
      lgs_ija(k) = nc

      nc = nn(j,i+1)   ! yields second largest nc
      k  = k+1
      if (k.gt.n_sprs) stop 'n_sprs too small!'
      lgs_sa(k) = -czs2(j,i)*OVI_WEIGHT &
                   *insq_g11_g(j,i)*insq_g22_g(j,i)   ! off-diagonal element
      lgs_ija(k) = nc

      nc = nn(j+1,i)   ! yields largest nc
      k  = k+1
      if (k.gt.n_sprs) stop 'n_sprs too small!'
      lgs_sa(k) = -czs3(j,i)*OVI_WEIGHT &
                   *insq_g11_g(j,i)*insq_g22_g(j,i)   ! off-diagonal element
      lgs_ija(k) = nc

   else   ! boundary condition

      lgs_sa(nr) = 1.0d0   ! diagonal element
      


   end if

   lgs_ija(nr+1)=k+1   ! as each row is completed, store index to next
   
   ! -- Right-hand side

   if ( &
        (i.ne.0).and.(i.ne.IMAX).and.(j.ne.0).and.(j.ne.JMAX) &   ! inner point
#if MARGIN==1
        .and.(maske(j,i).ne.2) &   ! no sea point
#endif
      ) then

      lgs_b(nr) = zs(j,i) + dtime*(as_perp(j,i)+dzb_dtau(j,i)-Q_b_tot(j,i)) &
                          + ( czs2(j,i)*(zs(j,i+1)-zs(j,i)) &
                             -czs2(j,i-1)*(zs(j,i)-zs(j,i-1)) &
                             +czs3(j,i)*(zs(j+1,i)-zs(j,i)) &
                             -czs3(j-1,i)*(zs(j,i)-zs(j-1,i)) ) &
                            *(1.0d0-OVI_WEIGHT) &
                            *insq_g11_g(j,i)*insq_g22_g(j,i)

   else   ! boundary condition

      lgs_b(nr) = zb_neu(j,i)

   end if

!      -- Initial guess for solution vector

   lgs_x(nr) = zs(j,i)   ! old surface topography

end do

!    ---- Solve system of linear equations

eps_sor = 1.0d-05*mean_accum*dtime   ! convergence parameter

call sor_sprs(lgs_sa, lgs_ija, lgs_b, n_sprs, nmax, &
              OMEGA_SOR, lgs_x, eps_sor, ier)

do nr=1, nmax
   i = ii(nr)
   j = jj(nr)
   zs_neu(j,i) = lgs_x(nr)
end do

deallocate(lgs_sa, lgs_ija, lgs_x, lgs_b)

#endif

!-------- Update of the mask --------

#if ZS_EVOL==1

do i=0, IMAX
do j=0, JMAX

   if (maske(j,i) <= 1) then   ! land point

      if ((zs_neu(j,i)-zb_neu(j,i)) >= H_MIN) then
         maske(j,i) = 0   ! glaciated land
      else
         maske(j,i) = 1   ! ice-free land
      end if

#if MARGIN>=2

   else if (maske(j,i) >= 2) then   ! sea or floating ice

      if (zb0(j,i) < z_mar) then
         maske(j,i) = 2   ! sea point
      else if ((zs_neu(j,i)-zb_neu(j,i)) >= H_MIN) then
         maske(j,i) = 3   ! floating ice
      else
         maske(j,i) = 2   ! sea point
      end if

#endif

   end if

#if MARGIN>=2

!  ------ Remove floating ice resulting from previously grounded ice

   if ( (maske_help(j,i) == 0).and.(maske(j,i) == 3) ) maske(j,i) = 2

#endif

end do
end do

#endif

!-------- Smoothing of zs_neu with numerical diffusion --------

!  ------ Unsmoothed thickness

do i=0, IMAX
do j=0, JMAX
   H_help(j,i) = zs_neu(j,i)-zb_neu(j,i)
end do
end do

!  ------ Unsmoothed ice volume (grounded ice only)

Vol = 0.0d0
do i=0, IMAX
do j=0, JMAX
   if (maske(j,i).eq.0) then
      Vol = Vol + H_help(j,i)*area(j,i)
   end if
end do
end do

!  ------ Smoothing

do i=1, IMAX-1
do j=1, JMAX-1
   dzs_smooth(j,i) = NUMDIFF_ZS* ( -4.0d0*zs_neu(j,i) &
                       +zs_neu(j,i+1)+zs_neu(j,i-1) &
                       +zs_neu(j+1,i)+zs_neu(j-1,i) )
end do
end do

do i=1, IMAX-1
do j=1, JMAX-1
   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
      zs_neu(j,i)  = zs_neu(j,i) + dzs_smooth(j,i)
      if ((zs_neu(j,i)-zb_neu(j,i)).lt.H_MIN) &
         zs_neu(j,i) = zb_neu(j,i)+1.0d0
!                ! If smoothing makes an originally glaciated point
!                ! ice-free, the thickness is set to 1 m
   end if
end do
end do

!  ------ Thickness after smoothing

do i=0, IMAX
do j=0, JMAX
   H_help(j,i) = zs_neu(j,i)-zb_neu(j,i)
end do
end do

!  ------ Ice volume after smoothing (grounded ice only)

Vol_smooth = 0.0d0
do i=0, IMAX
do j=0, JMAX
   if (maske(j,i).eq.0) then
      Vol_smooth = Vol_smooth + H_help(j,i)*area(j,i)
   end if
end do
end do

!  ------ Correction so that volume is not changed by the smoothing

korrfakt = max(Vol,eps_vol)/max(Vol_smooth,eps_vol)
do i=1, IMAX-1
do j=1, JMAX-1
   H_help(j,i) = H_help(j,i)*korrfakt
   zs_neu(j,i) = zb_neu(j,i)+H_help(j,i)
end do
end do

#if ZS_EVOL==0
zs_neu = zs   ! newly computed surface topography is discarded
#endif

!-------- Compute further quantities --------

do i=0, IMAX
do j=0, JMAX

   if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
!                                ! glaciated land or floating ice --------

!  ------ Limit the thickness of isolated glaciated points to H_isol_max

      if ( ((maske(j,i+1).eq.1).or.(maske(j,i+1).eq.2)) &
           .and. ((maske(j,i-1).eq.1).or.(maske(j,i-1).eq.2)) &
           .and. ((maske(j+1,i).eq.1).or.(maske(j+1,i).eq.2)) &
           .and. ((maske(j-1,i).eq.1).or.(maske(j-1,i).eq.2)) &
         ) then   ! all nearest neighbours ice-free
         zs_neu(j,i) = min(zs_neu(j,i),(zb_neu(j,i)+H_isol_max))
      end if

!  ------ Further stuff

      if (n_cts(j,i).eq.1) then
         H_t_neu(j,i) = H_t(j,i)*(zs_neu(j,i)-zb_neu(j,i)) &
                                /(zs(j,i)-zb(j,i))
         zm_neu(j,i)  = zb_neu(j,i)+H_t_neu(j,i)
         H_c_neu(j,i) = zs_neu(j,i)-zm_neu(j,i)
      else
         H_t_neu(j,i) = 0.0d0
         zm_neu(j,i)  = zb_neu(j,i)
         H_c_neu(j,i) = zs_neu(j,i)-zm_neu(j,i)
      end if

   else   ! maske(j,i).eq.1,2

      zs_neu(j,i) = zb_neu(j,i)
      zm_neu(j,i) = zb_neu(j,i)
      H_c_neu(j,i) = 0.0d0
      H_t_neu(j,i) = 0.0d0

   end if

   dzs_dtau(j,i)  = (zs_neu(j,i)-zs(j,i))*dtime_inv
   dzm_dtau(j,i)  = dH_t_dtau(j,i)+dzb_dtau(j,i)
   dH_c_dtau(j,i) = dzs_dtau(j,i)-dzm_dtau(j,i)

end do
end do

!-------- New gradients --------

#if TOPOGRAD==0
call topograd_1(dxi, deta, 2)
#elif TOPOGRAD==1
call topograd_2(dxi, deta, 2)
#endif

!-------- Compute the volume flux across the CTS, am_perp --------

#if CALCMOD==1

do i=1, IMAX-1
do j=1, JMAX-1

   if ( ((maske(j,i).eq.0).or.(maske(j,i).eq.3)) &
        .and.(n_cts(j,i).eq.1)) then

      am_perp_st(j,i) = &
                0.5d0*(vx_c(0,j,i)+vx_c(0,j,i-1))*dzm_dxi_g(j,i) &
              + 0.5d0*(vy_c(0,j,i)+vy_c(0,j-1,i))*dzm_deta_g(j,i) &
              - 0.5d0*(vz_c(0,j,i)+vz_t(KTMAX-1,j,i))
      am_perp(j,i) = am_perp_st(j,i) + dzm_dtau(j,i)

   else
      am_perp_st(j,i) = 0.0d0
      am_perp(j,i)    = 0.0d0
   end if

end do
end do

#endif

end subroutine calc_top
