!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ v x y _ b

!   Purpose    :  Computation of the basal horizontal velocity vx_b, vy_b.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_vxy_b(z_sl)

use sico_variables

implicit none
double precision, intent(in) :: z_sl

integer :: i, j
integer :: r_smw_1, s_smw_1, r_smw_2, s_smw_2
integer, dimension(0:JMAX,0:IMAX) :: p_weert, q_weert
double precision, dimension(0:JMAX,0:IMAX) :: c_slide, gamma_slide_inv
double precision :: gamma_slide_inv_1, gamma_slide_inv_2
double precision :: smw_coeff_1, smw_coeff_2
double precision, dimension(0:JMAX,0:IMAX) :: p_b, N_b, tau_b
double precision :: cvxy1, cvxy1a
double precision :: temp_diff
double precision :: vh_max
double precision :: year_sec_inv
logical, dimension(0:JMAX,0:IMAX) :: sub_melt_flag

!-------- Sliding-law coefficients --------

year_sec_inv = 1.0d0/YEAR_SEC

#if ( defined(ANT) \
      || defined(SCAND) \
      || defined(NHEM) \
      || defined(TIBET) \
      || defined(NMARS) \
      || defined(SMARS) \
      || defined(EMTP2SGE) )
         ! one sliding law for the entire modelling domain

gamma_slide_inv_1 = 1.0d0/max(GAMMA_SLIDE, eps)

do i=0, IMAX
do j=0, JMAX
   c_slide(j,i)         = C_SLIDE*year_sec_inv
   p_weert(j,i)         = P_WEERT
   q_weert(j,i)         = Q_WEERT
   gamma_slide_inv(j,i) = gamma_slide_inv_1
   sub_melt_flag(j,i)   = (GAMMA_SLIDE >= eps)
end do
end do

#elif ( defined(HEINO) )
        ! ISMIP HEINO,
        ! different sliding laws for hard rock and soft sediment

gamma_slide_inv_1 = 1.0d0/max(GAMMA_SLIDE, eps)
gamma_slide_inv_2 = 1.0d0/max(GAMMA_SLIDE_SEDI, eps)

do i=0, IMAX
do j=0, JMAX

   if (maske_sedi(j,i) /= 7) then   ! no soft sediment
      c_slide(j,i)         = C_SLIDE*year_sec_inv
      p_weert(j,i)         = P_WEERT
      q_weert(j,i)         = Q_WEERT
      gamma_slide_inv(j,i) = gamma_slide_inv_1
      sub_melt_flag(j,i)   = (GAMMA_SLIDE >= eps)
   else   ! maske_sedi(j,i) == 7, soft sediment
      c_slide(j,i)         = C_SLIDE_SEDI*year_sec_inv
      p_weert(j,i)         = P_WEERT_SEDI
      q_weert(j,i)         = Q_WEERT_SEDI
      gamma_slide_inv(j,i) = gamma_slide_inv_2
      sub_melt_flag(j,i)   = (GAMMA_SLIDE_SEDI >= eps)
   end if

end do
end do

#elif ( defined(GRL) && ICE_STREAM==1 )
        ! Greenland without ice streams,
        ! one sliding law for the entire modelling domain,
        ! with surface meltwater effect

gamma_slide_inv_1 = 1.0d0/max(GAMMA_SLIDE, eps)

r_smw_1     = R_SMW
s_smw_1     = S_SMW
smw_coeff_1 = SMW_COEFF*YEAR_SEC**s_smw_1

do i=0, IMAX
do j=0, JMAX
   c_slide(j,i)         = C_SLIDE*year_sec_inv &
                                 *(1.0d0+smw_coeff_1*runoff(j,i)**s_smw_1 &
                                   /max((H_c(j,i)+H_t(j,i))**r_smw_1, eps))
   p_weert(j,i)         = P_WEERT
   q_weert(j,i)         = Q_WEERT
   gamma_slide_inv(j,i) = gamma_slide_inv_1
   sub_melt_flag(j,i)   = (GAMMA_SLIDE >= eps)
end do
end do

#elif ( defined(GRL) && ICE_STREAM==2 )
        ! Greenland with ice streams,
        ! different sliding laws for hard rock and ice streams

gamma_slide_inv_1 = 1.0d0/max(GAMMA_SLIDE, eps)
gamma_slide_inv_2 = 1.0d0/max(GAMMA_SLIDE_SEDI, eps)

r_smw_1     = R_SMW
s_smw_1     = S_SMW
smw_coeff_1 = SMW_COEFF     *YEAR_SEC**s_smw_1
r_smw_2     = R_SMW_SEDI
s_smw_2     = S_SMW_SEDI
smw_coeff_2 = SMW_COEFF_SEDI*YEAR_SEC**s_smw_2

do i=0, IMAX
do j=0, JMAX

   if (maske_sedi(j,i) /= 7) then   ! no ice stream
      c_slide(j,i)         = C_SLIDE*year_sec_inv &
                                    *(1.0d0+smw_coeff_1*runoff(j,i)**s_smw_1 &
                                      /max((H_c(j,i)+H_t(j,i))**r_smw_1, eps))
      p_weert(j,i)         = P_WEERT
      q_weert(j,i)         = Q_WEERT
      gamma_slide_inv(j,i) = gamma_slide_inv_1
      sub_melt_flag(j,i)   = (GAMMA_SLIDE >= eps)
   else   ! maske_sedi(j,i) == 7, ice stream
      c_slide(j,i)         = C_SLIDE_SEDI*year_sec_inv &
                                         *(1.0d0+smw_coeff_2*runoff(j,i)**s_smw_2 &
                                           /max((H_c(j,i)+H_t(j,i))**r_smw_2, eps))
      p_weert(j,i)         = P_WEERT_SEDI
      q_weert(j,i)         = Q_WEERT_SEDI
      gamma_slide_inv(j,i) = gamma_slide_inv_2
      sub_melt_flag(j,i)   = (GAMMA_SLIDE_SEDI >= eps)
   end if

end do
end do

#else

stop ' Subroutine calc_vxy_b: No valid domain specified!'

#endif

!-------- Computation of basal stresses --------

!  ------ Basal pressure p_b, reduced pressure N_b

do i=0, IMAX
do j=0, JMAX

   if ( (maske(j,i) == 0).or.(maske(j,i) == 3) ) then
                                    ! glaciated land or floating ice

      p_b(j,i) = RHO*G*(H_c(j,i)+H_t(j,i))
      N_b(j,i) = p_b(j,i) - RHO_SW*G*max((z_sl-zb(j,i)), 0.0d0)
      N_b(j,i) = max(N_b(j,i), 0.2d0*p_b(j,i))   
                 ! in order to avoid very small values, which may lead to
                 ! huge sliding velocities

   else   ! maske(j,i) == (1 or 2)

      p_b(j,i) = 0.0d0
      N_b(j,i) = 0.0d0

   end if

end do
end do

!  ------ Absolute value of the basal shear stress, tau_b

do i=0, IMAX
do j=0, JMAX
      tau_b(j,i) = p_b(j,i)*(dzs_dxi_g(j,i)**2+dzs_deta_g(j,i)**2)**0.5d0
end do
end do

!-------- Computation of d_help_b (defined on the grid points (i,j)) --------

do i=0, IMAX
do j=0, JMAX

   if ( (maske(j,i) == 0).or.(maske(j,i) == 3) ) then
                                 ! glaciated land or floating ice

!  ------ Abbreviations

#if SLIDE_LAW==1
      cvxy1 = c_slide(j,i) &
              * (tau_b(j,i)**(p_weert(j,i)-1)/p_b(j,i)**q_weert(j,i)) * p_b(j,i)
              ! Basal sliding at pressure melting
#elif SLIDE_LAW==2
      cvxy1 = c_slide(j,i) &
              * (tau_b(j,i)**(p_weert(j,i)-1)/N_b(j,i)**q_weert(j,i)) * p_b(j,i)
              ! Basal sliding at pressure melting
#endif

!  ------ d_help_b

      if ( (maske(j,i) == 0).and.(n_cts(j,i) == -1) ) then
                                       ! grounded ice, cold ice base

         if (sub_melt_flag(j,i)) then
            temp_diff = max((temp_c_m(0,j,i)-temp_c(0,j,i)), 0.0d0)
            cvxy1a    = exp(-gamma_slide_inv(j,i)*temp_diff)  ! sub-melt sliding
         else
            cvxy1a    = 0.0d0   ! no sub-melt sliding
         end if

         d_help_b(j,i) = cvxy1*cvxy1a

      else if ( (maske(j,i) == 3).and.(n_cts(j,i) == -1) ) then
                                       ! floating ice, cold ice base
         d_help_b(j,i) = cvxy1    ! basal sliding
                                  ! (always pressure-melting conditions assumed,
                                  ! independent of actual basal temperature)

      else if (n_cts(j,i) == 0) then   ! temperate ice base

         d_help_b(j,i) = cvxy1   ! basal sliding
                                 ! (pressure-melting conditions)

      else   ! n_cts(j,i) == 1, temperate ice layer

         d_help_b(j,i) = cvxy1   ! basal sliding
                                 ! (pressure-melting conditions)

      end if

   else   ! maske(j,i) == (1 oder 2); ice-free land or sea

      d_help_b(j,i) = 0.0d0

   end if

end do
end do

!-------- Computation of vx_b (defined at (i+1/2,j)) --------

do i=0, IMAX-1
do j=1, JMAX-1
   vx_b(j,i) = -0.5d0*(d_help_b(j,i)+d_help_b(j,i+1))*dzs_dxi(j,i)
end do
end do

!-------- Computation of vy_b (defined at (i,j+1/2)) --------

do i=1, IMAX-1
do j=0, JMAX-1
   vy_b(j,i) = -0.5d0*(d_help_b(j,i)+d_help_b(j+1,i))*dzs_deta(j,i)
end do
end do

!-------- Limitation of computed vx_b, vy_b to the interval
!         [-VH_MAX, VH_MAX] --------

vh_max = VH_MAX/YEAR_SEC

do i=0, IMAX
do j=0, JMAX
   vx_b(j,i) = max(vx_b(j,i), -vh_max)
   vx_b(j,i) = min(vx_b(j,i),  vh_max)
   vy_b(j,i) = max(vy_b(j,i), -vh_max)
   vy_b(j,i) = min(vy_b(j,i),  vh_max)
end do
end do

end subroutine calc_vxy_b
