!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ v z

!   Purpose    :  Computation of the vertical velocity vz.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_vz(dxi, deta, dzeta_c, dzeta_t, avz3)

use sico_variables

implicit none
integer i, j, kc, kt
double precision dxi, deta, dzeta_c, dzeta_t
double precision vz_m(0:JMAX,0:IMAX)
double precision avz3(0:KCMAX)
double precision cvz00, cvz0(0:KTMAX), cvz1(0:KTMAX), cvz2(0:KTMAX), &
       cvz3(0:KCMAX), cvz4(0:KCMAX), cvz5(0:KCMAX)
double precision dxi_inv, deta_inv

dxi_inv  = 1.0d0/dxi
deta_inv = 1.0d0/deta

do i=1, IMAX-1
do j=1, JMAX-1

if ( (maske(j,i).eq.0).or.(maske(j,i).eq.3) ) then
!                                ! glaciated land or floating ice --------

!  ------ Abbreviations

   cvz00 = 0.5d0*(vx_t(0,j,i)+vx_t(0,j,i-1))*dzb_dxi_g(j,i) &
           +0.5d0*(vy_t(0,j,i)+vy_t(0,j-1,i))*dzb_deta_g(j,i) &
           +dzb_dtau(j,i)-Q_b_tot(j,i)

   kt=0
   cvz0(kt) = H_t(j,i)*insq_g11_g(j,i)*insq_g22_g(j,i) &
           *( (vx_t(kt,j,i)*sq_g22_sgx(j,i) &
              -vx_t(kt,j,i-1)*sq_g22_sgx(j,i-1))*dxi_inv &
             +(vy_t(kt,j,i)*sq_g11_sgy(j,i) &
              -vy_t(kt,j-1,i)*sq_g11_sgy(j-1,i))*deta_inv ) &
           *dzeta_t
   cvz1(kt) = (dzb_dxi_g(j,i)+zeta_t(kt)*dH_t_dxi_g(j,i)) &
           *(vx_t(kt+1,j,i)+vx_t(kt+1,j,i-1) &
            -vx_t(kt,j,i)  -vx_t(kt,j,i-1))*0.5d0
   cvz2(kt) = (dzb_deta_g(j,i)+zeta_t(kt)*dH_t_deta_g(j,i)) &
           *(vy_t(kt+1,j,i)+vy_t(kt+1,j-1,i) &
            -vy_t(kt,j,i)  -vy_t(kt,j-1,i))*0.5d0

   do kt=1, KTMAX-1
      cvz0(kt) = H_t(j,i)*insq_g11_g(j,i)*insq_g22_g(j,i) &
           *( (vx_t(kt,j,i)*sq_g22_sgx(j,i) &
              -vx_t(kt,j,i-1)*sq_g22_sgx(j,i-1))*dxi_inv &
             +(vy_t(kt,j,i)*sq_g11_sgy(j,i) &
              -vy_t(kt,j-1,i)*sq_g11_sgy(j-1,i))*deta_inv ) &
           *dzeta_t
      cvz1(kt) = (dzb_dxi_g(j,i)+zeta_t(kt)*dH_t_dxi_g(j,i)) &
           *(vx_t(kt+1,j,i)+vx_t(kt+1,j,i-1) &
            -vx_t(kt-1,j,i)-vx_t(kt-1,j,i-1))*0.25d0
      cvz2(kt) = (dzb_deta_g(j,i)+zeta_t(kt)*dH_t_deta_g(j,i)) &
           *(vy_t(kt+1,j,i)+vy_t(kt+1,j-1,i) &
            -vy_t(kt-1,j,i)-vy_t(kt-1,j-1,i))*0.25d0
   end do

   kt=KTMAX
   cvz0(kt) = H_t(j,i)*insq_g11_g(j,i)*insq_g22_g(j,i) &
           *( (vx_t(kt,j,i)*sq_g22_sgx(j,i) &
              -vx_t(kt,j,i-1)*sq_g22_sgx(j,i-1))*dxi_inv &
             +(vy_t(kt,j,i)*sq_g11_sgy(j,i) &
              -vy_t(kt,j-1,i)*sq_g11_sgy(j-1,i))*deta_inv ) &
           *dzeta_t
   cvz1(kt) = (dzb_dxi_g(j,i)+zeta_t(kt)*dH_t_dxi_g(j,i)) &
           *(vx_t(kt,j,i)  +vx_t(kt,j,i-1) &
            -vx_t(kt-1,j,i)-vx_t(kt-1,j,i-1))*0.5d0
   cvz2(kt) = (dzb_deta_g(j,i)+zeta_t(kt)*dH_t_deta_g(j,i)) &
           *(vy_t(kt,j,i)  +vy_t(kt,j-1,i) &
            -vy_t(kt-1,j,i)-vy_t(kt-1,j-1,i))*0.5d0

   kc=0
   cvz3(kc) = avz3(kc)*H_c(j,i) &
              *insq_g11_g(j,i)*insq_g22_g(j,i) &
           *( (vx_c(kc,j,i)*sq_g22_sgx(j,i) &
              -vx_c(kc,j,i-1)*sq_g22_sgx(j,i-1))*dxi_inv &
             +(vy_c(kc,j,i)*sq_g11_sgy(j,i) &
              -vy_c(kc,j-1,i)*sq_g11_sgy(j-1,i))*deta_inv )
   cvz4(kc) = (dzm_dxi_g(j,i) &
           +eaz_c_quotient(kc)*dH_c_dxi_g(j,i)) &
           *(vx_c(kc+1,j,i)+vx_c(kc+1,j,i-1) &
            -vx_c(kc,j,i)  -vx_c(kc,j,i-1))*0.5d0
   cvz5(kc) = (dzm_deta_g(j,i) &
           +eaz_c_quotient(kc)*dH_c_deta_g(j,i)) &
           *(vy_c(kc+1,j,i)+vy_c(kc+1,j-1,i) &
            -vy_c(kc,j,i)  -vy_c(kc,j-1,i))*0.5d0

   do kc=1, KCMAX-1
      cvz3(kc) = avz3(kc)*H_c(j,i) &
                 *insq_g11_g(j,i)*insq_g22_g(j,i) &
           *( (vx_c(kc,j,i)*sq_g22_sgx(j,i) &
              -vx_c(kc,j,i-1)*sq_g22_sgx(j,i-1))*dxi_inv &
             +(vy_c(kc,j,i)*sq_g11_sgy(j,i) &
              -vy_c(kc,j-1,i)*sq_g11_sgy(j-1,i))*deta_inv )
      cvz4(kc) = (dzm_dxi_g(j,i) &
           +eaz_c_quotient(kc)*dH_c_dxi_g(j,i)) &
           *(vx_c(kc+1,j,i)+vx_c(kc+1,j,i-1) &
            -vx_c(kc-1,j,i)-vx_c(kc-1,j,i-1))*0.25d0
      cvz5(kc) = (dzm_deta_g(j,i) &
           +eaz_c_quotient(kc)*dH_c_deta_g(j,i)) &
           *(vy_c(kc+1,j,i)+vy_c(kc+1,j-1,i) &
            -vy_c(kc-1,j,i)-vy_c(kc-1,j-1,i))*0.25d0
   end do

   kc=KCMAX
   cvz3(kc) = avz3(kc)*H_c(j,i) &
              *insq_g11_g(j,i)*insq_g22_g(j,i) &
           *( (vx_c(kc,j,i)*sq_g22_sgx(j,i) &
              -vx_c(kc,j,i-1)*sq_g22_sgx(j,i-1))*dxi_inv &
             +(vy_c(kc,j,i)*sq_g11_sgy(j,i) &
              -vy_c(kc,j-1,i)*sq_g11_sgy(j-1,i))*deta_inv )
   cvz4(kc) = (dzm_dxi_g(j,i) &
           +eaz_c_quotient(kc)*dH_c_dxi_g(j,i)) &
           *(vx_c(kc,j,i)  +vx_c(kc,j,i-1) &
            -vx_c(kc-1,j,i)-vx_c(kc-1,j,i-1))*0.5d0
   cvz5(kc) = (dzm_deta_g(j,i) &
           +eaz_c_quotient(kc)*dH_c_deta_g(j,i)) &
           *(vy_c(kc,j,i)  +vy_c(kc,j-1,i) &
            -vy_c(kc-1,j,i)-vy_c(kc-1,j-1,i))*0.5d0

!  ------ Compute vz

   vz_b(j,i) = cvz00

   if ((n_cts(j,i).eq.-1).or.(n_cts(j,i).eq.0)) then
!                       ! cold ice base, temperate ice base --------

      do kt=0, KTMAX-1
         vz_t(kt,j,i) = vz_b(j,i)
      end do

      vz_m(j,i) = vz_b(j,i)

      vz_c(0,j,i) = vz_m(j,i) &
                    +0.5d0*(-cvz3(0)+cvz4(0)+cvz5(0))

      do kc=1, KCMAX-1
         vz_c(kc,j,i) = vz_c(kc-1,j,i) &
                        -cvz3(kc)+cvz4(kc)+cvz5(kc)
      end do

   else   ! n_cts(j,i).eq.1, temperate ice layer

      vz_t(0,j,i) = vz_b(j,i) &
                    +0.5d0*(-cvz0(0)+cvz1(0)+cvz2(0))

      do kt=1, KTMAX-1
         vz_t(kt,j,i) = vz_t(kt-1,j,i) &
                        -cvz0(kt)+cvz1(kt)+cvz2(kt)
      end do

      vz_m(j,i) = vz_t(KTMAX-1,j,i) &
                  +0.5d0*(-cvz0(KTMAX)+cvz1(KTMAX)+cvz2(KTMAX))

      vz_c(0,j,i) = vz_m(j,i) &
                    +0.5d0*(-cvz3(0)+cvz4(0)+cvz5(0))

      do kc=1, KCMAX-1
         vz_c(kc,j,i) = vz_c(kc-1,j,i) &
                        -cvz3(kc)+cvz4(kc)+cvz5(kc)
      end do

   end if

else   ! maske(j,i).eq.(1 oder 2)

   do kt=0, KTMAX-1
      vz_t(kt,j,i) = 0.0d0
   end do

   do kc=0, KCMAX-1
      vz_c(kc,j,i) = 0.0d0
   end do

end if

end do
end do

return
end
