!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Function :  c r e e p

!   Purpose  :  Creep-response function for ice.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

double precision function creep(sigma_val)

use sico_variables

implicit none
double precision :: sigma_val

#if (FIN_VISC==1)

#if (FLOW_LAW==1)

creep = sigma_val*sigma_val
!       Glen's flow law (n=3)

#elif (FLOW_LAW==2)

creep = sigma_val**0.8d0 * GR_SIZE**(-1.4d0)
!       Goldsby-Kohlstedt flow law (n=1.8, d=1.4)

#elif (FLOW_LAW==3)

creep = sigma_val*sigma_val*sigma_val
!       Durham's flow law (n=4)

#endif

#elif (FIN_VISC==2)

#if (FLOW_LAW==1)

creep = sigma_val*sigma_val + SIGMA_RES*SIGMA_RES
!       Glen's flow (n=3) with additional finite viscosity

#elif (FLOW_LAW==2)

creep = (sigma_val**0.8d0 + SIGMA_RES**0.8d0) * GR_SIZE**(-1.4d0)
!       Goldsby-Kohlstedt flow law (n=1.8, d=1.4)
!       with additional finite viscosity

#elif (FLOW_LAW==3)

creep = sigma_val*sigma_val*sigma_val + SIGMA_RES*SIGMA_RES*SIGMA_RES
!       Durham's flow law (n=4) with additional finite viscosity

#endif

#endif

end function creep
