!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  g e o _ c o o r d

!   Purpose    :  Computation of longitude lambda and latitude phi
!                 for position (x,y) in the numerical domain.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine geo_coord(phi_val, lambda_val, x_val, y_val)

use sico_variables

implicit none
double precision, intent(in) :: x_val, y_val
double precision, intent(out) :: lambda_val, phi_val

double precision :: K

#if (GRID==0 || GRID==1)

!-------- Inverse stereographic projection --------

if (PHI0 > eps) then   ! for northern hemisphere

   K = (cos(0.25d0*pi-0.5d0*PHI0))**2

   phi_val = 0.5d0*pi-2.0d0*atan(sqrt(x_val**2+y_val**2)/(2.0d0*R*K))
   lambda_val = LAMBDA0 + atan2(y_val,x_val)+0.5d0*pi

   if (lambda_val < 0.0d0) then
      lambda_val = lambda_val + 2.0d0*pi
   else if (lambda_val >= (2.0d0*pi)) then
      lambda_val = lambda_val - 2.0d0*pi
   end if

else if (PHI0 < (-eps)) then   ! for southern hemisphere

   K = (cos(0.25d0*pi+0.5d0*PHI0))**2

   phi_val = -0.5d0*pi+2.0d0*atan(sqrt(x_val**2+y_val**2)/(2.0d0*R*K))
   lambda_val = LAMBDA0 - atan2(y_val,x_val)+0.5d0*pi

   if (lambda_val < 0.0d0) then
      lambda_val = lambda_val + 2.0d0*pi
   else if (lambda_val >= (2.0d0*pi)) then
      lambda_val = lambda_val - 2.0d0*pi
   end if

else

   stop ' PHI0 must be different from zero!'

end if

#elif GRID==2

!-------- Identity --------

lambda_val = x_val
phi_val    = y_val

#endif

end subroutine geo_coord
