!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Function :  k a p p a _ v a l

!   Purpose  :  Heat conductivity of ice:
!               Linear interpolation of tabulated values in KAPPA(.).

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

double precision function kappa_val(temp_val)

use sico_variables

implicit none
double precision, intent(in) :: temp_val

double precision :: kappa_ice

!-------- Heat conductivity of pure ice --------

#if defined(FRAC_DUST)
kappa_ice = KAPPA(floor(temp_val)) &
           +(KAPPA(floor(temp_val)+1)-KAPPA(floor(temp_val))) &
            *(temp_val-dble(floor(temp_val)))
#else
kappa_val = KAPPA(floor(temp_val)) &
           +(KAPPA(floor(temp_val)+1)-KAPPA(floor(temp_val))) &
            *(temp_val-dble(floor(temp_val)))
#endif

!-------- If dust is present (polar caps of Mars):
!         Heat conductivity of ice-dust mixture --------

#if defined(FRAC_DUST)
kappa_val = (1.0d0-FRAC_DUST)*kappa_ice + FRAC_DUST*KAPPA_C
#endif

end function kappa_val
