!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  m e t r i c

!   Purpose    :  Definition of the components g11 and g22 of the
!                 metric tensor of the applied coordinates.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine metric

use sico_variables

implicit none
integer i, j
double precision K
double precision g11_g(0:JMAX,0:IMAX), g22_g(0:JMAX,0:IMAX), &
       g11_sgx(0:JMAX,0:IMAX), g11_sgy(0:JMAX,0:IMAX), &
       g22_sgx(0:JMAX,0:IMAX), g22_sgy(0:JMAX,0:IMAX)
double precision xhelp, yhelp

#if GRID==0

!-------- Components g11, g22 on the grid points (_g) and between
!         the grid points (_sg) --------

do i=0, IMAX
do j=0, JMAX
   g11_g(j,i) = 1
   g22_g(j,i) = 1
   g11_sgx(j,i) = 1
   g11_sgy(j,i) = 1
   g22_sgx(j,i) = 1
   g22_sgy(j,i) = 1
end do
end do

#elif GRID==1

!-------- Components g11, g22 on the grid points (_g) --------

if (PHI0.gt.eps) then   ! for northern hemisphere
   K = (cos(0.25d0*pi-0.5d0*PHI0))**2
else if (PHI0.lt.(-eps)) then   ! for southern hemisphere
   K = (cos(0.25d0*pi+0.5d0*PHI0))**2
else
   stop ' PHI0 must be different from zero!'
end if

do i=0, IMAX
do j=0, JMAX
   g11_g(j,i) = 1.0d0/ &
      ( K**2*(1.0d0+(xi(i)**2+eta(j)**2)/(2.0d0*R*K)**2)**2 )
   g22_g(j,i) = g11_g(j,i)
end do
end do

!-------- Components g11, g22 between the grid points (_sg) --------

do i=0, IMAX-1
do j=0, JMAX
   xhelp = 0.5d0*(xi(i)+xi(i+1))
   g11_sgx(j,i) = 1.0d0/ &
      ( K**2*(1.0d0+(xhelp**2+eta(j)**2)/(2.0d0*R*K)**2)**2 )
   g22_sgx(j,i) = g11_sgx(j,i)
end do
end do

do i=0, IMAX
do j=0, JMAX-1
   yhelp = 0.5d0*(eta(j)+eta(j+1))
   g22_sgy(j,i) = 1.0d0/ &
      ( K**2*(1.0d0+(xi(i)**2+yhelp**2)/(2.0d0*R*K)**2)**2 )
   g11_sgy(j,i) = g22_sgy(j,i)
end do
end do

#elif GRID==2

!-------- Components g11, g22 on the grid points (_g) --------

do i=0, IMAX
do j=0, JMAX
   g11_g(j,i) = R**2*(cos(eta(j)))**2
   g22_g(j,i) = R**2
end do
end do

!-------- Components g11, g22 between the grid points (_sg) --------

do i=0, IMAX-1
do j=0, JMAX
   g11_sgx(j,i) = R**2*(cos(eta(j)))**2
   g22_sgx(j,i) = R**2
end do
end do

do i=0, IMAX
do j=0, JMAX-1
   yhelp = 0.5d0*(eta(j)+eta(j+1))
   g22_sgy(j,i) = R**2
   g11_sgy(j,i) = R**2*(cos(yhelp))**2
end do
end do

#endif

!-------- Square roots (sq_) and inverse square roots (insq_) of
!         g11 and g22 --------

do i=0, IMAX
do j=0, JMAX
   sq_g11_g(j,i)   = sqrt(g11_g(j,i))
   sq_g22_g(j,i)   = sqrt(g22_g(j,i))
   insq_g11_g(j,i) = 1.0d0/sq_g11_g(j,i)
   insq_g22_g(j,i) = 1.0d0/sq_g22_g(j,i)
end do
end do

do i=0, IMAX-1
do j=0, JMAX
   sq_g11_sgx(j,i)   = sqrt(g11_sgx(j,i))
   sq_g22_sgx(j,i)   = sqrt(g22_sgx(j,i))
   insq_g11_sgx(j,i) = 1.0d0/sq_g11_sgx(j,i)
end do
end do

do i=0, IMAX
do j=0, JMAX-1
   sq_g22_sgy(j,i)   = sqrt(g22_sgy(j,i))
   sq_g11_sgy(j,i)   = sqrt(g11_sgy(j,i))
   insq_g22_sgy(j,i) = 1.0d0/sq_g22_sgy(j,i)
end do
end do

end subroutine metric
