!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  n u m _ c o o r d

!   Purpose    :  Computation of position (x,y) in the numerical domain
!                 for longitude lambda and latitude phi (in rad).

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine num_coord(lambda_val, phi_val, x_val, y_val)

use sico_variables

implicit none
double precision, intent(in) :: lambda_val, phi_val
double precision, intent(out) :: x_val, y_val

double precision :: K

#if (GRID==0 || GRID==1)

!-------- Stereographic projection --------

if (PHI0 > eps) then   ! for northern hemisphere

   K = (cos(0.25d0*pi-0.5d0*PHI0))**2

   x_val =  2.0d0*R*K*tan(0.25d0*pi-0.5d0*phi_val)*sin(lambda_val-LAMBDA0)
   y_val = -2.0d0*R*K*tan(0.25d0*pi-0.5d0*phi_val)*cos(lambda_val-LAMBDA0)

else if (PHI0 < (-eps)) then   ! for southern hemisphere

   K = (cos(0.25d0*pi+0.5d0*PHI0))**2

   x_val =  2.0d0*R*K*tan(0.25d0*pi+0.5d0*phi_val)*sin(lambda_val-LAMBDA0)
   y_val =  2.0d0*R*K*tan(0.25d0*pi+0.5d0*phi_val)*cos(lambda_val-LAMBDA0)

else

   stop ' Subroutine num_coord: PHI0 must be different from zero!'

end if

#elif GRID==2

!-------- Identity --------

x_val = lambda_val
y_val = phi_val

#endif

end subroutine num_coord
