!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  o u t p u t 1

!   Purpose    :  Writing of time-slice files.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine output1(runname, time, delta_ts, glac_index, z_sl, ndat)

use sico_variables

implicit none
integer i, j, kc, kt, kr
integer ios
integer ndat, ndat_zehner, ndat_einer
real time_conv, delta_ts_conv, glac_index_conv, z_sl_conv, &
     H_R_conv, xi_conv(0:IMAX), eta_conv(0:JMAX), &
     zs_conv(0:JMAX,0:IMAX), zm_conv(0:JMAX,0:IMAX), &
     zb_conv(0:JMAX,0:IMAX), &
     H_c_conv(0:JMAX,0:IMAX), H_t_conv(0:JMAX,0:IMAX), &
     Q_bm_conv(0:JMAX,0:IMAX), Q_tld_conv(0:JMAX,0:IMAX), &
     am_perp_conv(0:JMAX,0:IMAX), &
     qx_conv(0:JMAX,0:IMAX), qy_conv(0:JMAX,0:IMAX), &
     temp_r_conv(0:KRMAX,0:JMAX,0:IMAX), &
     vx_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vy_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vz_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vx_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     vy_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     vz_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     temp_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     age_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     omega_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     age_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     dzs_dtau_conv(0:JMAX,0:IMAX), dzm_dtau_conv(0:JMAX,0:IMAX), &
     dzb_dtau_conv(0:JMAX,0:IMAX), &
     dH_c_dtau_conv(0:JMAX,0:IMAX), dH_t_dtau_conv(0:JMAX,0:IMAX)
double precision time, delta_ts, glac_index, z_sl
character (len=100) :: runname
character ch_help1, ch_help2

!-------- Create consecutively numbered file names --------

if (ndat.gt.99) then
   write(6,1000) 'Too many time-slice files!'
   stop
end if

ndat_zehner = ndat/10
ndat_einer  = ndat-ndat_zehner*10
ch_help1 = char(ndat_zehner+ichar('0'))
ch_help2 = char(ndat_einer +ichar('0'))

!-------- Open time-slice file runname.erg --------

open(11, iostat=ios, &
         file=OUTPATH//'/'//trim(runname)//ch_help1//ch_help2//'.erg', &
         status='new', form='unformatted')
if (ios.ne.0) then
   write(6,1000) 'Error when opening an erg file!'
   stop
end if

!-------- Convert data to real*4 --------

time_conv       = real(time)
delta_ts_conv   = real(delta_ts)
glac_index_conv = real(glac_index)
z_sl_conv       = real(z_sl)
H_R_conv        = real(H_R)

do i=0, IMAX
do j=0, JMAX

   xi_conv(i)          = real(xi(i))
   eta_conv(j)         = real(eta(j))
   zs_conv(j,i)        = real(zs(j,i))
   zm_conv(j,i)        = real(zm(j,i))
   zb_conv(j,i)        = real(zb(j,i))
   H_c_conv(j,i)       = real(H_c(j,i))
   H_t_conv(j,i)       = real(H_t(j,i))
   Q_bm_conv(j,i)      = real(Q_bm(j,i))
   Q_tld_conv(j,i)     = real(Q_tld(j,i))
   am_perp_conv(j,i)   = real(am_perp(j,i))
   qx_conv(j,i)        = real(qx(j,i))
   qy_conv(j,i)        = real(qy(j,i))
   dzs_dtau_conv(j,i)  = real(dzs_dtau(j,i))
   dzm_dtau_conv(j,i)  = real(dzm_dtau(j,i))
   dzb_dtau_conv(j,i)  = real(dzb_dtau(j,i))
   dH_c_dtau_conv(j,i) = real(dH_c_dtau(j,i))
   dH_t_dtau_conv(j,i) = real(dH_t_dtau(j,i))

   do kr=0, KRMAX
      temp_r_conv(kr,j,i) = real(temp_r(kr,j,i))
   end do

   do kt=0, KTMAX
      vx_t_conv(kt,j,i)    = real(vx_t(kt,j,i))
      vy_t_conv(kt,j,i)    = real(vy_t(kt,j,i))
      vz_t_conv(kt,j,i)    = real(vz_t(kt,j,i))
      omega_t_conv(kt,j,i) = real(omega_t(kt,j,i))
      age_t_conv(kt,j,i)   = real(age_t(kt,j,i))
   end do

   do kc=0, KCMAX
      vx_c_conv(kc,j,i)    = real(vx_c(kc,j,i))
      vy_c_conv(kc,j,i)    = real(vy_c(kc,j,i))
      vz_c_conv(kc,j,i)    = real(vz_c(kc,j,i))
      temp_c_conv(kc,j,i)  = real(temp_c(kc,j,i))
      age_c_conv(kc,j,i)   = real(age_c(kc,j,i))
   end do

end do
end do

!-------- Write data on file --------

write(11) time_conv
if (forcing_flag == 1) then
   write(11) delta_ts_conv
else if (forcing_flag == 2) then
   write(11) glac_index_conv
end if
write(11) z_sl_conv
write(11) xi_conv
write(11) eta_conv
write(11) maske
write(11) n_cts
write(11) zs_conv
write(11) zm_conv
write(11) zb_conv
write(11) H_c_conv
write(11) H_t_conv
write(11) H_R_conv
#if ERGDAT==1
write(11) vx_c_conv
write(11) vy_c_conv
write(11) vz_c_conv
write(11) vx_t_conv
write(11) vy_t_conv
write(11) vz_t_conv
write(11) temp_c_conv
write(11) omega_t_conv
write(11) temp_r_conv
#endif
write(11) Q_bm_conv
write(11) Q_tld_conv
write(11) am_perp_conv
write(11) qx_conv
write(11) qy_conv
#if ERGDAT==1
write(11) age_c_conv
write(11) age_t_conv
#endif
write(11) dzs_dtau_conv
write(11) dzm_dtau_conv
write(11) dzb_dtau_conv
write(11) dH_c_dtau_conv
write(11) dH_t_dtau_conv

close(11, status='keep')

 1000 format(a)

!-------- Increase file counter --------

ndat = ndat+1

return
end
