!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  o u t p u t 2

!   Purpose    :  Writing of time-series data in file.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine output2(time, dxi, deta, delta_ts, glac_index, z_sl)

use sico_variables

implicit none
integer :: i, j
integer :: n_base, n_tempbase
double precision :: time, dxi, deta, delta_ts, glac_index, z_sl
double precision :: time_val, H_max, zs_max, V_ges, z_sle, V_temp, &
                    A_ib, A_tb, V_fw, V_bm, V_tld, H_t_max, vs_max, Tbh_max
double precision :: x_pos, y_pos
double precision :: H_t_help(0:JMAX,0:IMAX)
double precision :: V_ges_redu, V_ges_eais, z_sle_eais, rho_ratio, A_surf, &
                    vs_help, Tbh_help
double precision :: H_ave_sed, Tbh_ave_sed, Atb_sed
double precision :: sum_area_sed

!-------- Thickness of the temperate layer --------

#if CALCMOD==1
do i=1, IMAX-1
do j=1, JMAX-1
   H_t_help(j,i) = H_t(j,i)
end do
end do
#elif CALCMOD==0
do i=1, IMAX-1
do j=1, JMAX-1
   H_t_help(j,i) = H_c(j,i)*eaz_c_quotient(kc_cts(j,i))
end do
end do
#endif

!-------- Maximum ice elevation and thickness, ice volume,
!         sea-level equivalent, volume of the temperate ice,
!         ice-covered basal area, basal area covered by temperate ice,
!         freshwater production due to melting and calving,
!         water drainage due to basal melting,
!         water drainage from the temperate layer,
!         maximum thickness of the temperate layer,
!         maximum surface velocity,
!         maximum homologous basal temperature --------

rho_ratio = RHO_SW/RHO

zs_max     = -1.11d+11
H_max      = 0.0d0
V_ges      = 0.0d0
V_ges_redu = 0.0d0
V_temp     = 0.0d0
A_ib       = 0.0d0
A_tb       = 0.0d0
V_fw       = 0.0d0
V_bm       = 0.0d0
V_tld      = 0.0d0
H_t_max    = 0.0d0
vs_max     = 0.0d0
Tbh_max    = -273.15d0

do i=1, IMAX-1
do j=1, JMAX-1

   if (zs(j,i) > zs_max) zs_max = zs(j,i)

   if (maske(j,i) == 0) then   ! glaciated land

      if ((H_c(j,i)+H_t(j,i)) > H_max) &
                                H_max = H_c(j,i)+H_t(j,i)
      if (H_t_help(j,i) > H_t_max) H_t_max = H_t_help(j,i)

      V_ges  = V_ges  + (H_c(j,i)+H_t(j,i))*area(j,i)
      V_temp = V_temp + H_t_help(j,i)*area(j,i)

      if (zb0(j,i) < z_sl) &
         V_ges_redu = V_ges_redu + rho_ratio*(z_sl-zb0(j,i))*area(j,i)

      A_ib = A_ib+area(j,i)
      if (n_cts(j,i) /= -1) A_tb = A_tb+area(j,i)

      V_bm  = V_bm  + Q_bm(j,i)     *area(j,i)
      V_tld = V_tld + Q_tld(j,i)    *area(j,i)

      vs_help = sqrt( &
               0.25d0*(vx_c(KCMAX,j,i)+vx_c(KCMAX,j,i-1))**2 &
              +0.25d0*(vy_c(KCMAX,j,i)+vy_c(KCMAX,j-1,i))**2 )
      if (vs_help > vs_max) vs_max = vs_help

      if (n_cts(j,i) >= 0) then   ! temperate base
         Tbh_max = 0.0d0
      else   ! cold base
         Tbh_help = min((temp_c(0,j,i)-temp_c_m(0,j,i)), -eps)
         if (Tbh_help > Tbh_max) Tbh_max = Tbh_help
      end if

   end if

   if ((maske(j,i) == 0).or.(maske_help(j,i) == 0)) then
                        ! glaciated land before or after the time step
      V_fw  = V_fw + accum(j,i)*area(j,i) &
              - (dzs_dtau(j,i)-dzb_dtau(j,i))*area(j,i)
   end if

end do
end do

!-------- Conversion --------

#if ( defined(ANT) \
      || defined(GRL) \
      || defined(SCAND) \
      || defined(NHEM) \
      || defined(TIBET) \
      || defined(EMTP2SGE) \
      || defined(HEINO) )   /* terrestrial ice sheet */

V_ges_eais = 26.0d+15    ! volume of east Antarctic ice sheet,
                         ! in m^3
z_sle_eais = 60.0d0      ! sea-level equivalent of east Antarctic
                         ! ice sheet, in m
z_sle = (V_ges-V_ges_redu) * z_sle_eais/V_ges_eais   ! m

#elif ( defined(NMARS) \
        || defined(SMARS) )   /* Martian ice sheet */

A_surf = 1.4441d+14     ! surface area of Mars, in m^2
z_sle  = V_ges/A_surf   ! m

#else

stop ' Subroutine output2: No valid domain specified!'

#endif

time_val = time   *1.0d-03/YEAR_SEC    ! s -> ka
H_max    = H_max  *1.0d-03             ! m -> km
zs_max   = zs_max *1.0d-03             ! m -> km
V_ges    = V_ges  *1.0d-15             ! m^3 -> 10^6 km^3
V_temp   = V_temp *1.0d-12             ! m^3 -> 10^3 km^3
A_ib     = A_ib   *1.0d-12             ! m^2 -> 10^6 km^2
A_tb     = A_tb   *1.0d-12             ! m^2 -> 10^6 km^2
vs_max   = vs_max *YEAR_SEC            ! m/s -> m/a

#if ( defined(ANT) \
      || defined(GRL) \
      || defined(SCAND) \
      || defined(NHEM) \
      || defined(TIBET) \
      || defined(EMTP2SGE) \
      || defined(HEINO) )   /* terrestrial ice sheet */

V_fw     = V_fw   *1.0d-09*YEAR_SEC *(RHO/RHO_W)
                  ! m^3/s ice equiv. -> km^3/a water equiv.
V_bm     = V_bm   *1.0d-09*YEAR_SEC *(RHO/RHO_W)
                  ! m^3/s ice equiv. -> km^3/a water equiv.
V_tld    = V_tld  *1.0d-09*YEAR_SEC *(RHO/RHO_W)
                  ! m^3/s ice equiv. -> km^3/a water equiv.

#elif ( defined(NMARS) \
        || defined(SMARS) )   /* Martian ice sheet */

V_fw     = V_fw   *1.0d-09*YEAR_SEC *(RHO_I/RHO_W)*(1.0d0-FRAC_DUST)
                  ! m^3/s (ice+dust) equiv. -> km^3/a water equiv.
V_bm     = V_bm   *1.0d-09*YEAR_SEC *(RHO_I/RHO_W)*(1.0d0-FRAC_DUST)
                  ! m^3/s (ice+dust) equiv. -> km^3/a water equiv.
V_tld    = V_tld  *1.0d-09*YEAR_SEC *(RHO_I/RHO_W)*(1.0d0-FRAC_DUST)
                  ! m^3/s (ice+dust) equiv. -> km^3/a water equiv.
#endif

!-------- Writing of data on time-series file --------

#if ( defined(ANT) \
      || defined(GRL) \
      || defined(SCAND) \
      || defined(NHEM) \
      || defined(TIBET) \
      || defined(EMTP2SGE) \
      || defined(HEINO) )   /* terrestrial ice sheet */

if (forcing_flag == 1) then

   write(12,1010) time_val, delta_ts, z_sl, &
      H_max, zs_max, V_ges, V_temp, V_fw, z_sle, &
      A_ib, A_tb, V_bm, V_tld, H_t_max, vs_max

else if (forcing_flag == 2) then

   write(12,1010) time_val, glac_index, z_sl, &
      H_max, zs_max, V_ges, V_temp, V_fw, z_sle, &
      A_ib, A_tb, V_bm, V_tld, H_t_max, vs_max

end if

#elif ( defined(NMARS) \
        || defined(SMARS) )   /* Martian ice sheet */

if (forcing_flag == 1) then

   write(12,1010) time_val, delta_ts, z_sl, &
      H_max, zs_max, V_ges, V_temp, V_fw, Tbh_max, &
      A_ib, A_tb, V_bm, V_tld, H_t_max, vs_max

else if (forcing_flag == 2) then

   write(12,1010) time_val, glac_index, z_sl, &
      H_max, zs_max, V_ges, V_temp, V_fw, Tbh_max, &
      A_ib, A_tb, V_bm, V_tld, H_t_max, vs_max

end if

#endif

1010 format(1pe13.6,2(1pe13.4),/,13x,6(1pe13.4),/,13x,6(1pe13.4),/)

!-------- Time-series data for the sediment region of HEINO --------

#if defined(HEINO)

!  ------ Average ice thickness, average homologous basal temperature,
!         temperate basal area

H_ave_sed    = 0.0d0
Tbh_ave_sed  = 0.0d0
Atb_sed      = 0.0d0
sum_area_sed = 0.0d0

do i=1, IMAX-1
do j=1, JMAX-1

   if (maske_sedi(j,i) == 7) then   ! sediment region

      sum_area_sed = sum_area_sed + area(j,i)

      H_ave_sed = H_ave_sed + area(j,i)*(H_c(j,i)+H_t(j,i))

      if (n_cts(j,i) /= -1) then   ! temperate base
         Tbh_help    = 0.0d0
      else   ! cold base
         Tbh_help    = min((temp_c(0,j,i)-temp_c_m(0,j,i)), -eps)
      end if
      Tbh_ave_sed = Tbh_ave_sed + area(j,i)*Tbh_help

      if (n_cts(j,i) /= -1) Atb_sed = Atb_sed + area(j,i)

   end if

end do
end do

if (sum_area_sed > eps) then
   H_ave_sed   = H_ave_sed   / sum_area_sed
   Tbh_ave_sed = Tbh_ave_sed / sum_area_sed
else
   stop ' Error in subroutine output2: No sediment area found!'
end if

!  ------ Conversion

H_ave_sed = H_ave_sed *1.0d-03   ! m -> km
Atb_sed   = Atb_sed   *1.0d-12   ! m^2 -> 10^6 km^2

!  ------ Writing of data on time-series file

if (forcing_flag == 1) then
   write(15,1011) time_val, delta_ts, z_sl, H_ave_sed, Tbh_ave_sed, Atb_sed
else if (forcing_flag == 2) then
   write(15,1011) time_val, glac_index, z_sl, H_ave_sed, Tbh_ave_sed, Atb_sed
end if

1011 format(1pe13.6,2(1pe13.4),/,13x,3(1pe13.4),/)

#endif

end subroutine output2
