!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  o u t p u t _ n c

!   Purpose    :  Writing of time-slice data in a netCDF file
!                 (by Magnus Weis).

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine output_nc(runname, time, delta_ts, glac_index, z_sl, ndat)

use sico_variables

implicit none
include NETCDF_INC
!                          ! include netCDF header file

integer ndat
double precision time, delta_ts, glac_index, z_sl
character (len=100) :: runname

!     Variables for conversion to real*4
real time_conv, delta_ts_conv, z_sl_conv, &
     H_R_conv, xi_conv(0:IMAX), eta_conv(0:JMAX), &
     zs_conv(0:JMAX,0:IMAX), zm_conv(0:JMAX,0:IMAX), &
     zb_conv(0:JMAX,0:IMAX), &
     H_c_conv(0:JMAX,0:IMAX), H_t_conv(0:JMAX,0:IMAX), &
     Q_bm_conv(0:JMAX,0:IMAX), Q_tld_conv(0:JMAX,0:IMAX), &
     am_perp_conv(0:JMAX,0:IMAX), &
     qx_conv(0:JMAX,0:IMAX), qy_conv(0:JMAX,0:IMAX), &
     temp_r_conv(0:KRMAX,0:JMAX,0:IMAX), &
     vx_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vy_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vz_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vx_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     vy_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     vz_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     temp_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     age_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     omega_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     age_t_conv(0:KTMAX,0:JMAX,0:IMAX)
integer i, j, kc, kt ,kr

integer ncid, errno, ncv
!     ncid:      netCDF id of the output file
!     errno:     netCDF error code
!     ncv:       netCDF variable
integer nc0d_dim, nc1d_dim, nc2d_dim, nc3d_dim
integer ncd, nc1d, nc2d(2), nc3d(3)
!     ncd:       Dimension of a scalar
!     ncd1:      Dimension of a 1-d array
!     ncd2:      Vector with the dimensions of a 2-d array
!     ncd3:      Vector with the dimensions of a 3-d array
integer nccor, nc1cor, nc2cor(2), nc3cor(3)
!     nccor:     Corner of a scalar
!     nc1cor:    Corner of a 1-d array
!     nc2cor(2): Corner of a 2-d array
!     nc3cor(3): Corner of a 3-d array
integer nc1cnt, nc2cnt(2), nc3cnt(3)
!     nc1cnt:    Count of a 1-d array
!     nc2cnt(2): Count of a 2-d array
!     nc3cnt(3): Count of a 3-d array
character buffer*255
character ch_help1, ch_help2
integer ndat_zehner, ndat_einer

!-------- Create consecutively numbered file names --------

if (ndat.gt.99) then
   write(6,1000) 'Too many time-slice files!'
   stop
end if

ndat_zehner = ndat / 10
ndat_einer  = ndat - ndat_zehner * 10
ch_help1 = char(ndat_zehner+ichar('0'))
ch_help2 = char(ndat_einer +ichar('0'))

!-------- Initialization --------

!  ------ Dimensions

nc0d_dim = 0
nc1d_dim = 1
nc2d_dim = 2
nc3d_dim = 3

!  ------ Open netCDF file

ncid = nccre(OUTPATH//'/'//trim(runname)//ch_help1//ch_help2//'.cdf', &
       NCCLOB, errno)
if (errno.ne.0) then
   write(6, 1000) 'Error when opening the netCDF file!'
   stop
end if

buffer = 'sicopolis netCDF - data'
call ncaptc(ncid, NCGLOBAL, 'title', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

!  ------ Definition of the dimensions

ncd = ncddef(ncid, 'IMAX', IMAX+1, errno)
ncd = ncddef(ncid, 'JMAX', JMAX+1, errno)
ncd = ncddef(ncid, 'KCMAX', KCMAX+1, errno)
ncd = ncddef(ncid, 'KTMAX', KTMAX+1, errno)
ncd = ncddef(ncid, 'KRMAX', KRMAX+1, errno)

!  ------ Definition of the variables

ncv = ncvdef(ncid, 't', NCFLOAT, nc0d_dim, 0, errno)
buffer = 'Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Modellzeit'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc1d = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'xi', NCFLOAT, nc1d_dim, nc1d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'xi Koordinate der Gitterpunkte i'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc1d = ncdid(ncid, 'JMAX', errno)
ncv = ncvdef(ncid, 'eta', NCFLOAT, nc1d_dim, nc1d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'eta Koordinate der Gitterpunkte j'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'maske', NCLONG, nc2d_dim, nc2d, errno)
buffer = 'Maske fuer horizontale Gitterpunkte. ' &
     //'0: glaciated land, ' &
     //'1: ice-free land, 2: sea point'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'n_cts', NCLONG, nc2d_dim, nc2d, errno)
buffer = 'Lage der CTS. ' &
     //'n_cts=-1: keine CTS vorhanden, ' &
     //'0: nur Basis temperiert, ' &
     //'1: temperierte Schicht vorhanden'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'zs', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Koordinate z der freien Oberflaeche'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'zm', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Koordinate z der CTS'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'zb', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Koordinate z des Felsgrundes'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'H_c', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Dicke des kalten Eises'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'H_t', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Dicke des temperierten Eises'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'Q_bm', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Kilogramm / Meter^2 / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'basale Schmelzrate'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'Q_tld', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Kilogramm / Meter^2 / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'aus dem temperierten Layer drainiertes Wasser'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'am_perp', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Volumenfluss durch die CTS'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'qx', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter^2 / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Vertikalintegriertes vx (bei i+1/2,j)'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc2d(1) = ncdid(ncid, 'JMAX', errno)
nc2d(2) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'qy', NCFLOAT, nc2d_dim, nc2d, errno)
buffer = 'Meter^2 / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Vertikalintegriertes vy (bei i,j+1/2)'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KRMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'temp_r', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Grad Celsius'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Temperatur im Felsgrund'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KTMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'vx_t', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Geschwindigkeit in x-Richtung im temp. Eis'// &
     '(bei (i+1/2,j,kt))'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KTMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'vy_t', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Geschwindigkeit in y-Richtung im temp. Eis'// &
     '(bei (i,j+1/2,kt))'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KTMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'vz_t', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Geschwindigkeit in z-Richtung im temp. Eis'// &
     '(bei (i,j,kt+1/2))'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KTMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'omega_t', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = '1'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Wassergehalt im temperierten Eis'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KTMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'age_t', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Eis-Alter (Isochronen) im temperierten Eis'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KCMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'vx_c', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Geschwindigkeit in x-Richtung im kalten Eis'// &
     '(bei (i+1/2,j,kt))'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KCMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'vy_c', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Geschwindigkeit in y-Richtung im kalten Eis'// &
     '(bei (i,j+1/2,kc))'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KCMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'vz_c', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Meter / Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Geschwindigkeit in z-Richtung im kalten Eis'// &
     '(bei (i,j,kc+1/2))'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KCMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'temp_c', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Grad Celsius'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Temperatur im kalten Eis'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

nc3d(1) = ncdid(ncid, 'KCMAX', errno)
nc3d(2) = ncdid(ncid, 'JMAX', errno)
nc3d(3) = ncdid(ncid, 'IMAX', errno)
ncv = ncvdef(ncid, 'age_c', NCFLOAT, nc3d_dim, nc3d, errno)
buffer = 'Sekunde'
call ncaptc(ncid, ncv, 'units', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)
buffer = 'Eis-Alter (Isochronen) im kalten Eis'
call ncaptc(ncid, ncv, 'long_name', NCCHAR, len_trim(buffer), &
     trim(buffer), errno)

call ncendf(ncid, errno)
if (errno.ne.0) then
   write(6, 1000) &
        'netCDF file does not change to the data mode!'
   stop
end if

!-------- Convert data to real*4 --------

time_conv     = real(time)
delta_ts_conv = real(delta_ts)
z_sl_conv     = real(z_sl)
H_R_conv      = real(H_R)

do i=0, IMAX
do j=0, JMAX

   xi_conv(i)        = real(xi(i))
   eta_conv(j)       = real(eta(j))
   zs_conv(j,i)      = real(zs(j,i))
   zm_conv(j,i)      = real(zm(j,i))
   zb_conv(j,i)      = real(zb(j,i))
   H_c_conv(j,i)     = real(H_c(j,i))
   H_t_conv(j,i)     = real(H_t(j,i))
   Q_bm_conv(j,i)    = real(Q_bm(j,i))
   Q_tld_conv(j,i)   = real(Q_tld(j,i))
   am_perp_conv(j,i) = real(am_perp(j,i))
   qx_conv(j,i)      = real(qx(j,i))
   qy_conv(j,i)      = real(qy(j,i))

   do kr=0, KRMAX
      temp_r_conv(kr,j,i) = real(temp_r(kr,j,i))
   end do

   do kt=0, KTMAX
      vx_t_conv(kt,j,i)    = real(vx_t(kt,j,i))
      vy_t_conv(kt,j,i)    = real(vy_t(kt,j,i))
      vz_t_conv(kt,j,i)    = real(vz_t(kt,j,i))
      omega_t_conv(kt,j,i) = real(omega_t(kt,j,i))
      age_t_conv(kt,j,i)   = real(age_t(kt,j,i))
   end do

   do kc=0, KCMAX
      vx_c_conv(kc,j,i)    = real(vx_c(kc,j,i))
      vy_c_conv(kc,j,i)    = real(vy_c(kc,j,i))
      vz_c_conv(kc,j,i)    = real(vz_c(kc,j,i))
      temp_c_conv(kc,j,i)  = real(temp_c(kc,j,i))
      age_c_conv(kc,j,i)   = real(age_c(kc,j,i))
   end do

end do
end do

!-------- Write data on file --------

ncv = ncvid(ncid, 't', errno)
nccor = 1
call ncvpt1(ncid, ncv, nccor, time_conv, errno)

ncv = ncvid(ncid, 'xi', errno)
nc1cor = 1
nc1cnt = IMAX + 1
call ncvpt(ncid, ncv, nc1cor, nc1cnt, xi_conv, errno)

ncv = ncvid(ncid, 'eta', errno)
nc1cor = 1
nc1cnt = JMAX + 1
call ncvpt(ncid, ncv, nc1cor, nc1cnt, eta_conv, errno)

ncv = ncvid(ncid, 'maske', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, maske, errno)

ncv = ncvid(ncid, 'n_cts', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, n_cts, errno)

ncv = ncvid(ncid, 'zs', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, zs_conv, errno)

ncv = ncvid(ncid, 'zm', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, zm_conv, errno)

ncv = ncvid(ncid, 'zb', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, zb_conv, errno)

ncv = ncvid(ncid, 'H_c', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, H_c_conv, errno)

ncv = ncvid(ncid, 'H_t', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, H_t_conv, errno)

ncv = ncvid(ncid, 'Q_bm', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, Q_bm_conv, errno)

ncv = ncvid(ncid, 'Q_tld', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, Q_tld_conv, errno)

ncv = ncvid(ncid, 'am_perp', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, am_perp_conv, errno)

ncv = ncvid(ncid, 'qx', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, qx_conv, errno)

ncv = ncvid(ncid, 'qy', errno)
nc2cor(1) = 1
nc2cor(2) = 1
nc2cnt(1) = JMAX + 1
nc2cnt(2) = IMAX + 1
call ncvpt(ncid, ncv, nc2cor, nc2cnt, qy_conv, errno)

ncv = ncvid(ncid, 'temp_r', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KRMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, temp_r_conv, errno)

ncv = ncvid(ncid, 'vx_t', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KTMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, vx_t_conv, errno)

ncv = ncvid(ncid, 'vy_t', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KTMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, vy_t_conv, errno)

ncv = ncvid(ncid, 'vz_t', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KTMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, vz_t_conv, errno)

ncv = ncvid(ncid, 'omega_t', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KTMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, omega_t_conv, errno)

ncv = ncvid(ncid, 'age_t', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KTMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, age_t_conv, errno)

ncv = ncvid(ncid, 'vx_c', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KCMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, vx_c_conv, errno)

ncv = ncvid(ncid, 'vy_c', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KCMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, vy_c_conv, errno)

ncv = ncvid(ncid, 'vz_c', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KCMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, vz_c_conv, errno)

ncv = ncvid(ncid, 'temp_c', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KCMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, temp_c_conv, errno)

ncv = ncvid(ncid, 'age_c', errno)
nc3cor(1) = 1
nc3cor(2) = 1
nc3cor(3) = 1
nc3cnt(1) = KCMAX + 1
nc3cnt(2) = JMAX + 1
nc3cnt(3) = IMAX + 1
call ncvpt(ncid, ncv, nc3cor, nc3cnt, age_c_conv, errno)

!-------- Close netCDF file --------

call ncsnc(ncid, errno)
if (errno.ne.0) then
   write(6, 1000) 'ncsnc error!'
end if

call ncclos(ncid, errno)
if (errno.ne.0) then
   write(6, 1000) &
        'netCDF file cannot be closed!'
   stop
end if

!-------- Increase file counter --------

ndat = ndat+1

return
 1000 format(a)
end
