!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  s h i f t _ c t s _ d o w n w a r d

!   Purpose    :  Downward shift of the CTS.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine shift_cts_downward(at1, at2_1, at2_2, at3_1, at3_2, &
              at4_1, at4_2, at5, at6, at7, atr1, am1, am2, alb1, &
              aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
              ai1, ai2, ai3, ai4, mean_accum_inv, dzeta_t, &
              dtime_temp, dtt_2dxi, dtt_2deta, dtime_temp_inv, &
              i, j)

use sico_variables

implicit none
integer i, j
double precision zm_shift
double precision at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), at7, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai3, ai4, &
       atr1, am1, am2, alb1
double precision aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld
double precision mean_accum_inv
double precision dtt_2dxi, dtt_2deta, dtime_temp, dtime_temp_inv, dzeta_t
double precision difftemp_a, difftemp_b, interpol

zm_shift = 1.0d0   ! CTS shift in intervals of 1 m

!-------- Temperature discrepancy from the computation of the main
!         program --------

difftemp_a = temp_c_neu(0,j,i)-(-BETA*H_c_neu(j,i))
if (difftemp_a.ge.0.0d0) return

!-------- Shift CTS downward until it is too low --------

   10 continue

   zm_neu(j,i)  = zm_neu(j,i) - zm_shift

!  ------ Special case: CTS too close to the base

   if (zm_neu(j,i).le.zb(j,i)) then

      zm_shift = (zm_neu(j,i)+zm_shift)-(zb(j,i)+0.001d0)
      zm_neu(j,i)  = zb(j,i)+0.001d0
      H_c_neu(j,i) = H_c_neu(j,i)+H_t_neu(j,i)-0.001d0
      H_t_neu(j,i) = 0.001d0
!                   ! CTS positioned 1 mm above ice base --------

      dH_t_dtau(j,i) = (zm_neu(j,i)-zm(j,i))*dtime_temp_inv
      dzm_dtau(j,i)  = dzb_dtau(j,i)+dH_t_dtau(j,i)
      dH_c_dtau(j,i) = dzs_dtau(j,i)-dzm_dtau(j,i)

      am_perp(j,i) = am_perp_st(j,i) + dzm_dtau(j,i)

      call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

      difftemp_b = difftemp_a
      difftemp_a = temp_c_neu(0,j,i)-(-BETA*H_c_neu(j,i))

      if (difftemp_a.ge.0.0d0) then ! CTS remains above the base

!    ---- Interpolate the CTS position from the last (_a) and the
!         last but one (_b) value, weighed with the temperature
!         discrepancies at the CTS --------

         interpol = difftemp_a/(difftemp_a-difftemp_b)*zm_shift

         zm_neu(j,i)  = zm_neu(j,i)  + interpol
         H_c_neu(j,i) = H_c_neu(j,i) - interpol
         H_t_neu(j,i) = H_t_neu(j,i) + interpol

         dH_t_dtau(j,i) = (zm_neu(j,i)-zm(j,i))*dtime_temp_inv
         dzm_dtau(j,i)  = dzb_dtau(j,i)+dH_t_dtau(j,i)
         dH_c_dtau(j,i) = dzs_dtau(j,i)-dzm_dtau(j,i)

         am_perp(j,i) = am_perp_st(j,i) + dzm_dtau(j,i)

         call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

      else   ! CTS disappears

         n_cts_neu(j,i) = 0
         zm_neu(j,i) = zb(j,i)
         H_c_neu(j,i) = H_c_neu(j,i)+H_t_neu(j,i)
         H_t_neu(j,i) = 0.0d0

         dH_t_dtau(j,i) = (zm_neu(j,i)-zm(j,i))*dtime_temp_inv
         dzm_dtau(j,i)  = dzb_dtau(j,i)+dH_t_dtau(j,i)
         dH_c_dtau(j,i) = dzs_dtau(j,i)-dzm_dtau(j,i)

         am_perp(j,i) = am_perp_st(j,i) + dzm_dtau(j,i)

         call calc_temp2(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, alb1, &
                 ai1, ai2, ai4, mean_accum_inv, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

      end if

      return
   end if

!  ------ End of treatment of special case

   H_c_neu(j,i) = H_c_neu(j,i) + zm_shift
   H_t_neu(j,i) = H_t_neu(j,i) - zm_shift

   dH_t_dtau(j,i) = (zm_neu(j,i)-zm(j,i))*dtime_temp_inv
   dzm_dtau(j,i)  = dzb_dtau(j,i)+dH_t_dtau(j,i)
   dH_c_dtau(j,i) = dzs_dtau(j,i)-dzm_dtau(j,i)

   am_perp(j,i) = am_perp_st(j,i) + dzm_dtau(j,i)

   call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta, i, j)

   difftemp_b = difftemp_a
   difftemp_a = temp_c_neu(0,j,i)-(-BETA*H_c_neu(j,i))

if (difftemp_a.lt.0.0d0) go to 10

!-------- Interpolate the CTS position from the last (_a) and the
!         last but one (_b) value, weighed with the temperature
!         discrepancies at the CTS --------

interpol = difftemp_a/(difftemp_a-difftemp_b)*zm_shift

zm_neu(j,i)  = zm_neu(j,i)  + interpol
H_c_neu(j,i) = H_c_neu(j,i) - interpol
H_t_neu(j,i) = H_t_neu(j,i) + interpol

dH_t_dtau(j,i) = (zm_neu(j,i)-zm(j,i))*dtime_temp_inv
dzm_dtau(j,i)  = dzb_dtau(j,i)+dH_t_dtau(j,i)
dH_c_dtau(j,i) = dzs_dtau(j,i)-dzm_dtau(j,i)

am_perp(j,i) = am_perp_st(j,i) + dzm_dtau(j,i)

call calc_temp3(at1, at2_1, at2_2, at3_1, at3_2, &
                at4_1, at4_2, at5, at6, at7, atr1, &
                am1, am2, alb1, &
                aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                ai1, ai2, ai3, mean_accum_inv, dzeta_t, &
                dtime_temp, dtt_2dxi, dtt_2deta, i, j)

return
end
