!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  s i c o _ m a i n _ l o o p

!   Purpose    :  Main loop of SICOPOLIS.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine sico_main_loop(delta_ts, glac_index, &
                    sum_accum, mean_accum, mean_accum_inv, &
                    dtime, dtime_temp, dtime_wss, dtime_out, dtime_ser, &
                    time, time_init, time_end, time_output, &
                    dxi, deta, dzeta_c, dzeta_t, dzeta_r, &
                    dtt_2dxi, dtt_2deta, dtime_temp_inv, &
                    griptemp, specmap_zsl, z_sle_forcing, time_forcing, &
                    z_sl, z_mar, &
                    grip_time_min, grip_time_stp, grip_time_max, &
                    specmap_time_min, specmap_time_stp, specmap_time_max, &
                    avxy3, avz3, aqxy1, azs2, azs3, &
                    at1, at2_1, at2_2, at3_1, at3_2, at4_1, &
                    at4_2, at5, at6, at7, &
                    aw1, aw2, aw3, aw4, aw5, aw8, aw9, aw7, &
                    ai1, ai2, ai3, ai4, atr1, am1, am2, atm1, atm2, &
                    acb1, acb2, acb3, acb4, alb1, &
                    aqbm1, aqbm2, aqbm3, aqbm4, aqtld, &
                    ii, jj, nn, &
                    ndat, ndata_forcing, n_output, &
                    runname, zs_switch)

use sico_variables

implicit none
integer :: i, j, kc, kt, kr, n
integer :: itercount, itercount_max
integer :: iter_temp, iter_wss, iter_ser, iter_out, iter_output(100)
integer :: ndat, ndata_forcing
integer :: n_output
integer :: ii((IMAX+1)*(JMAX+1)), jj((IMAX+1)*(JMAX+1)), nn(0:JMAX,0:IMAX)
integer :: grip_time_min, grip_time_stp, grip_time_max, &
           specmap_time_min, specmap_time_stp, specmap_time_max
double precision :: delta_ts, glac_index
double precision :: sum_accum, mean_accum, mean_accum_inv
double precision :: dtime, dtime_temp, dtime_wss, dtime_out, dtime_ser
double precision :: time, time_init, time_end, time_output(100)
double precision :: dxi, deta, dzeta_c, dzeta_t, dzeta_r
double precision :: dtt_2dxi, dtt_2deta, dtime_temp_inv
double precision :: griptemp(0:10000), specmap_zsl(0:10000), &
       z_sle_forcing(0:10000), time_forcing(0:10000), z_sl, z_mar
double precision :: avxy3(0:KCMAX), &
       avz3(0:KCMAX), &
       aqxy1(0:KCMAX), azs2, azs3, &
       at1(0:KCMAX), at2_1(0:KCMAX), at2_2(0:KCMAX), &
       at3_1(0:KCMAX), at3_2(0:KCMAX), at4_1(0:KCMAX), &
       at4_2(0:KCMAX), at5(0:KCMAX), at6(0:KCMAX), &
       at7, &
       aw1, aw2, aw3, aw4, aw5, aw8, aw9, aw7, &
       ai1(0:KCMAX), ai2(0:KCMAX), ai3, ai4, &
       atr1, &
       am1, am2, &
       atm1(0:KCMAX), atm2(0:KTMAX), &
       acb1, acb2, acb3, acb4, &
       alb1, &
       aqbm1, aqbm2, aqbm3, aqbm4, aqtld
character (len=100) :: runname
logical :: zs_switch

!-------- Begin of main loop --------

write(6,*) ' '

itercount_max = nint((time_end-time_init)/dtime)

iter_temp = nint(dtime_temp/dtime)
#if REBOUND==2
iter_wss = nint(dtime_wss/dtime)
#endif
iter_ser  = nint(dtime_ser/dtime)
#if OUTPUT==1
iter_out  = nint(dtime_out/dtime)
#elif OUTPUT==2
do n=1, n_output
   iter_output(n) = nint((time_output(n)-time_init)/dtime)
end do
#endif

main_loop : do itercount=1, itercount_max

write(6,'(i7)') itercount

!-------- Update of time --------

time      = time_init + dble(itercount)*dtime

!-------- Save old mask --------

do i=0, IMAX
do j=0, JMAX
   maske_help(j,i) = maske(j,i)
end do
end do

!-------- Boundary conditions --------

call boundary(time, dxi, deta, delta_ts, glac_index, griptemp, &
              specmap_zsl, z_sl, z_mar, &
              grip_time_min, grip_time_stp, grip_time_max, &
              specmap_time_min, specmap_time_stp, specmap_time_max, &
              z_sle_forcing, time_forcing, ndata_forcing)

!-------- Temperature, water content, age, flow enhancement factor --------

if ( mod(itercount, iter_temp).eq.0 ) then

   write(6,*) ' -------- Computation of T'

!  ------ Temperature, water content, age

#if CALCMOD==1
   call calc_temp_poly(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 acb1, acb2, acb3, acb4, &
                 ai1, ai2, ai3, ai4, mean_accum_inv, &
                 dxi, deta, dzeta_t, &
                 dtime_temp, dtime_temp_inv, dtt_2dxi, dtt_2deta)
!        ! polythermal mode --------

#elif CALCMOD==0
   call calc_temp_cold(at1, at2_1, at2_2, at3_1, at3_2, &
                 at4_1, at4_2, at5, at6, at7, atr1, &
                 am1, am2, alb1, &
                 aw1, aw2, aw3, aw4, aw5, aw7, aw8, aw9, aqtld, &
                 acb1, acb2, acb3, acb4, &
                 ai1, ai2, ai3, ai4, mean_accum_inv, &
                 dxi, deta, dzeta_t, &
                 dtime_temp, dtt_2dxi, dtt_2deta)
!        ! cold-ice mode --------

#endif

!  ------ Time derivative of H_t (further time derivatives are
!         computed in subroutine calc_top)

   do i=0, IMAX
   do j=0, JMAX
      dH_t_dtau(j,i) = (H_t_neu(j,i)-H_t(j,i))*dtime_temp_inv
   end do
   end do

!  ------ New values --> old values

   do i=0, IMAX
   do j=0, JMAX

      n_cts(j,i) = n_cts_neu(j,i)
      zm(j,i)  = zm_neu(j,i)
      H_c(j,i) = H_c_neu(j,i)
      H_t(j,i) = H_t_neu(j,i)

      do kc=0, KCMAX
         temp_c(kc,j,i) = temp_c_neu(kc,j,i)
         age_c(kc,j,i)  = age_c_neu(kc,j,i)
      end do

      do kt=0, KTMAX
         omega_t(kt,j,i) = omega_t_neu(kt,j,i)
         age_t(kt,j,i)   = age_t_neu(kt,j,i)
      end do

      do kr=0, KRMAX
         temp_r(kr,j,i)  = temp_r_neu(kr,j,i)
      end do

  end do
  end do

!  ------ Flow enhancement factor

   call calc_enhance(time)

end if
!     End of computation of temperature, water content, age and
!     enhancement factor

!-------- Velocity --------

call calc_vxy_b(z_sl)
call calc_vxy(avxy3, aqxy1, dzeta_t)

call calc_vz(dxi, deta, dzeta_c, dzeta_t, avz3)

!-------- Topography --------

call calc_top(time, time_init, z_sl, z_mar, dtime, dxi, deta, &
     dzeta_t, mean_accum, azs2, azs3, &
     ii, jj, nn, itercount, iter_wss, zs_switch)

!  ------ New values --> old values

do i=0, IMAX
do j=0, JMAX
   zs(j,i)  = zs_neu(j,i)
   zm(j,i)  = zm_neu(j,i)
   zb(j,i)  = zb_neu(j,i)
   H_c(j,i) = H_c_neu(j,i)
   H_t(j,i) = H_t_neu(j,i)
end do
end do

!-------- Melting temperature --------

call calc_temp_melt(atm1, atm2)

!-------- Basal melting rate --------

call calc_qbm(aqbm1, aqbm2, aqbm3, aqbm4)

!-------- Data output --------

#if OUTPUT == 1
if ( mod(itercount, iter_out).eq.0 ) then
#if NETCDF == 1
   call output1(runname, time, delta_ts, glac_index, z_sl, ndat)
#endif
#if NETCDF == 2
   call output_nc(runname, time, delta_ts, glac_index, z_sl, ndat)
#endif
#if NETCDF == 3
   call output_nc(runname, time, delta_ts, glac_index, z_sl, ndat)
   ndat = ndat-1
   call output1(runname, time, delta_ts, glac_index, z_sl, ndat)
#endif
end if
#endif /* OUTPUT == 1 */

#if OUTPUT == 2
do n=1, n_output
   if (itercount.eq.iter_output(n)) then
#if NETCDF == 1
      call output1(runname, time, delta_ts, glac_index, z_sl, ndat)
#endif
#if NETCDF == 2
      call output_nc(runname, time, delta_ts, glac_index, z_sl, ndat)
#endif
#if NETCDF == 3
      call output_nc(runname, time, delta_ts, glac_index, z_sl, ndat)
      ndat = ndat-1
      call output1(runname, time, delta_ts, glac_index, z_sl, ndat)
#endif
   end if
end do
#endif /* OUTPUT == 2 */

if ( mod(itercount, iter_ser).eq.0 ) then
   call output2(time, dxi, deta, delta_ts, glac_index, z_sl)
#if OUTSER==2
   call output3(time, delta_ts, glac_index, z_sl)
#endif
#if OUTSER==3
   call output4(time, dxi, deta, delta_ts, glac_index, z_sl)
#endif
end if

end do main_loop   ! End of main loop

end subroutine sico_main_loop
