!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Module     :  s i c o _ v a r i a b l e s

!   Purpose    :  Declarations of global variables for SICOPOLIS.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

module sico_variables

!-------- Field quantities --------

integer, dimension(0:JMAX,0:IMAX) :: &
       maske, maske_help, maske_neu, n_cts, n_cts_neu, kc_cts

double precision, dimension(0:IMAX) :: xi
double precision, dimension(0:JMAX) :: eta
double precision, dimension(0:KCMAX) :: zeta_c, eaz_c, eaz_c_quotient
double precision, dimension(0:KTMAX) :: zeta_t
double precision :: ea

double precision, dimension(0:JMAX,0:IMAX) :: &
       phi, lambda, area, &
       sq_g11_g, sq_g22_g, insq_g11_g, insq_g22_g, &
       sq_g11_sgx, sq_g11_sgy, sq_g22_sgx, sq_g22_sgy, &
       insq_g11_sgx, insq_g22_sgy, &
       zs, zm, zb, zb0, wss, H_c, H_t, &
       dzs_dxi, dzm_dxi, dzb_dxi, dH_c_dxi, dH_t_dxi, &
       dzs_deta, dzm_deta, dzb_deta, dH_c_deta, dH_t_deta, &
       dzs_dxi_g, dzm_dxi_g, dzb_dxi_g, dH_c_dxi_g, dH_t_dxi_g, &
       dzs_deta_g, dzm_deta_g, dzb_deta_g, dH_c_deta_g, dH_t_deta_g, &
       dzs_dtau, dzm_dtau, dzb_dtau, dH_c_dtau, dH_t_dtau, &
       d_help_b, vx_b, vy_b, vz_b, &
       h_diff, qx, qy, &
       q_geo, &
       Q_bm, Q_tld, Q_b_tot, &
       accum, evap, runoff, as_perp, temp_s, &
       am_perp, am_perp_st, &
       zs_neu, zm_neu, zb_neu, H_c_neu, H_t_neu, &
       zs_ref

double precision, dimension(0:JMAX,0:IMAX) :: &
       accum_present, &
       precip_ma_present, precip_ma_lgm_anom, &
       temp_ma_present, temp_mj_present, temp_ma_lgm_anom, temp_mj_lgm_anom

double precision, dimension(-JMAX:JMAX,-IMAX:IMAX) :: dist_dxdy

double precision, dimension(0:JMAX,0:IMAX,12) :: &
                  precip_present, precip_lgm_anom, gamma_precip_lgm_anom, &
                  temp_mm_present, temp_mm_lgm_anom

double precision, dimension(0:KCMAX,0:JMAX,0:IMAX) :: &
       d_help_c, vx_c, vy_c, vz_c, &
       temp_c, temp_c_neu, temp_c_m, age_c, age_c_neu, &
       txz_c, tyz_c, sigma_c, &
       enh_c

double precision, dimension(0:KTMAX,0:JMAX,0:IMAX) :: &
       d_help_t, vx_t, vy_t, vz_t, &
       omega_t, omega_t_neu, temp_t_m, age_t, age_t_neu, &
       txz_t, tyz_t, sigma_t, &
       enh_t

double precision, dimension(0:KRMAX,0:JMAX,0:IMAX) :: &
       temp_r, temp_r_neu

!-------- Physical parameters --------

double precision :: RHO, RHO_W, RHO_SW, &
       L, G, NUE, BETA, OMEGA_MAX, &
       H_R, RHO_C_R, KAPPA_R, FLEX_RIG_L, RHO_A, TIME_LAG, &
       R_T, SIGMA_RES, R, PHI0, LAMBDA0, &
       S_STAT_0, BETA1_0, BETA1_LT_0, BETA1_HT_0, &
       BETA2_0, BETA2_LT_0, BETA2_HT_0, PMAX_0, MU_0

#if ( defined(NMARS) \
      || defined(SMARS) )   /* Martian ice sheet */
double precision :: RHO_I, RHO_C, KAPPA_C, C_C
double precision :: rho_inv
#endif

double precision, dimension(-190:10) :: RF, KAPPA, C

!-------- Further quantities -------- 

#if ( defined(NMARS) \
      || defined(SMARS) )   /* Martian ice sheet */
integer :: insol_time_min, insol_time_stp, insol_time_max
double precision, dimension(0:100000) :: insol_ma_90, obl_data, ecc_data, &
                                         ave_data, cp_data
integer, dimension(0:JMAX,0:IMAX) :: maske_chasm
double precision :: time_chasm_init, time_chasm_end
double precision, dimension(0:JMAX,0:IMAX) :: q_geo_normal
#endif

#if defined(EMTP2SGE)   /* EISMINT Phase 2 Simplified Geometry Experiment */
double precision :: temp_min, s_t, x_hat, y_hat, b_max, s_b, eld
#elif defined(HEINO)    /* ISMIP HEINO */
integer, dimension(0:JMAX,0:IMAX) :: maske_sedi
double precision :: temp_min, s_t, x_hat, y_hat, rad, b_min, b_max
#elif ( defined(GRL) && ICE_STREAM==2 )   /* Greenland with ice streams */
integer, dimension(0:JMAX,0:IMAX) :: maske_sedi
#endif

integer :: forcing_flag

integer :: n_core
double precision, dimension(:), allocatable, save :: &
       lambda_core, phi_core, x_core, y_core

integer :: gi_time_min, gi_time_stp, gi_time_max, ndata_gi
double precision, dimension(:), allocatable, save :: glacial_index

double precision, dimension(-10000:10000) :: kei
integer:: n_data_kei
double precision :: kei_r_max, kei_r_incr

double precision, parameter :: &
       pi = 3.141592653589793d0, pi_inv = 1.0d0/pi, &
       pi_180 = pi/180.0d0, pi_180_inv = 180.0d0/pi, &
       eps = 1.0d-05

end module sico_variables
