!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  t r i _ s l e

!   Purpose    :  Solution of a system of linear equations Ax=b with
!                 tridiagonal matrix A.

!   Parameters :  Matrix a
!                 (a0(j): element A_(j,j-1), a1(j): element A_(j,j),
!                  a2(j): element A_(j,j+1)),
!                 solution vector x,
!                 inhomogeneity vector b,
!                 size of matrix nzeilen
!                 (indices run from 0 (!!!) to nzeilen).

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine tri_sle(a0, a1, a2, x, b, nzeilen)

implicit none
double precision, dimension(0:*) :: a0, a1, a2, x, b
double precision, allocatable, dimension(:) :: help_x
integer :: nzeilen
integer :: n

!--------  Generate an upper triangular matrix
!                      ('obere Dreiecksmatrix') --------

do n=1, nzeilen
   a1(n)   = a1(n) - a0(n)/a1(n-1)*a2(n-1)
end do

do n=1, nzeilen
   b(n)    = b(n) - a0(n)/a1(n-1)*b(n-1)
!         a0(n)  = 0.0d0 , not needed in the following, therefore
!                          not set
end do

!-------- Iterative solution of the new system --------

!      x(nzeilen) = b(nzeilen)/a1(nzeilen)

!      do n=nzeilen-1, 0, -1
!         x(n) = (b(n)-a2(n)*x(n+1))/a1(n)
!      end do

allocate(help_x(0:nzeilen))

help_x(0) = b(nzeilen)/a1(nzeilen)

do n=1, nzeilen
   help_x(n) = b(nzeilen-n)/a1(nzeilen-n) &
              -a2(nzeilen-n)/a1(nzeilen-n)*help_x(n-1)
end do

do n=0, nzeilen
   x(n) = help_x(nzeilen-n)
end do

!       (The trick with the help_x was introduced in order to avoid
!        the negative step in the original, blanked-out loop.)

deallocate(help_x)

!  WARNING: Subroutine does not check for elements of the main
!           diagonal becoming zero. In this case it crashes even
!           though the system may be solvable. Otherwise ok.

return
end
