!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  t o p o g r a p h y 3

!   Purpose    :  Definition of the initial surface and bedrock
!                 topography (including gradients) and of the
!                 horizontal grid spacings dxi, deta.
!                 For initial topography from previous simulation.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine topography3(dxi, deta, anfdatname)

use sico_variables

implicit none
integer i, j, kc, kt, kr, n
integer ios
real time_conv, delta_ts_conv, z_sl_conv, &
     H_R_conv, xi_conv(0:IMAX), eta_conv(0:JMAX), &
     zs_conv(0:JMAX,0:IMAX), zm_conv(0:JMAX,0:IMAX), &
     zb_conv(0:JMAX,0:IMAX), &
     H_c_conv(0:JMAX,0:IMAX), H_t_conv(0:JMAX,0:IMAX), &
     Q_bm_conv(0:JMAX,0:IMAX), Q_tld_conv(0:JMAX,0:IMAX), &
     am_perp_conv(0:JMAX,0:IMAX), &
     qx_conv(0:JMAX,0:IMAX), qy_conv(0:JMAX,0:IMAX), &
     temp_r_conv(0:KRMAX,0:JMAX,0:IMAX), &
     vx_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vy_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vz_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     vx_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     vy_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     vz_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     temp_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     age_c_conv(0:KCMAX,0:JMAX,0:IMAX), &
     omega_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     age_t_conv(0:KTMAX,0:JMAX,0:IMAX), &
     dzs_dtau_conv(0:JMAX,0:IMAX), dzm_dtau_conv(0:JMAX,0:IMAX), &
     dzb_dtau_conv(0:JMAX,0:IMAX), &
     dH_c_dtau_conv(0:JMAX,0:IMAX), dH_t_dtau_conv(0:JMAX,0:IMAX)
double precision dxi, deta
character (len=100) :: anfdatname
character ch_dummy

!-------- Read data from time-slice file of previous simulation --------

open(11, iostat=ios, &
         file=ANFDATPATH//'/'//trim(anfdatname), &
         status='old', form='unformatted')
if (ios.ne.0) stop ' Error when opening the initial-value file!'

read(11) time_conv
read(11) delta_ts_conv
read(11) z_sl_conv
read(11) xi_conv
read(11) eta_conv
read(11) maske
read(11) n_cts
read(11) zs_conv
read(11) zm_conv
read(11) zb_conv
read(11) H_c_conv
read(11) H_t_conv
read(11) H_R_conv
read(11) vx_c_conv
read(11) vy_c_conv
read(11) vz_c_conv
read(11) vx_t_conv
read(11) vy_t_conv
read(11) vz_t_conv
read(11) temp_c_conv
read(11) omega_t_conv
read(11) temp_r_conv
read(11) Q_bm_conv
read(11) Q_tld_conv
read(11) am_perp_conv
read(11) qx_conv
read(11) qy_conv
read(11) age_c_conv
read(11) age_t_conv
read(11, end=20) dzs_dtau_conv
read(11, end=20) dzm_dtau_conv
read(11, end=20) dzb_dtau_conv
read(11, end=20) dH_c_dtau_conv
read(11, end=20) dH_t_dtau_conv

go to 30

20 continue
   dzs_dtau_conv  = 0.0
   dzm_dtau_conv  = 0.0
   dzb_dtau_conv  = 0.0
   dH_c_dtau_conv = 0.0
   dH_t_dtau_conv = 0.0
   write(6,*) 'Warning: d***_dtau_conv cannot be read from initial-value file.'
   write(6,*) '         Therefore all values set to zero.'

30 continue

close(11,status='keep')

!-------- Convert data to double precision --------

H_R  = dble(H_R_conv)

do i=0, IMAX
do j=0, JMAX

   xi(i)        = dble(xi_conv(i))
   eta(j)       = dble(eta_conv(j))
   zs(j,i)      = dble(zs_conv(j,i))
   zm(j,i)      = dble(zm_conv(j,i))
   zb(j,i)      = dble(zb_conv(j,i))
   H_c(j,i)     = dble(H_c_conv(j,i))
   H_t(j,i)     = dble(H_t_conv(j,i))
   Q_bm(j,i)    = dble(Q_bm_conv(j,i))
   Q_tld(j,i)   = dble(Q_tld_conv(j,i))
   am_perp(j,i) = dble(am_perp_conv(j,i))
   qx(j,i)      = dble(qx_conv(j,i))
   qy(j,i)      = dble(qy_conv(j,i))
   dzs_dtau(j,i)  = dble(dzs_dtau_conv(j,i))
   dzm_dtau(j,i)  = dble(dzm_dtau_conv(j,i))
   dzb_dtau(j,i)  = dble(dzb_dtau_conv(j,i))
   dH_c_dtau(j,i) = dble(dH_c_dtau_conv(j,i))
   dH_t_dtau(j,i) = dble(dH_t_dtau_conv(j,i))

   do kr=0, KRMAX
      temp_r(kr,j,i) = dble(temp_r_conv(kr,j,i))
   end do

   do kt=0, KTMAX
      vx_t(kt,j,i)    = dble(vx_t_conv(kt,j,i))
      vy_t(kt,j,i)    = dble(vy_t_conv(kt,j,i))
      vz_t(kt,j,i)    = dble(vz_t_conv(kt,j,i))
      omega_t(kt,j,i) = dble(omega_t_conv(kt,j,i))
      age_t(kt,j,i)   = dble(age_t_conv(kt,j,i))
   end do

   do kc=0, KCMAX
      vx_c(kc,j,i)    = dble(vx_c_conv(kc,j,i))
      vy_c(kc,j,i)    = dble(vy_c_conv(kc,j,i))
      vz_c(kc,j,i)    = dble(vz_c_conv(kc,j,i))
      temp_c(kc,j,i)  = dble(temp_c_conv(kc,j,i))
      age_c(kc,j,i)   = dble(age_c_conv(kc,j,i))
   end do

end do
end do

!-------- Read topography of the relaxed bedrock --------

open(23, iostat=ios, &
     file=INPATH//'/scand/'//ZB0_FILE, &
     recl=8192, status='old')

if (ios.ne.0) stop ' Error when opening the zb0 file!'

do n=1, 6; read(23,'(a)') ch_dummy; end do

do j=JMAX, 0, -1
   read(23,*) (zb0(j,i), i=0,IMAX)
end do

close(23, status='keep')

!-------- Further stuff --------

dxi  = DX *1000.0d0   ! km -> m
deta = DX *1000.0d0   ! km -> m

do i=0, IMAX
do j=0, JMAX
   call geo_coord(phi(j,i), lambda(j,i), xi(i), eta(j))
end do
end do

!-------- Metric tensor, gradients of the topography --------

call metric

#if TOPOGRAD==0
call topograd_1(dxi, deta, 1)
#elif TOPOGRAD==1
call topograd_2(dxi, deta, 1)
#endif

!-------- Corresponding area of grid points --------

do i=0, IMAX
do j=0, JMAX
   area(j,i) = sq_g11_g(j,i)*sq_g22_g(j,i)*dxi*deta
end do
end do

return
end
